@@ -27,7 +27,7 @@
 #include <stdlib.h>
 #include <string.h>
 
-/*#define LOCAL_DEBUG*/
+#undef LOCAL_DEBUG
 
 #include "astypes.h"
 #include "ashash.h"
@@ -42,10 +42,13 @@ static unsigned int deallocated_used = 0 ;
 static inline void 
 free_ashash_item( ASHashItem *item ) 
 {
+
+#ifndef DEBUG_ALLOCS
 	if( deallocated_used < DEALLOC_CACHE_SIZE )
     {
     	deallocated_mem[deallocated_used++] = item ;
     }else
+#endif
         free( item );
 }
 
@@ -71,7 +74,7 @@ desc_long_compare_func (ASHashableValue value1, ASHashableValue value2)
 void
 init_ashash (ASHashTable * hash, Bool freeresources)
 {
-LOCAL_DEBUG_CALLER_OUT( " has = %p, free ? %d", hash, freeresources );
+	LOCAL_DEBUG_CALLER_OUT( " hash = %p, free ? %d", hash, freeresources );
 	if (hash)
 	{
 		if (freeresources)
@@ -89,10 +92,13 @@ create_ashash (ASHashKey size,
 {
 	ASHashTable  *hash;
 
+	LOCAL_DEBUG_CALLER_OUT( " size = %d", size );
+
 	if (size <= 0)
 		size = DEFAULT_HASH_SIZE;
 
 	hash = safemalloc (sizeof (ASHashTable));
+
 	init_ashash (hash, False);
 
 	hash->buckets = safecalloc (size, sizeof (ASHashBucket));
@@ -131,7 +137,7 @@ destroy_ashash_bucket (ASHashBucket * bucket, void (*item_destroy_func) (ASHasha
 void
 flush_ashash (ASHashTable * hash)
 {
-LOCAL_DEBUG_CALLER_OUT( " hash = %p, *hash = %p", hash, *hash  );
+LOCAL_DEBUG_CALLER_OUT( " hash = %p", hash  );
 	if (hash)
 	{
 		register int  i = hash->size;
@@ -194,7 +200,6 @@ check_hash_item_reused (ASHashItem *item)
 	return (deallocated_mem[deallocated_used] == item );
 }
 
-
 ASHashResult
 add_hash_item (ASHashTable * hash, ASHashableValue value, void *data)
 {
@@ -204,16 +209,19 @@ add_hash_item (ASHashTable * hash, ASHashableValue value, void *data)
 
 	if (hash == NULL)
         return ASH_BadParameter;
-
 	key = hash->hash_func (value, hash->size);
 	if (key >= hash->size)
         return ASH_BadParameter;
 
+#ifndef DEBUG_ALLOCS
     if( deallocated_used > 0 )
         item = deallocated_mem[--deallocated_used];
     else
+#endif	
         item = safecalloc (1, sizeof (ASHashItem));
 
+	LOCAL_DEBUG_OUT( "hash %p, value 0x%lX, data = %p, item = %p ", hash, value, data, item );
+
 	item->next = NULL;
 	item->value = value;
 	item->data = data;
@@ -231,7 +239,7 @@ add_hash_item (ASHashTable * hash, ASHashableValue value, void *data)
 }
 
 #ifdef DEBUG_ALLOCS
-#define safecalloc(a) countcalloc(__FUNCTION__, __LINE__, a,b)
+#define safecalloc(a,b) countcalloc(__FUNCTION__, __LINE__, a,b)
 #define add_hash_item(a,b,c) countadd_hash_item(__FUNCTION__, __LINE__,a,b,c)
 #define free(a) countfree(__FUNCTION__, __LINE__, a)
 #endif
@@ -255,6 +263,26 @@ print_ashash (ASHashTable * hash, void (*item_print_func) (ASHashableValue value
 	}
 }
 
+void
+print_ashash2 (ASHashTable * hash, void (*item_print_func) (ASHashableValue value, void *data))
+{
+	register int  i;
+	ASHashItem   *item;
+
+	for (i = 0; i < hash->size; i++)
+	{
+		if (hash->buckets[i] == NULL)
+			continue;
+		fprintf (stderr, "Bucket # %d:", i);
+		for (item = hash->buckets[i]; item != NULL; item = item->next)
+			if (item_print_func)
+				item_print_func (item->value, item->data);
+			else
+				fprintf (stderr, "[0x%lX(%ld):%p]", item->value, item->value, item->data);
+		fprintf (stderr, "\n");
+	}
+}
+
 
 static ASHashItem **
 find_item_in_bucket (ASHashBucket * bucket,
@@ -360,7 +388,7 @@ sort_hash_items (ASHashTable * hash, ASHashableValue * values, void **data, unsi
 
 		if (max_items == 0)
 			max_items = hash->items_num;
-
+		
 		buckets = safecalloc (hash->buckets_used, sizeof (ASHashBucket));
 		for (i = 0; i < hash->size; i++)
 			if (hash->buckets[i])
