@@ -28,6 +28,7 @@
 #include "../libAfterStep/asdatabase.h"
 #include "../libAfterStep/parser.h"
 #include "../libAfterStep/hints.h"
+#include "../libAfterStep/session.h"
 
 #include "afterconf.h"
 
@@ -121,6 +122,8 @@ TermDef       StyleTerms[] = {
 	{TF_NO_MYNAME_PREPENDING, "NoMotifHints", 12, TT_FLAG, DATABASE_NoMotifHints_ID, NULL},
 	{TF_NO_MYNAME_PREPENDING, "HonorGnomeHints", 15, TT_FLAG, DATABASE_HonorGnomeHints_ID, NULL},
 	{TF_NO_MYNAME_PREPENDING, "NoGnomeHints", 12, TT_FLAG, DATABASE_NoGnomeHints_ID, NULL},
+	{TF_NO_MYNAME_PREPENDING, "HonorKDEHints", 13, TT_FLAG, DATABASE_HonorKDEHints_ID, NULL},
+	{TF_NO_MYNAME_PREPENDING, "NoKDEHints", 10, TT_FLAG, DATABASE_NoKDEHints_ID, NULL},
 	{TF_NO_MYNAME_PREPENDING, "HonorExtWMHints", 15, TT_FLAG, DATABASE_HonorExtWMHints_ID, NULL},
 	{TF_NO_MYNAME_PREPENDING, "NoExtWMHints", 12, TT_FLAG, DATABASE_NoExtWMHints_ID, NULL},
 	{TF_NO_MYNAME_PREPENDING, "HonorXResources", 15, TT_FLAG, DATABASE_HonorXResources_ID, NULL},
@@ -131,7 +134,11 @@ TermDef       StyleTerms[] = {
 	{TF_NO_MYNAME_PREPENDING, "ShortLiving", 12, TT_FLAG, DATABASE_ShortLiving_ID, NULL},
 	{TF_NO_MYNAME_PREPENDING, "IgnoreConfig", 13, TT_FLAG, DATABASE_IgnoreConfig_ID, NULL},
 	{TF_NO_MYNAME_PREPENDING, "HonorConfig", 12, TT_FLAG, DATABASE_HonorConfig_ID, NULL},
-	
+	{TF_NO_MYNAME_PREPENDING, "UseCurrentViewport", 18, TT_FLAG, DATABASE_UseCurrentViewport_ID, NULL},
+	{TF_NO_MYNAME_PREPENDING, "UseAnyViewport", 14, TT_FLAG, DATABASE_UseAnyViewport_ID, NULL},
+	ASCF_DEFINE_KEYWORD(DATABASE,TF_NO_MYNAME_PREPENDING,Fullscreen,TT_FLAG,NULL),
+	ASCF_DEFINE_KEYWORD(DATABASE,TF_NO_MYNAME_PREPENDING,NoFullscreen,TT_FLAG,NULL),
+	ASCF_DEFINE_KEYWORD(DATABASE,TF_NO_MYNAME_PREPENDING,WindowOpacity,TT_INTEGER,NULL),
 	{0, NULL, 0, 0, 0}
 };
 
@@ -151,6 +158,7 @@ SyntaxDef     StyleSyntax = {
 };
 
 flag_options_xref StyleFlags[] = {
+	{STYLE_FULLSCREEN,			DATABASE_Fullscreen_ID, 		DATABASE_NoFullscreen_ID},
 	{STYLE_STICKY, 				DATABASE_Sticky_ID, 			DATABASE_Slippery_ID},
 	{STYLE_TITLE, 				DATABASE_Title_ID, 				DATABASE_NoTitle_ID},
 	{STYLE_CIRCULATE, 			DATABASE_CirculateHit_ID, 		DATABASE_CirculateSkip_ID},
@@ -166,11 +174,13 @@ flag_options_xref StyleFlags[] = {
 	{STYLE_TRANSIENT_HINTS, 	DATABASE_HonorTransientHints_ID, DATABASE_NoTransientHints_ID},
 	{STYLE_MOTIF_HINTS, 		DATABASE_HonorMotifHints_ID, 	DATABASE_NoMotifHints_ID},
 	{STYLE_GNOME_HINTS, 		DATABASE_HonorGnomeHints_ID, 	DATABASE_NoGnomeHints_ID},
+	{STYLE_KDE_HINTS, 			DATABASE_HonorKDEHints_ID, 		DATABASE_NoKDEHints_ID},
 	{STYLE_EXTWM_HINTS, 		DATABASE_HonorExtWMHints_ID, 	DATABASE_NoExtWMHints_ID},
 	{STYLE_XRESOURCES_HINTS, 	DATABASE_HonorXResources_ID, 	DATABASE_NoXResources_ID},
 	{STYLE_FOCUS_ON_MAP, 		DATABASE_FocusOnMap_ID, 		DATABASE_NoFocusOnMap_ID},
 	{STYLE_LONG_LIVING,			DATABASE_LongLiving_ID,			DATABASE_ShortLiving_ID},
 	{STYLE_IGNORE_CONFIG, 	    DATABASE_IgnoreConfig_ID, 	    DATABASE_HonorConfig_ID},
+	{STYLE_CURRENT_VIEWPORT,    DATABASE_UseCurrentViewport_ID, 	DATABASE_UseAnyViewport_ID},
 	{STYLE_ICON, 				0 /* special case */ , 			DATABASE_NoIcon_ID},
 	{STYLE_STARTUP_DESK, 		0 /*special case */ , 			DATABASE_StartsAnywhere_ID},
 	{STYLE_FRAME, 				0 /*special case */ , 			DATABASE_NoFrame_ID},
@@ -181,11 +191,13 @@ flag_options_xref StyleDataFlags[] = {
 	{STYLE_ICON, 				0 /* special case */ , 			DATABASE_NoIcon_ID},
 	{STYLE_STARTUP_DESK, 		0 /*special case */ , 			DATABASE_StartsAnywhere_ID},
 	{STYLE_FRAME, 				0 /*special case */ , 			DATABASE_NoFrame_ID},
+	{STYLE_VIEWPORTY,			0 /*special case */ , 			DATABASE_UseAnyViewport_ID},
+	{STYLE_VIEWPORTX,			0 /*special case */ , 			DATABASE_UseAnyViewport_ID},
 	{0, 0, 0}
 };
 
 TermDef       DatabaseTerms[] = {
-	{TF_NAMED_SUBCONFIG | TF_NO_MYNAME_PREPENDING, "Style", 5, TT_QUOTED_TEXT, DATABASE_STYLE_ID, &StyleSyntax}
+	{TF_NAMED | TF_NO_MYNAME_PREPENDING, "Style", 5, TT_QUOTED_TEXT, DATABASE_STYLE_ID, &StyleSyntax}
 	,
 	{0, NULL, 0, 0, 0}
 };
@@ -228,6 +240,7 @@ style_init (name_list * nl)
 	nl->ViewportX = -1;
 	nl->ViewportY = -1;
 	nl->gravity = NorthWestGravity;
+	nl->window_opacity = 100 ;
 
 	nl->off_buttons = 0;
 	nl->on_buttons = 0;
@@ -289,6 +302,8 @@ style_copy (name_list * to, name_list * from)
 			to->layer = from->layer;
 		if (get_flags (from->set_data_flags, STYLE_GRAVITY))
 			to->gravity = from->gravity;
+		if (get_flags (from->set_data_flags, STYLE_WINDOW_OPACITY))
+			to->window_opacity = from->window_opacity;
 
 		to->set_flags |= from->set_flags;
 		to->flags |= (from->set_flags & from->flags);
@@ -399,11 +414,16 @@ ParseSingleStyle (FreeStorageElem * storage, name_list * style)
 		return;
 	for (; storage; storage = storage->next)
 	{
+		Bool is_flag = False ; 
 		if (storage->term == NULL)
 			continue;
+		
 		if (ReadFlagItem (&(style->set_flags), &(style->flags), storage, StyleFlags))
-			continue;
+			is_flag = True;
 		if (ReadFlagItem (&(style->set_data_flags), NULL, storage, StyleDataFlags))
+			is_flag = True;
+		
+		if( is_flag )
 			continue;
 		if (!ReadConfigItem (&item, storage))
 			continue;
@@ -483,6 +503,12 @@ ParseSingleStyle (FreeStorageElem * storage, name_list * style)
 		 case DATABASE_ViewportY_ID:
 			 set_flags (style->set_data_flags, STYLE_VIEWPORTY);
 			 style->ViewportY = item.data.integer;
+			 LOCAL_DEBUG_OUT( "style->ViewportY = %d", style->ViewportY );
+			 break;
+		 case DATABASE_WindowOpacity_ID:
+			 set_flags (style->set_data_flags, STYLE_WINDOW_OPACITY);
+			 style->window_opacity = item.data.integer;
+			 LOCAL_DEBUG_OUT( "style->window_opacity = %d", style->window_opacity );
 			 break;
 		 case DATABASE_Frame_ID:
 			 set_string_value (&(style->frame_name), item.data.string, &(style->set_data_flags), STYLE_FRAME);
@@ -526,7 +552,6 @@ ParseDatabaseOptions (const char *filename, char *myname)
 	item.memory = NULL;
 	PrintConfigReader (ConfigReader);
 	ParseConfig (ConfigReader, &Storage);
-	PrintFreeStorage (Storage);
 
 	/* getting rid of all the crap first */
 	StorageCleanUp (&Storage, &more_stuff, CF_DISABLED_OPTION);
@@ -553,6 +578,64 @@ ParseDatabaseOptions (const char *filename, char *myname)
 	return config;
 }
 
+name_list    *
+string2DatabaseStyle (char *style_txt)
+{
+	ConfigData    cd ;
+	ConfigDef    *ConfigReader;
+	name_list    *style = NULL;
+	FreeStorageElem *Storage = NULL;
+
+	cd.data = style_txt ;
+	ConfigReader = InitConfigReader ("afterstep", &StyleSyntax, CDT_Data, cd, NULL);
+	if (!ConfigReader)
+		return NULL;
+	
+	PrintConfigReader (ConfigReader);
+	
+	ParseConfig (ConfigReader, &Storage);
+
+	if ((style = style_new (NULL)) != NULL )
+		ParseSingleStyle (Storage, style);
+
+	DestroyFreeStorage (&Storage);
+	DestroyConfig (ConfigReader);
+	return style;
+}
+
+
+Bool ReloadASDatabase()
+{
+	char *configfile  = make_session_file(Session, DATABASE_FILE, False ); 
+	destroy_asdatabase();
+    if( configfile != NULL )
+    {
+	    struct name_list *list = NULL ;
+	
+		/* memory management for parsing buffer */
+	    list = ParseDatabaseOptions (configfile, "afterstep");
+    	if( list )
+	    {
+    	    Database = build_asdb( list );
+        	if( is_output_level_under_threshold( OUTPUT_LEVEL_DATABASE ) )
+	            print_asdb( NULL, NULL, Database );
+    	    while (list != NULL)
+        	    delete_name_list (&(list));
+	        show_progress("DATABASE configuration loaded from \"%s\" ...", configfile);
+	    }else
+    	    show_progress( "no database records loaded." );
+	    /* XResources : */
+    	load_user_database();
+        free( configfile );
+    }else
+    {
+        show_warning("DATABASE configuration file cannot be found!");
+        return False ;
+    }
+	return True ;
+}
+
+
 /*****************************************************************************/
 /*                      Writer stuff                                         */
 /*****************************************************************************/
