@@ -180,6 +180,14 @@ ProcessWindowBoxOptions (FreeStorageElem * options)
                     aswbox->max_layer = item.data.integer ;
                     set_flags( aswbox->set_flags, ASA_MaxLayerSet);
                     break ;
+                case WINDOWBOX_XSpacing_ID        :
+                    aswbox->x_spacing = item.data.integer ;
+                    set_flags( aswbox->set_flags, ASA_XSpacingSet);
+                    break ;
+                case WINDOWBOX_YSpacing_ID        :
+                    aswbox->y_spacing = item.data.integer ;
+                    set_flags( aswbox->set_flags, ASA_YSpacingSet);
+                    break ;
                 default:
 					item.ok_to_free = 1;
                     show_warning( "Unexpected WindowBox definition keyword \"%s\" . Ignoring.", options->term->keyword );
@@ -198,7 +206,7 @@ windowbox_parse (char *tline, FILE * fd, char **unused1, int *unused2)
     ConfigDef    *ConfigReader ;
     FreeStorageElem *Storage = NULL, *more_stuff = NULL;
     ASWindowBox **aswbox_list = &(ASDefaultScr->Feel.window_boxes);
-	int *count = &(ASDefaultScr->Feel.window_boxes_num);
+	unsigned int *count = &(ASDefaultScr->Feel.window_boxes_num);
     ASWindowBox *new_box ;
 	ConfigData cd ;
 
@@ -215,7 +223,6 @@ windowbox_parse (char *tline, FILE * fd, char **unused1, int *unused2)
 
 	PrintConfigReader (ConfigReader);
 	ParseConfig (ConfigReader, &Storage);
-	PrintFreeStorage (Storage);
 
 	/* getting rid of all the crap first */
     StorageCleanUp (&Storage, &more_stuff, CF_DISABLED_OPTION);
@@ -272,13 +279,13 @@ DestroyFeelConfig (FeelConfig * config)
  *  Feel Parsing code :
  **********************************************************************/
 unsigned long
-BindingSpecialFunc (ConfigDef * config, FreeStorageElem ** storage)
+BindingSpecialFunc (ConfigDef * config)
 {
     /* since we have have subconfig of Functions that has \n as line terminator
      * we are going to get entire line again at config->cursor
      * so lets skip 3 tokens of <key/mouse button> <context> <modifyers> ,
      * since those are not parts of following function */
-	return TrailingFuncSpecial( config, storage, 3 );
+	return TrailingFuncSpecial( config, 3 );
 }
 
 void
@@ -400,7 +407,6 @@ ParseFeelOptions (const char *filename, char *myname)
     item.memory = NULL;
 	PrintConfigReader (ConfigReader);
 	ParseConfig (ConfigReader, &Storage);
-    PrintFreeStorage (Storage);
 
 	/* getting rid of all the crap first */
     StorageCleanUp (&Storage, &(config->more_stuff), CF_DISABLED_OPTION);
@@ -783,7 +789,6 @@ ParseAutoExecOptions (const char *filename, char *myname)
     item.memory = NULL;
 	PrintConfigReader (ConfigReader);
 	ParseConfig (ConfigReader, &Storage);
-    PrintFreeStorage (Storage);
 
 	/* getting rid of all the crap first */
     StorageCleanUp (&Storage, &(config->more_stuff), CF_DISABLED_OPTION);
@@ -892,7 +897,6 @@ ParseThemeFile (const char *filename, char *myname)
 
 	PrintConfigReader (ConfigReader);
 	ParseConfig (ConfigReader, &Storage);
-	PrintFreeStorage (Storage);
 
 	/* getting rid of all the crap first */
     StorageCleanUp (&Storage, &(config->more_stuff), CF_DISABLED_OPTION);
