@@ -59,8 +59,8 @@ struct SyntaxDef     *TbarLayoutSyntaxPtr = &TbarLayoutSyntax;
 TermDef       MyFrameTerms[] = {
     {TF_NO_MYNAME_PREPENDING|TF_SYNTAX_START, "MyFrame", 7,     TT_QUOTED_TEXT, MYFRAME_START_ID, NULL},
     {TF_NO_MYNAME_PREPENDING, "North", 5,       TT_OPTIONAL_PATHNAME,    MYFRAME_North_ID, NULL},
-    {TF_NO_MYNAME_PREPENDING, "South", 5,       TT_OPTIONAL_PATHNAME,    MYFRAME_South_ID, NULL},
     {TF_NO_MYNAME_PREPENDING, "East", 4,        TT_OPTIONAL_PATHNAME,    MYFRAME_East_ID, NULL},
+    {TF_NO_MYNAME_PREPENDING, "South", 5,       TT_OPTIONAL_PATHNAME,    MYFRAME_South_ID, NULL},
     {TF_NO_MYNAME_PREPENDING, "West", 4,        TT_OPTIONAL_PATHNAME,    MYFRAME_West_ID, NULL},
     {TF_NO_MYNAME_PREPENDING, "NorthWest", 9,   TT_OPTIONAL_PATHNAME,    MYFRAME_NorthWest_ID, NULL},
     {TF_NO_MYNAME_PREPENDING, "NorthEast", 9,   TT_OPTIONAL_PATHNAME,    MYFRAME_NorthEast_ID, NULL},
@@ -83,23 +83,23 @@ TermDef       MyFrameTerms[] = {
     {TF_NO_MYNAME_PREPENDING, "FrameStickyStyle", 16,   TT_QUOTED_TEXT,  MYFRAME_FrameStickyStyle_ID, NULL},
     {TF_NO_MYNAME_PREPENDING, "FrameSStyle", 11,        TT_QUOTED_TEXT,  MYFRAME_FrameStickyStyle_ID, NULL},
     {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED, "SideSize", 8,        TT_GEOMETRY,MYFRAME_SideSize_ID, NULL},
-    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED_SUBCONFIG, "SideAlign", 9,       TT_FLAG,    MYFRAME_SideAlign_ID, &AlignSyntax},
-    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED_SUBCONFIG, "SideBevel", 9,       TT_FLAG,    MYFRAME_SideBevel_ID, &BevelSyntax},
-    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED_SUBCONFIG, "SideFBevel", 10,       TT_FLAG,    MYFRAME_SideFBevel_ID, &BevelSyntax},
-    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED_SUBCONFIG, "SideUBevel", 10,       TT_FLAG,    MYFRAME_SideUBevel_ID, &BevelSyntax},
-    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED_SUBCONFIG, "SideSBevel", 10,       TT_FLAG,    MYFRAME_SideSBevel_ID, &BevelSyntax},
-    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED_SUBCONFIG, "SideFocusedBevel", 16,      TT_FLAG,    MYFRAME_SideFBevel_ID, &BevelSyntax},
-    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED_SUBCONFIG, "SideUnfocusedBevel", 18,    TT_FLAG,    MYFRAME_SideUBevel_ID, &BevelSyntax},
-    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED_SUBCONFIG, "SideStickyBevel", 15,       TT_FLAG,    MYFRAME_SideSBevel_ID, &BevelSyntax},
+    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED, "SideAlign", 9,       TT_FLAG,    MYFRAME_SideAlign_ID, &AlignSyntax},
+    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED, "SideBevel", 9,       TT_FLAG,    MYFRAME_SideBevel_ID, &BevelSyntax},
+    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED, "SideFBevel", 10,       TT_FLAG,    MYFRAME_SideFBevel_ID, &BevelSyntax},
+    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED, "SideUBevel", 10,       TT_FLAG,    MYFRAME_SideUBevel_ID, &BevelSyntax},
+    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED, "SideSBevel", 10,       TT_FLAG,    MYFRAME_SideSBevel_ID, &BevelSyntax},
+    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED, "SideFocusedBevel", 16,      TT_FLAG,    MYFRAME_SideFBevel_ID, &BevelSyntax},
+    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED, "SideUnfocusedBevel", 18,    TT_FLAG,    MYFRAME_SideUBevel_ID, &BevelSyntax},
+    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED, "SideStickyBevel", 15,       TT_FLAG,    MYFRAME_SideSBevel_ID, &BevelSyntax},
     {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED, "CornerSize", 10,     TT_GEOMETRY,MYFRAME_CornerSize_ID, NULL},
-    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED_SUBCONFIG, "CornerAlign", 11,    TT_FLAG,    MYFRAME_CornerAlign_ID, &AlignSyntax},
-    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED_SUBCONFIG, "CornerBevel", 11,    TT_FLAG,    MYFRAME_CornerBevel_ID, &BevelSyntax},
-    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED_SUBCONFIG, "CornerFBevel", 12,    TT_FLAG,    MYFRAME_CornerFBevel_ID, &BevelSyntax},
-    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED_SUBCONFIG, "CornerUBevel", 12,    TT_FLAG,    MYFRAME_CornerUBevel_ID, &BevelSyntax},
-    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED_SUBCONFIG, "CornerSBevel", 12,    TT_FLAG,    MYFRAME_CornerSBevel_ID, &BevelSyntax},
-    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED_SUBCONFIG, "CornerFocusedBevel", 18,   TT_FLAG,    MYFRAME_CornerFBevel_ID, &BevelSyntax},
-    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED_SUBCONFIG, "CornerUnfocusedBevel", 20, TT_FLAG,    MYFRAME_CornerUBevel_ID, &BevelSyntax},
-    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED_SUBCONFIG, "CornerStickyBevel", 17,    TT_FLAG,    MYFRAME_CornerSBevel_ID, &BevelSyntax},
+    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED, "CornerAlign", 11,    TT_FLAG,    MYFRAME_CornerAlign_ID, &AlignSyntax},
+    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED, "CornerBevel", 11,    TT_FLAG,    MYFRAME_CornerBevel_ID, &BevelSyntax},
+    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED, "CornerFBevel", 12,    TT_FLAG,    MYFRAME_CornerFBevel_ID, &BevelSyntax},
+    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED, "CornerUBevel", 12,    TT_FLAG,    MYFRAME_CornerUBevel_ID, &BevelSyntax},
+    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED, "CornerSBevel", 12,    TT_FLAG,    MYFRAME_CornerSBevel_ID, &BevelSyntax},
+    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED, "CornerFocusedBevel", 18,   TT_FLAG,    MYFRAME_CornerFBevel_ID, &BevelSyntax},
+    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED, "CornerUnfocusedBevel", 20, TT_FLAG,    MYFRAME_CornerUBevel_ID, &BevelSyntax},
+    {TF_NO_MYNAME_PREPENDING|TF_DIRECTION_INDEXED|TF_NAMED, "CornerStickyBevel", 17,    TT_FLAG,    MYFRAME_CornerSBevel_ID, &BevelSyntax},
     {TF_NO_MYNAME_PREPENDING, "TitleBevel", 10,         TT_FLAG,         MYFRAME_TitleBevel_ID, &BevelSyntax},
     {TF_NO_MYNAME_PREPENDING, "TitleFBevel", 11,         TT_FLAG,         MYFRAME_TitleFBevel_ID, &BevelSyntax},
     {TF_NO_MYNAME_PREPENDING, "TitleUBevel", 11,         TT_FLAG,         MYFRAME_TitleUBevel_ID, &BevelSyntax},
@@ -172,6 +172,13 @@ SyntaxDef     MyFrameSyntax = {
 	0
 };
 
+const char *frame_side_idx2name( int idx )
+{
+	if( idx >= 0 && idx < FRAME_PARTS )
+		return	MyFrameTerms[1+idx].keyword ;
+	return NULL;
+}	 
+
 /********************************************************************/
 /* First MyFrame processing code				    */
 /********************************************************************/
@@ -337,7 +344,6 @@ LOCAL_DEBUG_OUT( "fd(%p)->tline(\"%s\")->fpd.data(\"%s\")", fd, tline, fpd.data
 
 	PrintConfigReader (ConfigReader);
 	ParseConfig (ConfigReader, &Storage);
-	PrintFreeStorage (Storage);
 
 	/* getting rid of all the crap first */
     StorageCleanUp (&Storage, &more_stuff, CF_DISABLED_OPTION);
@@ -515,7 +521,7 @@ ProcessMyFrameOptions (FreeStorageElem * options, MyFrameDefinition ** tail)
 					DestroyMyFrameDefinitions (tail);
                 	fd = AddMyFrameDefinition (tail);
 				}
-            	set_string_value (&(fd->name), item.data.string, NULL, 0);
+            	set_string (&(fd->name), item.data.string);
         	} else if( rel_id <= FRAME_PARTS )
 			{
             	--rel_id ;
@@ -535,22 +541,22 @@ ProcessMyFrameOptions (FreeStorageElem * options, MyFrameDefinition ** tail)
                     	set_flags( fd->parts_mask, (0x01<<rel_id));
                     	break;
                 	case MYFRAME_TitleUnfocusedStyle_ID :
-                    	set_string_value (&(fd->title_styles[BACK_UNFOCUSED]), item.data.string, NULL, 0);
+                    	set_string (&(fd->title_styles[BACK_UNFOCUSED]), item.data.string);
                     	break;
                 	case MYFRAME_TitleFocusedStyle_ID :
-                    	set_string_value (&(fd->title_styles[BACK_FOCUSED]), item.data.string, NULL, 0);
+                    	set_string (&(fd->title_styles[BACK_FOCUSED]), item.data.string);
                     	break;
                 	case MYFRAME_TitleStickyStyle_ID :
-                    	set_string_value (&(fd->title_styles[BACK_STICKY]), item.data.string, NULL, 0);
+                    	set_string (&(fd->title_styles[BACK_STICKY]), item.data.string);
                     	break;
                 	case MYFRAME_FrameUnfocusedStyle_ID :
-                    	set_string_value (&(fd->frame_styles[BACK_UNFOCUSED]), item.data.string, NULL, 0);
+                    	set_string (&(fd->frame_styles[BACK_UNFOCUSED]), item.data.string);
                     	break;
                 	case MYFRAME_FrameFocusedStyle_ID :
-                    	set_string_value (&(fd->frame_styles[BACK_FOCUSED]), item.data.string, NULL, 0);
+                    	set_string (&(fd->frame_styles[BACK_FOCUSED]), item.data.string);
                     	break;
                 	case MYFRAME_FrameStickyStyle_ID :
-                    	set_string_value (&(fd->frame_styles[BACK_STICKY]), item.data.string, NULL, 0);
+                    	set_string (&(fd->frame_styles[BACK_STICKY]), item.data.string);
                     	break;
                 	case MYFRAME_SideSize_ID :
                 	case MYFRAME_CornerSize_ID :
@@ -624,7 +630,7 @@ ProcessMyFrameOptions (FreeStorageElem * options, MyFrameDefinition ** tail)
 							int index = options->term->id - MYFRAME_TitleBackground_ID_START ;
 							if( index >= 0 && index < MYFRAME_TITLE_BACKS )
 							{
-		                    	set_string_value (&(fd->title_backs[index]), item.data.string, NULL, 0);
+		                    	set_string (&(fd->title_backs[index]), item.data.string);
 							}else
 							{
 								item.ok_to_free = 1;
@@ -713,8 +719,7 @@ LOCAL_DEBUG_OUT( "fd(%p)->tline(\"%s\")->fpd.data(\"%s\")", fd, tline, fpd.data
 
 	PrintConfigReader (ConfigReader);
 	ParseConfig (ConfigReader, &Storage);
-	PrintFreeStorage (Storage);
-
+	
 	/* getting rid of all the crap first */
     StorageCleanUp (&Storage, &more_stuff, CF_DISABLED_OPTION);
     DestroyFreeStorage (&more_stuff);
