@@ -222,10 +222,25 @@ CreatePagerConfig (int ndesks)
 void
 DestroyPagerConfig (PagerConfig * config)
 {
+	int i ;
 	if (config->labels)
+	{
+		for( i = 0 ; i < config->ndesks ; ++i )
+			if( config->labels[i] ) 
+				free( config->labels[i] );
 		free (config->labels);
+	}
 	if (config->styles)
+	{
+		for( i = 0 ; i < config->ndesks ; ++i )
+			if( config->styles[i] ) 
+				free( config->styles[i] );
 		free (config->styles);
+	}
+    destroy_string( &(config->selection_color) );
+    destroy_string( &(config->grid_color));
+    destroy_string( &(config->border_color));
+
 /*
     if( config->shade_button[0] )
         free (config->shade_button[0]);
@@ -235,8 +250,6 @@ DestroyPagerConfig (PagerConfig * config)
     Destroy_balloonConfig (config->balloon_conf);
     DestroyFreeStorage (&(config->more_stuff));
 	DestroyMyStyleDefinitions (&(config->style_defs));
-	if( config->shade_btn ) 
-		free( config->shade_btn );
 	free (config);
 }
 
@@ -335,7 +348,6 @@ ParsePagerOptions (const char *filename, char *myname, int desk1, int desk2)
 	item.memory = NULL;
 	PrintConfigReader (PagerConfigReader);
 	ParseConfig (PagerConfigReader, &Storage);
-	PrintFreeStorage (Storage);
 
 	/* getting rid of all the crap first */
 	StorageCleanUp (&Storage, &(config->more_stuff), CF_DISABLED_OPTION);
