@@ -213,6 +213,12 @@ LOCAL_DEBUG_OUT( "face found : %p", face );
 					font->type = ASF_Freetype ;
 					font->flags = flags ;
 					font->ft_face = face ;
+					if( FT_HAS_KERNING( face ) )
+					{	
+						set_flags( font->flags, ASF_HasKerning );
+						/* fprintf( stderr, "@@@@@@@font %s has kerning!!!\n", realfilename );*/
+					}/*else
+						fprintf( stderr, "@@@@@@@font %s don't have kerning!!!\n", realfilename ); */
 					/* lets confine the font to square cell */
 					FT_Set_Pixel_Sizes( font->ft_face, size, size );
 					/* but let make our own cell width smaller then height */
@@ -1032,13 +1038,19 @@ load_glyph_freetype( ASFont *font, ASGlyph *asg, int glyph, UNICODE_CHAR uc )
 		int src_step ;
 /* 		int hpad = (face->glyph->bitmap_left<0)? -face->glyph->bitmap_left: face->glyph->bitmap_left ;
 */
+		asg->font_gid = glyph ; 
 		asg->width   = bmap->width ;
 		asg->height  = bmap->rows ;
 		/* Combining Diacritical Marks : */
 		if( uc >= 0x0300 && uc <= 0x0362 ) 
 			asg->step = 0 ; 
 		else
+#if 0			
 			asg->step = bmap->width+face->glyph->bitmap_left ;
+#else
+			asg->step = face->glyph->advance.x>>6 ;
+#endif
+				
 		/* we only want to keep lead if it was negative */
 		if( uc >= 0x0300 && uc <= 0x0362 && face->glyph->bitmap_left >= 0 ) 
 			asg->lead    = -((int)font->space_size - (int)face->glyph->bitmap_left) ;
@@ -1154,12 +1166,12 @@ load_freetype_locale_glyph( ASFont *font, UNICODE_CHAR uc )
 		load_glyph_freetype( font, asg, FT_Get_Char_Index( font->ft_face, uc), uc);
 		if( add_hash_item( font->locale_glyphs, AS_HASHABLE(uc), asg ) != ASH_Success )
 		{
-			LOCAL_DEBUG_OUT( "Failed to add glyph %p for char %d to hash", asg, uc );
+			LOCAL_DEBUG_OUT( "Failed to add glyph %p for char %ld to hash", asg, uc );
 			asglyph_destroy( 0, asg);
 			asg = NULL ;
 		}else
 		{
-			LOCAL_DEBUG_OUT( "added glyph %p for char %d to hash font attr(%d,%d,%d) glyph attr (%d,%d)", asg, uc, font->max_ascend, font->max_descend, font->max_height, asg->ascend, asg->descend );
+			LOCAL_DEBUG_OUT( "added glyph %p for char %ld to hash font attr(%d,%d,%d) glyph attr (%d,%d)", asg, uc, font->max_ascend, font->max_descend, font->max_height, asg->ascend, asg->descend );
 
 			if( asg->ascend > font->max_ascend )
 				font->max_ascend = asg->ascend ;
@@ -1260,7 +1272,7 @@ inline ASGlyph *get_unicode_glyph( const UNICODE_CHAR uc, ASFont *font )
 	ASHashData hdata = {0} ;
 	for( r = font->codemap ; r != NULL ; r = r->above )
 	{
-LOCAL_DEBUG_OUT( "looking for glyph for char %lu (%p) if range (%d,%d)", uc, asg, r->min_char, r->max_char);
+LOCAL_DEBUG_OUT( "looking for glyph for char %lu (%p) if range (%ld,%ld)", uc, asg, r->min_char, r->max_char);
 
 		if( r->max_char >= uc )
 			if( r->min_char <= uc )
@@ -1276,6 +1288,7 @@ LOCAL_DEBUG_OUT( "Found glyph for char %lu (%p)", uc, asg );
 	{
 #ifdef HAVE_FREETYPE
 		asg = load_freetype_locale_glyph( font, uc );
+LOCAL_DEBUG_OUT( "glyph for char %lu  loaded as %p", uc, asg );
 #endif
 	}else
 		asg = hdata.vptr ;
@@ -1355,6 +1368,7 @@ utf8_to_unicode ( const unsigned char *s )
 inline ASGlyph *get_utf8_glyph( const char *utf8, ASFont *font )
 {
 	UNICODE_CHAR uc = utf8_to_unicode ( (const unsigned char*)utf8 );
+	LOCAL_DEBUG_OUT( "translated to Unicode 0x%lX(%ld), UTF8 size = %d", uc, uc, UTF8_CHAR_SIZE(utf8[0]) );
 	return get_unicode_glyph( uc, font );
 }
 
@@ -1372,6 +1386,7 @@ typedef struct ASGlyphMap
 #define GLYPH_EOT	((ASGlyph*)0x00000000)
 	ASGlyph 	**glyphs;
 	int 		  glyphs_num;
+	short 		 *x_kerning ;
 }ASGlyphMap;
 
 
@@ -1425,6 +1440,17 @@ goto_tab_stop( ASTextAttributes *attr, unsigned int space_size, unsigned int lin
 	return tab_stop;		
 }
 
+#ifdef HAVE_FREETYPE
+#define GET_KERNING(var,prev_gid,this_gid)   \
+	do{ if( (prev_gid) != 0 && font->type == ASF_Freetype && get_flags(font->flags, ASF_Monospaced|ASF_HasKerning) == ASF_HasKerning ) { \
+		FT_Vector delta; \
+		FT_Get_Kerning( font->ft_face, (prev_gid), (this_gid), FT_KERNING_DEFAULT, &delta );\
+		(var) = delta.x >> 6; \
+	}}while(0)
+#else
+#define GET_KERNING(var,prev_gid,this_gid)	do{(var)=0;}while(0)	  
+#endif
+/*		fprintf( stderr, "####### pair %d ,%d 	has kerning = %d\n", prev_gid,this_gid, var ); */
 
 
 #define FILL_TEXT_GLYPH_MAP(name,type,getglyph,incr) \
@@ -1433,7 +1459,7 @@ name( const type *text, ASFont *font, ASGlyphMap *map, ASTextAttributes *attr, i
 { \
 	unsigned int w = 0, line_count = 0, line_width = 0; \
 	int i = -1, g = 0 ; \
-	ASGlyph *last_asg = NULL ; \
+	ASGlyph *last_asg = NULL ; unsigned int last_gid = 0 ; \
 	do \
 	{ \
 		++i ; \
@@ -1442,7 +1468,7 @@ name( const type *text, ASFont *font, ASGlyphMap *map, ASTextAttributes *attr, i
 		{ \
 			if( last_asg && last_asg->width+last_asg->lead > last_asg->step ) \
 				line_width += last_asg->width+last_asg->lead - last_asg->step ; \
-			last_asg = NULL; \
+			last_asg = NULL; last_gid = 0 ; \
 			if( line_width > w ) \
 				w = line_width ; \
 			line_width = 0 ; \
@@ -1453,19 +1479,21 @@ name( const type *text, ASFont *font, ASGlyphMap *map, ASTextAttributes *attr, i
 		{ \
 			last_asg = NULL ; \
 			if( text[i] == ' ' ) \
-			{ \
+			{   last_gid = 0 ; \
 				line_width += space_size ; \
 				map->glyphs[g++] = GLYPH_SPACE; \
 			}else if( text[i] == '\t' ) \
-			{ \
+			{   last_gid = 0 ; \
 				if( !get_flags(attr->rendition_flags, ASTA_UseTabStops) ) line_width += space_size*attr->tab_size ; \
 				else line_width = goto_tab_stop( attr, space_size, line_width ); \
 				map->glyphs[g++] = GLYPH_TAB; \
 			}else \
 			{ \
 				last_asg = getglyph; \
-				map->glyphs[g++] = last_asg; \
-				line_width += last_asg->step+offset_3d_x ; \
+				map->glyphs[g] = last_asg; \
+				GET_KERNING(map->x_kerning[g],last_gid,last_asg->font_gid); \
+				line_width += last_asg->step+offset_3d_x+map->x_kerning[g]; \
+				++g; last_gid = last_asg->font_gid ; \
 				LOCAL_DEBUG_OUT("pre_i=%d",i); \
 				incr; /* i+=CHAR_SIZE(text[i])-1; */ \
 				LOCAL_DEBUG_OUT("post_i=%d",i); \
@@ -1492,6 +1520,8 @@ free_glyph_map( ASGlyphMap *map, Bool reusable )
     {
 		if( map->glyphs )
 	        free( map->glyphs );
+		if( map->x_kerning )
+	        free( map->x_kerning );
         if( !reusable )
             free( map );
     }
@@ -1545,6 +1575,7 @@ get_text_glyph_map( const char *text, ASFont *font, ASGlyphMap *map, ASTextAttri
 	if( map->glyphs_num == 0 )
 		return False;
 	map->glyphs = safecalloc( map->glyphs_num, sizeof(ASGlyph*));
+	map->x_kerning = safecalloc( map->glyphs_num, sizeof(short));
 
 	if( attr->char_type == ASCT_Char )
 		line_count = fill_text_glyph_map_Char( text, font, map, attr, space_size, offset_3d_x );
@@ -1570,7 +1601,7 @@ get_text_glyph_map( const char *text, ASFont *font, ASGlyphMap *map, ASTextAttri
 		if( terminated ) { \
 			if( last_asg && last_asg->width+last_asg->lead > last_asg->step ) \
 				line_width += last_asg->width+last_asg->lead - last_asg->step ; \
-			last_asg = NULL; \
+			last_asg = NULL; last_gid = 0 ; \
 			if( line_width > w ) \
 				w = line_width ; \
 			line_width = 0 ; \
@@ -1578,13 +1609,15 @@ get_text_glyph_map( const char *text, ASFont *font, ASGlyphMap *map, ASTextAttri
 		}else { \
 			last_asg = NULL ; \
 			if( text[i] == ' ' ){ \
-				line_width += space_size ; \
-			}else if( text[i] == '\t' ){ \
+				line_width += space_size ; last_gid = 0 ;\
+			}else if( text[i] == '\t' ){ last_gid = 0 ; \
 				if( !get_flags(attr->rendition_flags, ASTA_UseTabStops) ) line_width += space_size*attr->tab_size ; \
 				else line_width = goto_tab_stop( attr, space_size, line_width ); \
-			}else{ \
+			}else{ int kerning = 0 ;\
 				last_asg = getglyph; \
-				line_width += last_asg->step+offset_3d_x ;  \
+				GET_KERNING(kerning,last_gid,last_asg->font_gid); \
+				line_width += last_asg->step+offset_3d_x +kerning ;  \
+				last_gid = last_asg->font_gid ; \
 				incr ; \
 			} \
 		} \
@@ -1599,6 +1632,8 @@ get_text_size_internal( const char *src_text, ASFont *font, ASTextAttributes *at
 	ASGlyph *last_asg = NULL ;
 	int space_size = 0;
 	unsigned int offset_3d_x = 0, offset_3d_y = 0 ;
+	unsigned int last_gid = 0 ;
+
 
 	apply_text_3D_type( attr->type, &offset_3d_x, &offset_3d_y );
 	if( src_text == NULL || font == NULL )
@@ -1611,7 +1646,8 @@ get_text_size_internal( const char *src_text, ASFont *font, ASTextAttributes *at
 	if( !get_flags( font->flags, ASF_Monospaced) )
 		space_size  = (space_size>>1)+1 ;
 	space_size += offset_3d_x ;
-		
+
+	LOCAL_DEBUG_OUT( "char_type = %d", attr->char_type );
 	if( attr->char_type == ASCT_Char )
 	{
 		char *text = (char*)&src_text[0] ;
@@ -1623,7 +1659,18 @@ get_text_size_internal( const char *src_text, ASFont *font, ASTextAttributes *at
 	}else if( attr->char_type == ASCT_UTF8 )
 	{
 		char *text = (char*)&src_text[0] ;
-		GET_TEXT_SIZE_LOOP(get_utf8_glyph(&text[i],font),i+=UTF8_CHAR_SIZE(text[i])-1,length);
+		int byte_length = 0 ;
+		if( length > 0 )
+		{
+			int k ; 
+			for( k = 0 ; k < length ; ++k )
+			{
+				if( text[byte_length] == '\0' ) 
+					break;
+				byte_length += UTF8_CHAR_SIZE(text[byte_length]);		   
+			}	 
+		}	 
+		GET_TEXT_SIZE_LOOP(get_utf8_glyph(&text[i],font),i+=UTF8_CHAR_SIZE(text[i])-1,byte_length);
 	}else if( attr->char_type == ASCT_Unicode )
 	{
 		UNICODE_CHAR *text = (UNICODE_CHAR*)&src_text[0] ;
@@ -1787,11 +1834,37 @@ draw_text_internal( const char *text, ASFont *font, ASTextAttributes *attr, int
 	CARD32 alpha_7 = 0x007F, alpha_9 = 0x009F, alpha_A = 0x00AF, alpha_C = 0x00CF, alpha_F = 0x00FF, alpha_E = 0x00EF;
 	START_TIME(started);	   
 
+	// Perform line breaks if a fixed width is specified
+	// TODO: this is a quick and dirty fix and should work for now, but we really should fix it 
+	// so we don't have to calculate text size so many times as well as make it UNICODE friendly
+	// and remove mangling of the source text (Sasha): 
+	if (attr->width)
+	{
+        unsigned int width, height; // SMA
+        get_text_size(  text , font, attr->type, &width, &height ); 
+        if ( (width > attr->width)  &&  (strchr(text, ' ')) )
+        {
+          char *tryPtr = strchr(text,' ');
+          char *oldTryPtr = tryPtr;
+          while (tryPtr)
+            {        
+               *tryPtr = 0;
+               get_text_size(  text , font, attr->type, &width, &height ); 
+               if (width > attr->width)
+                   *oldTryPtr = '\n';
+               
+               *tryPtr = ' ';
+               oldTryPtr = tryPtr;
+               tryPtr = strchr(tryPtr + 1,' ');
+            }
+        }
+	}	    
+
 LOCAL_DEBUG_CALLER_OUT( "text = \"%s\", font = %p, compression = %d", text, font, compression );
 	if( !get_text_glyph_map( text, font, &map, attr, length) )
 		return NULL;
 	
-	if( map.width == 0 ) 
+	if( map.width <= 0 ) 
 		return NULL;
 
 	apply_text_3D_type( attr->type, &(offset_3d_x), &(offset_3d_y) );
@@ -1905,7 +1978,9 @@ LOCAL_DEBUG_OUT( "line_height is %d, space_size is %d, base_line is %d", line_he
 				int start_x = 0;
 
 				if( get_flags(font->flags, ASF_RightToLeft) )
-					pen_x  -= asg->step+offset_3d_x ;
+					pen_x  -= asg->step+offset_3d_x +map.x_kerning[i];
+				else
+					pen_x += map.x_kerning[i] ;
 				if( asg->lead > 0 )
 					start_x = pen_x + asg->lead ;
 				else if( pen_x  > -asg->lead )
