@@ -172,6 +172,8 @@ asimage_init (ASImage * im, Bool free_resources)
 				free( im->alt.argb32 );
 			if( im->alt.vector )
 				free( im->alt.vector );
+			if( im->name ) 
+				free( im->name );
 		}
 		memset (im, 0x00, sizeof (ASImage));
 		im->magic = MAGIC_ASIMAGE ;
@@ -233,6 +235,32 @@ asimage_start (ASImage * im, unsigned int width, unsigned int height, unsigned i
 	}
 }
 
+Bool
+asimage_replace (ASImage *im, ASImage *from)
+{
+	if ( im && from && im != from )
+		if( im->magic == MAGIC_ASIMAGE && from->magic == MAGIC_ASIMAGE && from->imageman == NULL )
+		{
+			int ref_count = im->ref_count ;
+			ASImageManager *imageman = im->imageman ;
+			char *name = im->name ; 
+
+			im->name = NULL ; 
+			asimage_init (im, True);
+
+			memcpy( im, from, sizeof(ASImage) );
+			memset( from, 0x00, sizeof(ASImage) );		
+
+			im->ref_count = ref_count ; 
+			im->imageman = imageman ;
+			im->name = name ;
+
+			return True ;
+		}
+	return False;
+}
+
+
 static ASImage* 
 check_created_asimage( ASImage *im, unsigned int width, unsigned int height )
 {
@@ -387,7 +415,8 @@ asimage_destroy (ASHashableValue value, void *data)
 			else
 				im->imageman = NULL ;
 		}
-		free( (char*)value );
+		if( (char*)value != im->name ) 
+			free( (char*)value );/* name */
 		destroy_asimage( &im );
 	}
 }
@@ -450,6 +479,8 @@ store_asimage( ASImageManager* imageman, ASImage *im, const char *name )
 		if( im->imageman == NULL )
 		{
 			int hash_res ;
+			if( im->name ) 
+				free( im->name );
 			im->name = mystrdup( name );
 			hash_res = add_hash_item( imageman->image_hash, AS_HASHABLE(im->name), im);
 			res = ( hash_res == ASH_Success);
@@ -542,9 +573,8 @@ forget_asimage( ASImage *im )
 			ASImageManager *imman = im->imageman ;
 			if( !AS_ASSERT(imman) )
 				remove_hash_item(imman->image_hash, (ASHashableValue)(char*)im->name, NULL, False);
-/*            im->ref_count = 0;     release_asimage should still correctly work !!!!
- *            im->imageman = NULL;
- */
+            im->ref_count = 0;
+            im->imageman = NULL;
 		}
 	}
 }
@@ -697,7 +727,7 @@ flip_gradient( ASGradient *orig, int flip )
 
 /* ******************** ASImageLayer ****************************/
 
-inline void
+void
 init_image_layers( register ASImageLayer *l, int count )
 {
 	memset( l, 0x00, sizeof(ASImageLayer)*count );
@@ -812,7 +842,7 @@ void print_asimage( ASImage *im, int flags, char * func, int line )
 
 void print_component( register CARD32 *data, int nonsense, int len );
 
-inline int
+int
 asimage_decode_line (ASImage * im, ColorPart color, CARD32 * to_buf, unsigned int y, unsigned int skip, unsigned int out_width)
 {
 	ASStorageID id = im->channels[color][y];
