@@ -325,7 +325,7 @@ file2pixmap(ASVisual *asv, Window root, const char *realfilename, Pixmap *mask_o
 	}
 	if( mask_out )
 	{
-		if( *mask_out )
+		if( *mask_out && asv )
 			XFreePixmap( asv->dpy, *mask_out );
 		*mask_out = mask ;
 	}
@@ -333,6 +333,22 @@ file2pixmap(ASVisual *asv, Window root, const char *realfilename, Pixmap *mask_o
 	return trg ;
 }
 
+static ASImage *
+load_image_from_path( const char *file, char **path, double gamma)
+{
+	if( path[2] == NULL )
+		return file2ASImage( file, 0xFFFFFFFF, gamma, 100, path[0], path[1], NULL );
+#if (MAX_SEARCH_PATHS!=8)
+#error "Fixme: please update file2ASImage call to match max number of search paths in ImageManager"
+#else
+	else
+		return file2ASImage( file, 0xFFFFFFFF, gamma, 100, path[0], path[1], path[2],
+			    	       path[3], path[4], path[5], path[6], path[7], NULL );
+#endif
+
+	return NULL ;
+}
+
 ASImage *
 get_asimage( ASImageManager* imageman, const char *file, ASFlagType what, unsigned int compression )
 {
@@ -340,22 +356,52 @@ get_asimage( ASImageManager* imageman, const char *file, ASFlagType what, unsign
 	if( imageman && file )
 		if( (im = fetch_asimage(imageman, file )) == NULL )
 		{
-			char **path = &(imageman->search_path[0]);
-			if( path[2] == NULL )
-				im = file2ASImage( file, what, imageman->gamma, compression, path[0], path[1], NULL );
-#if (MAX_SEARCH_PATHS!=8)
-#error "Fixme: please update file2ASImage call to match max number of search paths in ImageManager"
-#else
-			else
-				im = file2ASImage( file, what, imageman->gamma, compression, path[0], path[1], path[2],
-			    	               path[3], path[4], path[5], path[6], path[7], NULL );
-#endif
+			im = load_image_from_path( file, &(imageman->search_path[0]), imageman->gamma);
 			if( im )
+			{
 				store_asimage( imageman, im, file );
+				set_flags( im->flags, ASIM_NAME_IS_FILENAME );
+			}
+				
 		}
 	return im;
 }
 
+Bool
+reload_asimage_manager( ASImageManager *imman )
+{
+#if (HAVE_AFTERBASE_FLAG==1)
+	if( imman != NULL ) 
+	{
+		ASHashIterator iter ;
+		if( start_hash_iteration (imman->image_hash, &iter) )
+		{
+			do
+			{
+				ASImage *im = curr_hash_data( &iter );
+/*fprintf( stderr, "im = %p. flags = 0x%lX\n", im, im->flags );		*/
+				if( get_flags( im->flags, ASIM_NAME_IS_FILENAME ) )
+				{
+/*fprintf( stderr, "reloading image \"%s\" ...", im->name );*/
+					ASImage *reloaded_im = load_image_from_path( im->name, &(imman->search_path[0]), imman->gamma);
+/*fprintf( stderr, "Done. reloaded_im = %p.\n", reloaded_im );*/					
+					if( reloaded_im ) 
+					{
+						if( asimage_replace (im, reloaded_im) ) 
+							free( reloaded_im );
+						else
+							destroy_asimage( &reloaded_im );
+					}				
+				}
+			}while( next_hash_item( &iter ) );
+			return True;		
+		}
+	}
+#endif
+	return False;
+}
+
+
 ASImageListEntry * 
 ref_asimage_list_entry( ASImageListEntry *entry )
 {
@@ -396,6 +442,8 @@ unref_asimage_list_entry( ASImageListEntry *entry )
 					free( entry->name );
 				if( entry->fullfilename )
 					free( entry->fullfilename );
+				if( entry->buffer ) 
+					destroy_asimage_list_entry_buffer( &(entry->buffer) );
 				memset( entry, 0x00, sizeof(ASImageListEntry));
 				free( entry );
 				entry = NULL ; 
@@ -431,6 +479,108 @@ destroy_asimage_list( ASImageListEntry **plist )
 	}
 }
 
+void destroy_asimage_list_entry_buffer( ASImageListEntryBuffer **pbuffer )
+{
+	if( pbuffer && *pbuffer ) 
+	{		 
+		if( (*pbuffer)->data ) 
+			free( (*pbuffer)->data ) ;
+		free( *pbuffer );
+		*pbuffer = NULL ;
+	}
+}	 
+
+struct ASImageListAuxData
+{
+	ASImageListEntry **pcurr;
+	ASImageListEntry *last ;
+	ASFlagType preview_type ;
+	unsigned int preview_width ;
+	unsigned int preview_height ;
+	unsigned int preview_compression ;
+	ASVisual *asv;
+};
+
+Bool 
+direntry2ASImageListEntry( const char *fname, const char *fullname, 
+						   struct stat *stat_info, void *aux_data)
+{
+	struct ASImageListAuxData *data = (struct ASImageListAuxData*)aux_data;
+	ASImageFileTypes file_type ;
+	ASImageListEntry *curr ;
+	   	
+	if (S_ISDIR (stat_info->st_mode))
+		return False;
+	
+	file_type = check_image_type( fullname );
+	if( file_type != ASIT_Unknown && as_image_file_loaders[file_type] == NULL )
+		file_type = ASIT_Unknown ;
+
+	curr = create_asimage_list_entry();
+	*(data->pcurr) = curr ; 
+	if( data->last )
+		data->last->next = curr ;
+	curr->prev = data->last ;
+	data->last = curr ;
+	data->pcurr = &(data->last->next);
+
+	curr->name = mystrdup( fname );
+	curr->fullfilename = mystrdup(fullname);
+	curr->type = file_type ;
+   	curr->d_mode = stat_info->st_mode;
+	curr->d_mtime = stat_info->st_mtime;
+	curr->d_size  = stat_info->st_size;
+
+	if( curr->type != ASIT_Unknown && data->preview_type != 0 )
+	{
+		ASImageImportParams iparams = {0} ;
+		ASImage *im = as_image_file_loaders[file_type](fullname, &iparams);
+		if( im )
+		{
+			int scale_width = im->width ;
+			int scale_height = im->height ;
+			int tile_width = im->width ;
+			int tile_height = im->height ;
+
+			if( data->preview_width > 0 )
+			{
+				if( get_flags( data->preview_type, SCALE_PREVIEW_H ) )
+					scale_width = data->preview_width ;
+				else
+					tile_width = data->preview_width ;
+			}
+			if( data->preview_height > 0 )
+			{
+				if( get_flags( data->preview_type, SCALE_PREVIEW_V ) )
+					scale_height = data->preview_height ;
+				else
+					tile_height = data->preview_height ;
+			}
+			if( scale_width != im->width || scale_height != im->height )
+			{
+				ASImage *tmp = scale_asimage( data->asv, im, scale_width, scale_height, ASA_ASImage, data->preview_compression, ASIMAGE_QUALITY_DEFAULT );
+				if( tmp != NULL )
+				{
+					destroy_asimage( &im );
+					im = tmp ;
+				}
+			}
+			if( tile_width != im->width || tile_height != im->height )
+			{
+				ASImage *tmp = tile_asimage( data->asv, im, 0, 0, tile_width, tile_height, TINT_NONE, ASA_ASImage, data->preview_compression, ASIMAGE_QUALITY_DEFAULT );
+				if( tmp != NULL )
+				{
+					destroy_asimage( &im );
+					im = tmp ;
+				}
+			}
+		}
+
+		curr->preview = im ;
+	}
+	return True;
+}
+
 
 ASImageListEntry *
 get_asimage_list( ASVisual *asv, const char *dir,
@@ -442,96 +592,22 @@ get_asimage_list( ASVisual *asv, const char *dir,
 {
 	ASImageListEntry *im_list = NULL ;
 #ifndef _WIN32
-	ASImageListEntry **curr = &im_list, *last = NULL ;
-	struct direntry  **list;
-	int n, i, count = 0;
-	int dir_name_len ;
-
-
+	struct ASImageListAuxData aux_data ; 
+	int count ; 
+	
+	aux_data.pcurr = &im_list;
+	aux_data.last = NULL;
+	aux_data.preview_type = preview_type;
+	aux_data.preview_width = preview_width;
+	aux_data.preview_height = preview_height;
+	aux_data.preview_compression  = preview_compression;
+	aux_data.asv = asv ; 
+	
+	
 	if( asv == NULL || dir == NULL )
 		return NULL ;
 
-	dir_name_len = strlen(dir);
-	n = my_scandir ((char*)dir, &list, select, NULL);
-	if( n > 0 )
-	{
-		for (i = 0; i < n; i++)
-		{
-			if (!S_ISDIR (list[i]->d_mode))
-			{
-				ASImageFileTypes file_type ;
-				char *realfilename ;
-
-				realfilename = safemalloc( dir_name_len + 1 + strlen(list[i]->d_name)+1);
-				sprintf( realfilename, "%s/%s", dir, list[i]->d_name );
-				file_type = check_image_type( realfilename );
-				if( file_type != ASIT_Unknown && as_image_file_loaders[file_type] == NULL )
-					file_type = ASIT_Unknown ;
-
-				++count ;
-				*curr = create_asimage_list_entry();
-				if( last )
-					last->next = *curr ;
-				(*curr)->prev = last ;
-				last = *curr ;
-				curr = &(last->next);
-
-				last->name = mystrdup( list[i]->d_name );
-				last->fullfilename = realfilename ;
-				last->type = file_type ;
-
-				if( last->type != ASIT_Unknown && preview_type != 0 )
-				{
-					ASImageImportParams iparams = {0} ;
-					ASImage *im = as_image_file_loaders[file_type](realfilename, &iparams);
-					if( im )
-					{
-						int scale_width = im->width ;
-						int scale_height = im->height ;
-						int tile_width = im->width ;
-						int tile_height = im->height ;
-
-						if( preview_width > 0 )
-						{
-							if( get_flags( preview_type, SCALE_PREVIEW_H ) )
-								scale_width = preview_width ;
-							else
-								tile_width = preview_width ;
-						}
-						if( preview_height > 0 )
-						{
-							if( get_flags( preview_type, SCALE_PREVIEW_V ) )
-								scale_height = preview_height ;
-							else
-								tile_height = preview_height ;
-						}
-						if( scale_width != im->width || scale_height != im->height )
-						{
-							ASImage *tmp = scale_asimage( asv, im, scale_width, scale_height, ASA_ASImage, preview_compression, ASIMAGE_QUALITY_DEFAULT );
-							if( tmp != NULL )
-							{
-								destroy_asimage( &im );
-								im = tmp ;
-							}
-						}
-						if( tile_width != im->width || tile_height != im->height )
-						{
-							ASImage *tmp = tile_asimage( asv, im, 0, 0, tile_width, tile_height, TINT_NONE, ASA_ASImage, preview_compression, ASIMAGE_QUALITY_DEFAULT );
-							if( tmp != NULL )
-							{
-								destroy_asimage( &im );
-								im = tmp ;
-							}
-						}
-					}
-
-			  		last->preview = im ;
-				}
-			}
-			free( list[i] );
-		}
-		free (list);
-	}
+	count = my_scandir_ext ((char*)dir, select, direntry2ASImageListEntry, &aux_data);
 
 	if( count_ret )
 		*count_ret = count ;
@@ -556,6 +632,62 @@ char *format_asimage_list_entry_details( ASImageListEntry *entry, Bool vertical
 	return details_text;
 }	 
 
+Bool 
+load_asimage_list_entry_data( ASImageListEntry *entry, size_t max_bytes )
+{
+	char * new_buffer ; 
+	size_t new_buffer_size ;
+	FILE *fp;
+	Bool binary = False ; 
+	if( entry == NULL ) 
+		return False;
+	if( entry->buffer == NULL ) 
+		entry->buffer = safecalloc( 1, sizeof(ASImageListEntryBuffer) );
+	if( entry->buffer->size == entry->d_size || entry->buffer->size >= max_bytes )
+		return True;
+	new_buffer_size = min( max_bytes, (size_t)entry->d_size ); 
+	new_buffer = malloc( new_buffer_size );
+	if( new_buffer == NULL ) 
+		return False ;
+	if( entry->buffer->size > 0 ) 
+	{	
+		memcpy( new_buffer, entry->buffer->data, entry->buffer->size ) ;
+		free( entry->buffer->data );
+	}
+	entry->buffer->data = new_buffer ; 
+	/* TODO read new_buffer_size - entry->buffer_size bytes into the end of the buffer */
+	fp = fopen(entry->fullfilename, "rb");
+	if ( fp != NULL ) 
+	{
+		int len = new_buffer_size - entry->buffer->size ;
+		if( entry->buffer->size > 0 ) 
+			fseek( fp, entry->buffer->size, SEEK_SET );
+		len = fread(entry->buffer->data, 1, len, fp);
+		if( len > 0 ) 
+			entry->buffer->size += len ;
+		fclose(fp);
+	}
+
+	if( entry->type == ASIT_Unknown ) 
+	{
+		int i = entry->buffer->size ; 
+		register char *ptr = entry->buffer->data ;
+		while ( --i >= 0 )	
+			if( !isprint(ptr[i]) && ptr[i] != '\n'&& ptr[i] != '\r'&& ptr[i] != '\t' )	
+				break;
+		binary = (i >= 0);				
+	}else
+		binary = (entry->type != ASIT_Xpm  && entry->type != ASIT_XMLScript &&
+			  	  entry->type != ASIT_HTML && entry->type != ASIT_XML ); 
+	if( binary ) 
+		set_flags( entry->buffer->flags, ASILEB_Binary );
+   	else
+		clear_flags( entry->buffer->flags, ASILEB_Binary );
+	 
+
+
+	return True;
+}
 
 /***********************************************************************************/
 /* Some helper functions :                                                         */
@@ -610,7 +742,7 @@ check_image_type( const char *realfilename )
 {
 	int filename_len = strlen( realfilename );
 #define FILE_HEADER_SIZE	512
-	CARD8 head[FILE_HEADER_SIZE+1] ;
+	char head[FILE_HEADER_SIZE+1] ;
 	int bytes_in = 0 ;
 	FILE *fp ;
 	ASImageFileTypes type = ASIT_Unknown ;
@@ -632,7 +764,7 @@ check_image_type( const char *realfilename )
  */
 		if( bytes_in > 3 )
 		{
-			if( head[0] == 0xff && head[1] == 0xd8 && head[2] == 0xff)
+			if( (CARD8)head[0] == 0xff && (CARD8)head[1] == 0xd8 && (CARD8)head[2] == 0xff)
 				type = ASIT_Jpeg;
 			else if (strstr ((char *)&(head[0]), "XPM") != NULL)
 				type =  ASIT_Xpm;
@@ -662,12 +794,12 @@ check_image_type( const char *realfilename )
 		}	 
 		if( type == ASIT_Unknown && bytes_in  > 8 )
 		{
-			if( strncmp((char *)&(head[0]), XCF_SIGNATURE, (size_t) XCF_SIGNATURE_LEN) == 0)
+			if( strncmp(&(head[0]), XCF_SIGNATURE, (size_t) XCF_SIGNATURE_LEN) == 0)
 				type = ASIT_Xcf;
 	   		else if (head[0] == 0 && head[1] == 0 &&
 			    	 head[2] == 2 && head[3] == 0 && head[4] == 0 && head[5] == 0 && head[6] == 0 && head[7] == 0)
 				type = ASIT_Targa;
-			else if (strncmp ((char *)&(head[0]), "#define", (size_t) 7) == 0)
+			else if (strncmp (&(head[0]), "#define", (size_t) 7) == 0)
 				type = ASIT_Xbm;
 			else
 			{/* the nastiest check - for XML files : */
@@ -729,6 +861,14 @@ check_image_type( const char *realfilename )
 }
 
 
+ASImageFileTypes
+check_asimage_file_type( const char *realfilename )
+{
+	if( realfilename == NULL ) 
+		return ASIT_Unknown;
+	return check_image_type( realfilename );
+}
+
 /***********************************************************************************/
 #ifdef HAVE_XPM      /* XPM XPM XPM XPM XPM XPM XPM XPM XPM XPM XPM XPM XPM XPM XPM XPM */
 
@@ -2139,5 +2279,73 @@ tga2ASImage( const char * path, ASImageImportParams *params )
 	SHOW_TIME("image loading",started);
 	return im ;
 }
+/*************************************************************************/
+/* ARGB 																 */
+/*************************************************************************/
+ASImage *
+convert_argb2ASImage( ASVisual *asv, int width, int height, ARGB32 *argb, CARD8 *gamma_table )
+{
+	ASImage *im = NULL ;
+	ASImageOutput  *imout ;
+	im = create_asimage( width, height, 100 );
+	if((imout = start_image_output( NULL, im, ASA_ASImage, 0, ASIMAGE_QUALITY_DEFAULT)) == NULL )
+	{
+   		destroy_asimage( &im );
+		return NULL;
+	}else
+	{	
+		ASScanline    buf;
+		int y ;
+		prepare_scanline( im->width, 0, &buf, True );
+		for( y = 0 ; y < height ; ++y ) 
+		{	  
+			int x ;
+			for( x = 0 ; x < width ; ++x ) 
+			{
+				ARGB32 c = argb[x];
+				buf.alpha[x] 	= ARGB32_ALPHA8(c);	
+				buf.red[x] 	= ARGB32_RED8(c);	  
+				buf.green[x] 	= ARGB32_GREEN8(c);	  
+				buf.blue[x] 	= ARGB32_BLUE8(c);	  
+			}	 
+			argb += width ;			
+			set_flags( buf.flags, SCL_DO_RED|SCL_DO_GREEN|SCL_DO_BLUE|SCL_DO_ALPHA );
+			imout->output_image_scanline( imout, &buf, 1);
+		}
+		stop_image_output( &imout );
+		free_scanline( &buf, True );
+	}   
+						
+	return im ;	
+}
 
 
+ASImage *
+argb2ASImage( const char *path, ASImageImportParams *params )
+{
+	ASVisual fake_asv ;
+	long argb_data_len = -1; 
+	char *argb_data = NULL ;
+	ASImage *im = NULL ;
+
+	memset( &fake_asv, 0x00, sizeof(ASVisual) );
+
+	argb_data = load_binary_file(path, &argb_data_len);
+	if(argb_data == NULL || argb_data_len < 8 )
+		show_error( "unable to load file \"%s\" file is either too big or is not readable.\n", path );
+	else
+	{
+		int width = ((CARD32*)argb_data)[0] ;
+		int height = ((CARD32*)argb_data)[1] ;
+		if( 2 + width*height > argb_data_len/sizeof(CARD32))
+		{
+			show_error( "file \"%s\" is too small for specified image size of %dx%d.\n", path, width, height );
+		}else
+			im = convert_argb2ASImage( &fake_asv, width, height, (ARGB32*)argb_data+2, params->gamma_table );
+	}
+	if( argb_data ) 
+		free( argb_data );
+	
+	return im ;
+}
+
