@@ -187,21 +187,14 @@ int
 GetRootDimensions (int *width, int *height)
 {
 #ifndef X_DISPLAY_MISSING
-	Window root;
-	int w_x, w_y;
-	unsigned int junk;
-    if( dpy == NULL )
-        return 0;
-	if (!XGetGeometry (dpy, RootWindow(dpy,DefaultScreen(dpy)), &root,
-					     &w_x, &w_y, (unsigned int *)width, (unsigned int *)height, &junk, &junk))
-    {
-    	*width = 0;
-    	*height = 0;
-    }
-	return (*width > 0 && *height > 0) ? 1 : 0;
-#else
-	return 0;
+	if( dpy ) 
+	{	
+		*height = XDisplayHeight(dpy, DefaultScreen(dpy) );
+		*width = XDisplayWidth(dpy, DefaultScreen(dpy) );
+	}
+	return 1 ;
 #endif
+	return 0;
 }
 
 int
