@@ -24,9 +24,13 @@ struct wild_reg_exp;
 #define STYLE_LAYER             (1 << 8)   /* style->layer - layer is defined */
 #define STYLE_FRAME             (1 << 9)   /* style->frame - frame is defined */
 #define STYLE_WINDOWBOX         (1 << 10)  /* style->windowbox - windowbox is defined */
+#define STYLE_WINDOW_OPACITY    (1 << 11)  /* style->window_opacity - opacity of the window is defined */
 
 
 /* this are pure flags */
+#define STYLE_FULLSCREEN		(1 << 6)
+#define STYLE_KDE_HINTS         (1 << 7)    /* if set - then we should honor KDE Hints */
+#define STYLE_CURRENT_VIEWPORT  (1 << 8)   /* Ignore ConfigRequests for the client */   
 #define STYLE_IGNORE_CONFIG     (1 << 9)   /* Ignore ConfigRequests for the client */   
 #define STYLE_LONG_LIVING       (1 << 10)
 #define STYLE_STICKY            (1 << 11)
@@ -57,7 +61,7 @@ struct wild_reg_exp;
 #define STYLE_DEFAULTS		(STYLE_TITLE|STYLE_CIRCULATE|STYLE_WINLIST| \
                              STYLE_FOCUS|STYLE_FOCUS_ON_MAP|STYLE_HANDLES| \
 							 STYLE_ICON_TITLE|STYLE_PPOSITION|STYLE_GROUP_HINTS| \
-							 STYLE_TRANSIENT_HINTS|STYLE_MOTIF_HINTS| \
+							 STYLE_TRANSIENT_HINTS|STYLE_MOTIF_HINTS|STYLE_KDE_HINTS| \
 							 STYLE_GNOME_HINTS|STYLE_EXTWM_HINTS|STYLE_LONG_LIVING)
 
 /************************************************************************************
@@ -99,6 +103,8 @@ typedef struct name_list
   char *frame_name;
   char *windowbox_name;
 
+  int   window_opacity;
+
   unsigned long on_buttons;
   unsigned long off_buttons;
 }
@@ -129,8 +135,9 @@ typedef struct ASDatabaseRecord
   char *icon_file;
   char *frame_name;
   char *windowbox_name;
-  char *window_styles[BACK_STYLES];
+  int   window_opacity;
 
+  char *window_styles[BACK_STYLES];
   Bool  own_strings ;
 
 }ASDatabaseRecord;
