@@ -138,6 +138,7 @@ destroy_asfeel( ASFeel *feel, Bool reusable )
                     destroy_aswindow_box( &(feel->window_boxes[i]), True );
                 free( feel->window_boxes );
             }
+			destroy_string( &(feel->default_window_box_name ) );
         }
 		if( !reusable )
 			free(feel);
@@ -399,6 +400,24 @@ find_window_box( ASFeel *feel, const char *name )
 /*************************************************************************
  * Menus :
  *************************************************************************/
+void
+menu_data_destroy(ASHashableValue value, void *data)
+{
+    MenuData *md = data ;
+LOCAL_DEBUG_CALLER_OUT( "menu_data_destroy(\"%s\", %p)", (char*)value, data );
+    if( (char*)value )
+        free( (char*)value );
+    if( md )
+    {
+        if( md->magic == MAGIC_MENU_DATA )
+        {
+            if( md->name == (char*)value )
+                md->name = NULL ;
+			destroy_menu_data( &md ); 
+        }
+    }
+}
+
 void
 init_list_of_menus(ASHashTable **list, Bool force)
 {
