@@ -315,21 +315,27 @@ set_asbtn_block_layer( ASTile* tile, ASImageLayer *layer, unsigned int state, AS
 /* ASIcon :                                                         */
 /********************************************************************/
 static void
-free_asicon( ASTile* tile )
+free_asimage_tile( ASTile* tile )
 {
-    if (tile->data.icon)
+    if (tile->data.image.im)
     {
-        safe_asimage_destroy (tile->data.icon);
-        tile->data.icon = NULL ;
+        safe_asimage_destroy (tile->data.image.im);
+        tile->data.image.im = NULL ;
     }
     ASSetTileType(tile,AS_TileFreed);
 }
 
 static int
-set_asicon_layer( ASTile* tile, ASImageLayer *layer, unsigned int state, ASImage **scrap_images, int max_width, int max_height )
+set_asimage_layer( ASTile* tile, ASImageLayer *layer, unsigned int state, ASImage **scrap_images, int max_width, int max_height )
 {
     int dst_width, dst_height ;
-    ASImage *im = tile->data.icon ;
+    ASImage *im = tile->data.image.im ;
+	unsigned int slice_x_start = tile->data.image.slice_x_start;
+	unsigned int slice_x_end = tile->data.image.slice_x_end;
+	unsigned int slice_y_start = tile->data.image.slice_y_start;
+	unsigned int slice_y_end = tile->data.image.slice_y_end;
+	Bool do_slice_x = ((slice_x_start != 0 || slice_x_end != 0 )&&get_flags( tile->flags, AS_TileHResize ));
+	Bool do_slice_y = ((slice_y_start != 0 || slice_y_end != 0 )&&get_flags( tile->flags, AS_TileVResize ));
 
     if( im == NULL )
         return 0;
@@ -339,7 +345,7 @@ set_asicon_layer( ASTile* tile, ASImageLayer *layer, unsigned int state, ASImage
 
     /* if( ASTileResizeable( *tile ) ) */
     {
-        if( get_flags( tile->flags, AS_TileHScale ) )
+        if( get_flags( tile->flags, AS_TileHScale ) || do_slice_x )
             dst_width = max_width ;
 		else if( get_flags( tile->flags, AS_TileHResize ) && max_width < im->width )
 		{
@@ -351,7 +357,7 @@ set_asicon_layer( ASTile* tile, ASImageLayer *layer, unsigned int state, ASImage
 					layer->clip_x = ((int)im->width - max_width) ;
 			}
 		}
-        if( get_flags( tile->flags, AS_TileVScale ) )
+        if( get_flags( tile->flags, AS_TileVScale ) || do_slice_y )
             dst_height = max_height ;
         else if( get_flags( tile->flags, AS_TileVResize ) && max_height < im->height )
 		{
@@ -367,16 +373,21 @@ set_asicon_layer( ASTile* tile, ASImageLayer *layer, unsigned int state, ASImage
 	LOCAL_DEBUG_OUT( "flags = %lX, dst_size = %dx%d, im_size = %dx%d, max_size = %dx%d, clip = %+d%+d", tile->flags, dst_width, dst_height, im->width, im->height, max_width, max_height, layer->clip_x, layer->clip_y );
     if( im->width != dst_width || im->height != dst_height )
     {
-        im = scale_asimage( ASDefaultVisual, im, dst_width, dst_height, ASA_ASImage, 0, ASIMAGE_QUALITY_DEFAULT );
+        if( do_slice_x || do_slice_y ) 
+			im = slice_asimage2( ASDefaultVisual, im, slice_x_start, slice_x_end, slice_y_start, slice_y_end, dst_width, dst_height, 
+								 get_flags( tile->flags, AS_TileHScale|AS_TileVScale ), ASA_ASImage, 0, ASIMAGE_QUALITY_DEFAULT );
+		else
+			im = scale_asimage( ASDefaultVisual, im, dst_width, dst_height, ASA_ASImage, 0, ASIMAGE_QUALITY_DEFAULT );
+
         if( im == NULL )
-            im = tile->data.icon ;
+            im = tile->data.image.im ;
         else
             *scrap_images = im ;
     }
 
     layer->im = im;
-    layer->dst_x = tile->x;
-    layer->dst_y = tile->y ;
+    layer->dst_x = tile->x - layer->clip_x;
+    layer->dst_y = tile->y - layer->clip_y;
     if( ASTileHResizeable(*tile) )
         layer->clip_width  = max_width ;
     else
@@ -532,7 +543,7 @@ struct
 {
 {"Spacer",  NULL, NULL, NULL},
 {"Buttons", free_asbtn_block, check_btn_point, NULL,                  set_asbtn_block_layer},
-{"Icon",    free_asicon,      NULL,            NULL,                  set_asicon_layer},
+{"Image",    free_asimage_tile, NULL,            NULL,                set_asimage_layer},
 {"Label",   free_aslabel,     NULL,            aslabel_style_changed, set_aslabel_layer},
 {"none",    NULL, NULL, NULL},
 {"none",    NULL, NULL, NULL},
@@ -625,6 +636,9 @@ calculate_astbar_height (ASTBarData * tbar)
     if (tbar)
     {
         register int i = tbar->tiles_num ;
+#if	defined(LOCAL_DEBUG) && !defined(NO_DEBUG_OUTPUT)
+         print_astbar_tiles(tbar); 
+#endif
         while( --i >= 0 )
             if( ASTileType(tbar->tiles[i]) != AS_TileFreed &&
                 !ASTileIgnoreHeight(tbar->tiles[i]))
@@ -732,14 +746,14 @@ update_astbar_bevel_size (ASTBarData * tbar)
 		if (tbar->style[i])
 		{
             mystyle_make_bevel (tbar->style[i], &bevel, tbar->hilite[i], False);
-			if (tbar->left_bevel < bevel.left_outline)
-				tbar->left_bevel = bevel.left_outline;
-			if (tbar->top_bevel < bevel.top_outline)
-				tbar->top_bevel = bevel.top_outline;
-			if (tbar->right_bevel < bevel.right_outline)
-				tbar->right_bevel = bevel.right_outline;
-			if (tbar->bottom_bevel < bevel.bottom_outline)
-				tbar->bottom_bevel = bevel.bottom_outline;
+			if (tbar->left_bevel < bevel.left_outline + bevel.left_inline)
+				tbar->left_bevel = bevel.left_outline + bevel.left_inline;
+			if (tbar->top_bevel < bevel.top_outline + bevel.top_inline)
+				tbar->top_bevel = bevel.top_outline + bevel.top_inline;
+			if (tbar->right_bevel < bevel.right_outline + bevel.right_inline)
+				tbar->right_bevel = bevel.right_outline + bevel.right_inline;
+			if (tbar->bottom_bevel < bevel.bottom_outline + bevel.bottom_inline)
+				tbar->bottom_bevel = bevel.bottom_outline + bevel.bottom_inline;
 		}
 }
 
@@ -861,7 +875,36 @@ set_astbar_style (ASTBarData * tbar, unsigned int state, const char *style_name)
     return changed;
 }
 
-static ASTile *
+Bool
+invalidate_astbar_style (ASTBarData * tbar, int state)
+{
+	Bool          changed = False;
+
+	if( tbar && state < BAR_STATE_NUM )
+	{
+		if( state < 0 ) 
+		{
+			int i ;
+			for( i = 0 ; i < BAR_STATE_NUM ; ++i ) 
+				if( tbar->back[i] )
+					changed = invalidate_astbar_style (tbar, i);
+		}else if( tbar->style[state] != NULL )
+		{
+			changed = True;
+			tbar->style[state] = NULL;
+			if( tbar->back[state] )
+			{
+				destroy_asimage (&(tbar->back[state]));
+				tbar->back[state] = NULL;
+			}
+			set_flags( tbar->state, BAR_FLAGS_REND_PENDING );
+		}
+	}
+    return changed;
+}
+
+
+static int
 add_astbar_tile( ASTBarData *tbar, int type, unsigned char col, unsigned char row, int flip, int align )
 {
     int new_idx = -1;
@@ -934,18 +977,6 @@ add_astbar_tile( ASTBarData *tbar, int type, unsigned char col, unsigned char ro
 			set_flags( align_flags, FIT_LABEL_WIDTH );
 	}	 
 
-#if 0
-    if( get_flags( flip, FLIP_VERTICAL ) )
-        align_flags = (((align&PAD_H_MASK)>>PAD_H_OFFSET)<<PAD_V_OFFSET)|
-                      (((align&PAD_V_MASK)>>PAD_V_OFFSET)<<PAD_H_OFFSET)|
-                      (((align&RESIZE_H_MASK)>>RESIZE_H_OFFSET)<<RESIZE_V_OFFSET)|
-                      (((align&RESIZE_V_MASK)>>RESIZE_V_OFFSET)<<RESIZE_H_OFFSET) ;
-
-    if( get_flags( flip, FLIP_UPSIDEDOWN ) )
-        align_flags = ((align_flags&0x0005)<<1)|((align_flags&(0x0005<<1))>>1)|(align_flags&RESIZE_MASK);
-#endif
-
-
 	LOCAL_DEBUG_CALLER_OUT( "type = %d, flip = %d, align = 0x%X, align_flags = 0x%lX", type, flip, align, align_flags );
     align_flags &= (PAD_MASK|RESIZE_MASK|FIT_LABEL_SIZE);
 
@@ -956,7 +987,7 @@ add_astbar_tile( ASTBarData *tbar, int type, unsigned char col, unsigned char ro
                                  ((flip<<AS_TileFlipOffset)&AS_TileFlipMask)|
                                  ((align_flags<<AS_TileFloatingOffset));
     set_flags( tbar->state, BAR_FLAGS_REND_PENDING );
-    return &(tbar->tiles[new_idx]);
+    return new_idx;
 }
 
 Bool
@@ -990,10 +1021,11 @@ add_astbar_spacer( ASTBarData *tbar, unsigned char col, unsigned char row, int f
 {
     if( tbar )
     {
-        ASTile *tile = add_astbar_tile( tbar, AS_TileSpacer, col, row, flip, align );
+		int idx = add_astbar_tile( tbar, AS_TileSpacer, col, row, flip, align );
+        ASTile *tile = &(tbar->tiles[idx]) ;
         tile->width = width ;
         tile->height = height ;
-        return tbar->tiles_num-1;
+        return idx;
     }
     return -1;
 }
@@ -1005,7 +1037,9 @@ add_astbar_btnblock( ASTBarData * tbar, unsigned char col, unsigned char row, in
 {
     if( tbar )
     {
-        ASTile *tile = add_astbar_tile( tbar, AS_TileBtnBlock, col, row, flip, align );
+        int idx = add_astbar_tile( tbar, AS_TileBtnBlock, col, row, flip, align );
+        ASTile *tile = &(tbar->tiles[idx]) ;
+
 /*      if( get_flags( flip, FLIP_VERTICAL ) )
         {
             int tmp = top_margin ; 
@@ -1015,7 +1049,7 @@ add_astbar_btnblock( ASTBarData * tbar, unsigned char col, unsigned char row, in
  */ 
         build_btn_block( tile, from_list, context_mask, count, left_margin, top_margin,
                          spacing, order );
-        return tbar->tiles_num-1;
+        return idx;
     }
     return -1;
 }
@@ -1025,11 +1059,13 @@ add_astbar_icon( ASTBarData * tbar, unsigned char col, unsigned char row, int fl
 {
     if( tbar && icon )
     {
-        ASTile *tile = add_astbar_tile( tbar, AS_TileIcon, col, row, flip, align );
+        int idx = add_astbar_tile( tbar, AS_TileIcon, col, row, flip, align );
+		ASTile *tile = &(tbar->tiles[idx]) ;
+
         if( flip == 0 )
         {
-            if( (tile->data.icon = dup_asimage( icon )) == NULL )
-                    tile->data.icon = clone_asimage( icon, 0xFFFFFFFF );
+            if( (tile->data.image.im = dup_asimage( icon )) == NULL )
+                    tile->data.image.im = clone_asimage( icon, 0xFFFFFFFF );
         }else
 		{
 			int dst_width = icon->width, dst_height = icon->height ;
@@ -1038,23 +1074,46 @@ add_astbar_icon( ASTBarData * tbar, unsigned char col, unsigned char row, int fl
 				dst_width = icon->height ;
 				dst_height = icon->width ;
 			}
-            tile->data.icon = flip_asimage( ASDefaultVisual, icon, 0, 0, dst_width, dst_height, flip, ASA_ASImage, 100, ASIMAGE_QUALITY_DEFAULT );
+            tile->data.image.im = flip_asimage( ASDefaultVisual, icon, 0, 0, dst_width, dst_height, flip, ASA_ASImage, 100, ASIMAGE_QUALITY_DEFAULT );
 		}
-        tile->width = tile->data.icon->width;
-        tile->height = tile->data.icon->height;
+        tile->width = tile->data.image.im->width;
+        tile->height = tile->data.image.im->height;
         ASSetTileSublayers(*tile,1);
-        return tbar->tiles_num-1;
+        return idx;
     }
     return -1;
 }
 
+int
+add_astbar_image( ASTBarData * tbar, unsigned char col, unsigned char row, int flip, int align, ASImage *im, 
+				  unsigned short slice_x_start, unsigned short slice_x_end,
+				  unsigned short slice_y_start, unsigned short slice_y_end)
+{
+    int idx = add_astbar_icon( tbar, col, row, flip, align, im);
+	if( idx >= 0 ) 
+	{	
+		ASImageTile *imtile = &(tbar->tiles[idx].data.image) ;
+		imtile->slice_x_start = slice_x_start;
+		imtile->slice_x_end = slice_x_end;
+		imtile->slice_y_start = slice_y_start;
+		imtile->slice_y_end = slice_y_end;
+        
+		return idx;
+	}
+    return -1;
+}
+
+
 int
 add_astbar_label( ASTBarData * tbar, unsigned char col, unsigned char row, int flip, int align, short h_padding, short v_padding, const char *text, unsigned long encoding)
 {
 LOCAL_DEBUG_CALLER_OUT( "encoding = %ld, label \"%s\"", encoding, text?text:"null" );
     if( tbar )
     {
-        ASTile *tile = add_astbar_tile( tbar, AS_TileLabel, col, row, flip, align );
+		
+        int idx = add_astbar_tile( tbar, AS_TileLabel, col, row, flip, align );
+		ASTile *tile = &(tbar->tiles[idx]) ;
+
         ASLabel *lbl = &(tile->data.label);
         lbl->text = mystrdup(text);
 		lbl->encoding = encoding ;
@@ -1063,7 +1122,7 @@ LOCAL_DEBUG_CALLER_OUT( "encoding = %ld, label \"%s\"", encoding, text?text:"nul
         set_astile_styles( tbar, tile, -1 );
 
         ASSetTileSublayers(*tile,1);
-        return tbar->tiles_num-1;
+        return idx;
     }
     return -1;
 }
@@ -1077,8 +1136,10 @@ LOCAL_DEBUG_CALLER_OUT( "tbar(%p)->index(%d)->label(ENC = %ld, TXT=\"%s\")", tba
     if (tbar && tbar->tiles)
 	{
         ASLabel *lbl = &(tbar->tiles[index].data.label);
-        if( ASTileType(tbar->tiles[index]) != AS_TileLabel )
+		if( ASTileType(tbar->tiles[index]) != AS_TileLabel )
             return False;
+		LOCAL_DEBUG_OUT( "old_label(ENC = %ld, TXT=\"%s\" )", lbl->encoding, lbl->text?lbl->text:"null" );
+        
 
         if (label == NULL)
 		{
@@ -1108,7 +1169,6 @@ LOCAL_DEBUG_CALLER_OUT( "tbar(%p)->index(%d)->label(ENC = %ld, TXT=\"%s\")", tba
 	return changed;
 }
 
-
 Bool
 change_astbar_first_label (ASTBarData * tbar, const char *label, unsigned long encoding)
 {
@@ -1191,7 +1251,10 @@ set_astbar_focused (ASTBarData * tbar, ASCanvas * pc, Bool focused)
 			set_flags (tbar->state, BAR_STATE_FOCUSED);
 		else
 			clear_flags (tbar->state, BAR_STATE_FOCUSED);
-        if( old_focused != new_focused && pc != NULL )
+
+        if( old_focused != new_focused )
+            set_flags(tbar->state, BAR_FLAGS_REND_PENDING);
+        if( get_flags(tbar->state, BAR_FLAGS_REND_PENDING) && pc != NULL )
             render_astbar( tbar, pc );
         return (new_focused != old_focused);
 	}
@@ -1296,6 +1359,37 @@ is_astbar_shaped( ASTBarData *tbar, int state )
     return shaped;
 }
 
+
+static inline int 
+trim_astbar_grid_dim( short *dim, int size, int space_left )
+{
+	int l = 0 ; 
+	int changed = 0 ;
+	while( space_left < -1 )     /* ohh, no, we need to trim rows to fit */
+	{
+		if( dim[l] > 0 ) 		
+		{
+			int to_trim = (-space_left*dim[l])/size ; 
+			if( dim[l] < to_trim )
+				to_trim = dim[l] - 1 ;
+			if( to_trim == 0 ) 
+				to_trim = 1 ;
+			dim[l] -= to_trim;
+			space_left  += to_trim;
+			++changed ;
+		}	 
+		if( ++l >= AS_TileRows )
+		{	
+			if( changed == 0 ) 
+				break;	  
+			l = 0 ;
+			changed = 0;
+		}
+	}	 
+	return space_left ;
+}
+
+
 #ifdef TRACE_render_astbar
 #undef render_astbar
 Bool render_astbar (ASTBarData * tbar, ASCanvas * pc);
@@ -1322,7 +1416,7 @@ render_astbar (ASTBarData * tbar, ASCanvas * pc)
 	ASImageBevel  bevel;
     ASImageLayer *layers;
     ASImage     **scrap_images = NULL;
-	ASImage      *merged_im;
+	ASImage      *merged_im = NULL ;
 	int           state;
 	ASAltImFormats fmt = ASA_ScratchXImageAndAlpha;
     int l ;
@@ -1403,8 +1497,11 @@ LOCAL_DEBUG_OUT("back-try2(%p)", back );
             good_layers += ASTileSublayers(tbar->tiles[l]);
 			if( !ASTileIgnoreWidth(tbar->tiles[l]) )
 			{
-            	if( col_width[pos] < tbar->tiles[l].width )
-                	col_width[pos] = tbar->tiles[l].width;
+				int w = tbar->tiles[l].width  ;
+				if( w > tbar->width ) 
+					w = tbar->width ;
+            	if( col_width[pos] < w )
+                	col_width[pos] = w;
                 /* floating column has at least one element padded/resizable in horizontal dir,
                  * and no fixed elements */
                 if( !ASTileHFloating(tbar->tiles[l]) )
@@ -1416,8 +1513,11 @@ LOCAL_DEBUG_OUT("back-try2(%p)", back );
             pos = ASTileRow(tbar->tiles[l]);
 			if( !ASTileIgnoreHeight(tbar->tiles[l]) )
 			{
-            	if( row_height[pos] < tbar->tiles[l].height )
-                	row_height[pos] = tbar->tiles[l].height;
+				int h = tbar->tiles[l].height  ;
+				if( h > tbar->height ) 
+					h = tbar->height ;
+            	if( row_height[pos] < h )
+                	row_height[pos] = h;
                 /* floating row has at least one element padded/resizable in vertical dir,
                  * and no fixed elements */
                 if( !ASTileVFloating(tbar->tiles[l]) )
@@ -1427,8 +1527,10 @@ LOCAL_DEBUG_OUT("back-try2(%p)", back );
             }
         }
     /* pass 2: see how much space we have left that needs to be floating to some rows/columns : */
-    space_left_x = tbar->width - (h_bevel_size+tbar->h_border*2+bevel.left_inline+bevel.right_inline);
-    space_left_y = tbar->height- (v_bevel_size+tbar->v_border*2+bevel.top_inline+bevel.bottom_inline);
+    LOCAL_DEBUG_OUT( "bar_size = %dx%d", tbar->width, tbar->height );
+
+    space_left_x = tbar->width -(h_bevel_size+tbar->h_border*2+bevel.left_inline+bevel.right_inline);
+    space_left_y = tbar->height-(v_bevel_size+tbar->v_border*2+bevel.top_inline+bevel.bottom_inline);
     LOCAL_DEBUG_OUT( "from: space_left_x = %d, space_left_y = %d", space_left_x, space_left_y );
     for( l = 0 ; l < AS_TileColumns ; ++l )
     {
@@ -1471,6 +1573,9 @@ LOCAL_DEBUG_OUT("back-try2(%p)", back );
                     break;
             }
         }
+	if( space_left_x < -1 )
+		space_left_x = trim_astbar_grid_dim( &(col_width[0]), tbar->width, space_left_x );
+
     /* pass 4: now we determine spread padding among affected rows : */
     if( floating_rows_count > 0 && space_left_y != 0)
         for( l = 0 ; l < AS_TileRows ; ++l )
@@ -1492,10 +1597,13 @@ LOCAL_DEBUG_OUT("back-try2(%p)", back );
                     break;
             }
         }
+	if( space_left_y < -1 )
+		space_left_y = trim_astbar_grid_dim( &(row_height[0]), tbar->height, space_left_y );
+
 
     /* pass 5: now we determine offset of each row/column : */
     x = bevel.left_outline+bevel.left_inline+tbar->h_border ;
-    y = bevel.top_outline +bevel.top_inline+tbar->v_border ;
+    y = bevel.top_outline+bevel.top_inline+tbar->v_border ;
     for( l = 0 ; l < AS_TileColumns ; ++l )
     {
         col_x[l] = x ;
@@ -1659,8 +1767,8 @@ LOCAL_DEBUG_OUT( "bar's geometry = %dx%d%+d%+d, pointer posish = %+d%+d", tbar->
 void
 on_astbar_pointer_action( ASTBarData *tbar, int context, Bool leave, Bool pointer_moved )
 {
-    LOCAL_DEBUG_CALLER_OUT( "%p, %s, %d", tbar, context2text(context), leave );
 	static ASBalloon *last_balloon = NULL ;
+    LOCAL_DEBUG_CALLER_OUT( "%p, %s, %d", tbar, context2text(context), leave );
     
 	if( tbar == NULL )
     {
