@@ -554,24 +554,32 @@ purge_menu_data_items(MenuData *md)
 }
 
 void
-menu_data_destroy(ASHashableValue value, void *data)
+destroy_menu_data( MenuData **pmd ) 
 {
-    MenuData *md = data ;
-LOCAL_DEBUG_CALLER_OUT( "menu_data_destroy(\"%s\", %p)", (char*)value, data );
-    if( (char*)value )
-        free( (char*)value );
-    if( md )
-    {
-        if( md->magic == MAGIC_MENU_DATA )
-        {
-            if( md->name != (char*)value )
+	if( pmd && *pmd) 
+	{
+	    MenuData *md = *pmd ;
+		if( md->magic == MAGIC_MENU_DATA )
+		{	
+LOCAL_DEBUG_CALLER_OUT( "menu_data_destroy(\"%s\", %p)", md->name?md->name:"(null)", md );
+       		if( md->name )
                 free( md->name );
 
             purge_menu_data_items(md);
             md->magic = 0 ;
-        }
-        free(data);
-    }
+        	free(md);
+			*pmd = NULL ; 
+		}
+	}				 
+}
+
+MenuData *
+create_menu_data( char *name ) 
+{	
+    MenuData *md = (MenuData*) safecalloc (1, sizeof(MenuData));
+    md->name = mystrdup(name);
+    md->magic = MAGIC_MENU_DATA ;
+	return md;
 }
 
 MenuData    *
@@ -587,16 +595,10 @@ new_menu_data( ASHashTable *list, char *name )
     if( get_hash_item( list, AS_HASHABLE(name), &hdata.vptr) == ASH_Success )
         return (MenuData*)hdata.vptr;
 
-    md = (MenuData*) safecalloc (1, sizeof(MenuData));
-    md->name = mystrdup(name);
-    md->magic = MAGIC_MENU_DATA ;
-	md->recent_items = ASDefaultScr->Feel.recent_submenu_items ;
+    md = create_menu_data( name ); 
 
     if( add_hash_item( list, AS_HASHABLE(md->name), md) != ASH_Success )
-    {
-        menu_data_destroy( AS_HASHABLE(md->name), md );
-        md = NULL ;
-    }
+        destroy_menu_data( &md );
     return md;
 }
 
@@ -643,25 +645,49 @@ assign_minipixmap( MenuDataItem *mdi, char *minipixmap )
     }
 }
 
+Bool 
+check_fdata_availability( FunctionData *fdata )
+{
+	if( fdata == NULL ) 
+		return False ;
+    if ( fdata->func < F_ExecToolStart || fdata->func > F_ExecToolEnd )
+	{	
+	 	if(IsSwallowFunc(fdata->func) || IsExecFunc(fdata->func) )
+    	{
+			LOCAL_DEBUG_OUT( "now really checking availability for \"%s\"", fdata->name?fdata->name:"nameless" );
+        	if (!is_executable_in_path (fdata->text))
+        	{
+				LOCAL_DEBUG_OUT( "unavailable :  \"%s\"", fdata->name?fdata->name:"nameless" );
+				return False ;
+        	}
+    	}else if( fdata->func == F_CHANGE_BACKGROUND_FOREIGN )
+		{
+			ASImageFileTypes type = check_asimage_file_type( fdata->text );
+			LOCAL_DEBUG_OUT( "foreign back image \"%s\" has type %d", fdata->text, type );
+			if( type > ASIT_Supported ) 
+        	{
+				LOCAL_DEBUG_OUT( "foreign back image of unknown type :  \"%s\"", fdata->text );
+				return False;
+        	}
+		}
+	}
+	return True;
+}
+
+
+
 static void
 check_availability( MenuDataItem *mdi )
 {
     clear_flags( mdi->flags, MD_Disabled );
 #ifndef NO_AVAILABILITYCHECK
-LOCAL_DEBUG_OUT( "checking availability for \"%s\"", mdi->fdata->name?mdi->fdata->name:"nameless" );
-    if ( mdi->fdata->func!= F_ExecInTerm && (IsSwallowFunc(mdi->fdata->func) || IsExecFunc(mdi->fdata->func)) )
-    {
-LOCAL_DEBUG_OUT( "now really checking availability for \"%s\"", mdi->fdata->name?mdi->fdata->name:"nameless" );
-        if (!is_executable_in_path (mdi->fdata->text))
-        {
-LOCAL_DEBUG_OUT( "unavailable :  \"%s\"", mdi->fdata->name?mdi->fdata->name:"nameless" );
-			set_flags( mdi->flags, MD_Disabled );
-        }
-    }
+	LOCAL_DEBUG_OUT( "checking availability for \"%s\"", mdi->fdata->name?mdi->fdata->name:"nameless" );
+	check_fdata_availability( mdi->fdata );
 #endif /* NO_AVAILABILITYCHECK */
 }
 
-void
+
+MenuDataItem *
 add_menu_data_item( MenuData *menu, int func, char *name, char *minipixmap )
 {
     MenuDataItem *mdi = new_menu_data_item(menu);
@@ -673,12 +699,13 @@ add_menu_data_item( MenuData *menu, int func, char *name, char *minipixmap )
         check_availability( mdi );
         assign_minipixmap( mdi, minipixmap );
     }
+	return mdi;
 }
 
-void
+MenuDataItem *
 add_menu_fdata_item( MenuData *menu, FunctionData *fdata, char *minipixmap, struct ASImage *img )
 {
-    MenuDataItem *mdi ;
+    MenuDataItem *mdi = NULL;
 
     if( fdata )
         if( (mdi = new_menu_data_item(menu)) != NULL )
@@ -696,10 +723,11 @@ add_menu_fdata_item( MenuData *menu, FunctionData *fdata, char *minipixmap, stru
 			LOCAL_DEBUG_OUT( "mdi_fdata_encoding = %d, fdata_encoding = %d",
 			mdi->fdata->name_encoding, fdata->name_encoding );
         }
+	return mdi;
 }
 
 ASImage *
-check_scale_menu_pmap( ASImage *im ) 
+check_scale_menu_pmap( ASImage *im, ASFlagType flags ) 
 {	
     if( im )
     {
@@ -721,6 +749,26 @@ check_scale_menu_pmap( ASImage *im )
             if( w == 0 )
                 w = 1 ;
         }
+		if( get_flags( flags, MD_ScaleMinipixmapDown ) )
+		{
+			int tmp_h = asxml_var_get("menu.font.size")+8;				
+			int tmp_w = (w*tmp_h)/h ; 
+			if( w  > tmp_w || h > tmp_h ) 
+			{
+				w = tmp_w ; 
+				h = tmp_h ; 	   
+			}	 
+		}else if( get_flags( flags, MD_ScaleMinipixmapUp ) )
+		{
+			int tmp_w = asxml_var_get("minipixmap.width");				   
+			int tmp_h = asxml_var_get("minipixmap.height") ; 
+			if( w  > tmp_w || h > tmp_h ) 
+			{
+				w = tmp_w ; 
+				h = tmp_h ; 	   
+			}	 
+		}
+
         if( w != im->width || h != im->height )
         {
 			return scale_asimage( ASDefaultVisual, im, w, h, ASA_ASImage, 100, ASIMAGE_QUALITY_DEFAULT );
@@ -733,31 +781,48 @@ void
 reload_menu_pmaps( MenuData *menu )
 {
     MenuDataItem *curr ;
+	char *scaled_name = NULL ;
+	int scaled_name_len = 0 ; 
 	
 	LOCAL_DEBUG_OUT( "menu = %p, image_manager = %p", menu, ASDefaultScr->image_manager );
 
 	if( menu && ASDefaultScr->image_manager ) 
 		for( curr = menu->first ; curr != NULL ; curr = curr->next )
 		{	
-    		if( curr->minipixmap )
+			ASImage *tmp = NULL ;
+			char *minipixmap = curr->minipixmap ;
+            if( curr->minipixmap_image )
+                safe_asimage_destroy(curr->minipixmap_image);
+			
+			if( minipixmap == NULL && curr->fdata->func == F_CHANGE_BACKGROUND_FOREIGN ) 
+				minipixmap = curr->fdata->text;
+    		
+			if( minipixmap )
+            	tmp = get_asimage( ASDefaultScr->image_manager, minipixmap, ASFLAGS_EVERYTHING, 100 );
+			
+			if( tmp )				   
         	{
-				ASImage *tmp ;
-            	if( curr->minipixmap_image )
-                	safe_asimage_destroy(curr->minipixmap_image);
-            	tmp = get_asimage( ASDefaultScr->image_manager, curr->minipixmap, ASFLAGS_EVERYTHING, 100 );
-				curr->minipixmap_image = check_scale_menu_pmap( tmp ); 
+				curr->minipixmap_image = check_scale_menu_pmap( tmp, curr->flags ); 
 				if( tmp != curr->minipixmap_image )
 				{	
-					char *n ;
+					int len = strlen( minipixmap ) + 64 ;
 					safe_asimage_destroy(tmp);
 					/* we also need to add our icon into the image_manager ! : */
-					n = safemalloc( strlen( curr->minipixmap ) + 64 );
-					sprintf( n, "%s_scaled_to_%dx%d",curr->minipixmap, curr->minipixmap_image->width, curr->minipixmap_image->height );
-					store_asimage( ASDefaultScr->image_manager, curr->minipixmap_image, n );					 
+					if( len > scaled_name_len ) 
+					{
+						scaled_name = saferealloc( scaled_name, len );
+						scaled_name_len = len ;
+					}
+					sprintf( scaled_name, "%s_scaled_to_%dx%d", minipixmap, curr->minipixmap_image->width, curr->minipixmap_image->height );
+					store_asimage( ASDefaultScr->image_manager, curr->minipixmap_image, scaled_name );					 
 				}
-        	}
-			LOCAL_DEBUG_OUT( "minipixmap = \"%s\", minipixmap_image = %p",  curr->minipixmap, curr->minipixmap_image );
+        	}else
+			{	
+				LOCAL_DEBUG_OUT( "minipixmap = \"%s\", minipixmap_image = %p",  curr->minipixmap, curr->minipixmap_image );
+			}
 		}
+	if( scaled_name ) 
+		free( scaled_name );
 }
 
 
@@ -797,36 +862,56 @@ parse_menu_item_name (MenuDataItem * item, char **name)
 	return 0;
 }
 
-void
-menu_data_item_from_func (MenuData * menu, FunctionData * fdata)
+MenuDataItem     *
+menu_data_item_from_func (MenuData * menu, FunctionData * fdata, Bool do_check_availability)
 {
     MenuDataItem     *item = NULL;
 
-    if( fdata == NULL )
-        return ;
-    if (fdata->func == F_MINIPIXMAP)
-	{
-		if (menu->last)
-			item = menu->last;
-		else
+    if( fdata != NULL )
+	{	
+    	if (IsMinipixmapFunc(fdata->func) )
 		{
-            item = new_menu_data_item (menu);
-			item->fdata = fdata;
+			if (menu->last)
+				item = menu->last;
+			else
+			{
+            	item = new_menu_data_item (menu);
+				item->fdata = fdata;
+			}
+			if( fdata->func == F_SMALL_MINIPIXMAP ) 
+				set_flags( item->flags, MD_ScaleMinipixmapDown );
+			else if( fdata->func == F_LARGE_MINIPIXMAP ) 
+				set_flags( item->flags, MD_ScaleMinipixmapUp );
+		   
+        	item->minipixmap = mystrdup(fdata->name);
+    	} else
+    	{
+        	item = new_menu_data_item(menu);
+			if (parse_menu_item_name (item, &(fdata->name)) >= 0)
+				item->fdata = fdata;
+			if( do_check_availability )
+				check_availability(item);
+		}
+		if( item == NULL || item->fdata != fdata )
+		{
+			free_func_data (fdata);					   /* insurance measure */
+			free( fdata );
 		}
-        item->minipixmap = mystrdup(fdata->name);
-    } else
-    {
-        item = new_menu_data_item(menu);
-		if (parse_menu_item_name (item, &(fdata->name)) >= 0)
-			item->fdata = fdata;
-	}
-	if( item == NULL || item->fdata != fdata )
-	{
-		free_func_data (fdata);					   /* insurance measure */
-		free( fdata );
 	}
+	return item ;
 }
 
+int
+compare_func_data_name(const void *a, const void *b) 
+{
+	FunctionData *fda = *(FunctionData **)a;
+	FunctionData *fdb = *(FunctionData **)b;
+
+	return strcmp(fda->name ? fda->name : "", fdb->name ? fdb->name : "");
+}
+
+
+
 void
 print_func_data(const char *file, const char *func, int line, FunctionData *data)
 {
