@@ -13,6 +13,8 @@ struct MwmHints;
 struct ASDatabaseRecord;
 struct ASVector;
 struct ScreenInfo;
+struct ASFeel;
+struct ASImage;
 /***********************************************************************/
 /*       AfterStep structure to hold summary of all the hints :        */
 /***********************************************************************/
@@ -57,6 +59,8 @@ struct ScreenInfo;
 #define AS_ShapedIcon           (1<<21)        /* icon decorations use shaped MyStyle */
 #define AS_UnMapPending         (1<<22)
 #define AS_MoveresizeInProgress (1<<23)
+#define AS_Fullscreen			(1<<24)
+#define AS_Urgent				(1<<25)
 
 
 /***********************************************************/
@@ -81,22 +85,21 @@ struct ScreenInfo;
 #define AS_AvoidCover           (1<<17)
 #define AS_IconTitle            (1<<18)
 #define AS_Icon                 (1<<19)
-#define AS_ClientIcon           (1<<20)
-#define AS_ClientIconPixmap     (1<<21)
-#define AS_ClientIconPosition   (1<<22)
-#define AS_Windowbox            (1<<23)
-#define AS_FocusOnMap           (1<<24)
-#define AS_ShortLived           (1<<25)
-#define AS_Module				(1<<26)
-#define AS_IgnoreConfigRequest  (1<<27)
-#define AS_WMDockApp			(1<<28)  /* res_class == "DockApp" and main 
+#define AS_Windowbox            (1<<20)
+#define AS_FocusOnMap           (1<<21)
+#define AS_ShortLived           (1<<22)
+#define AS_Module				(1<<23)
+#define AS_IgnoreConfigRequest  (1<<24)
+#define AS_WMDockApp			(1<<25)  /* res_class == "DockApp" and main 
 										  * window is 1x1 (just don't ask why)
 										  * usually that means that icon 
 										  * window should be animated */ 
-
+#define AS_UseCurrentViewport  	(1<<26)
+#define AS_WindowOpacity	   	(1<<27)
 
 #define NOLOOK_HINT_FLAGS	(AS_IgnoreConfigRequest|AS_Module|AS_ShortLived| \
-							 AS_AvoidCover|AS_AcceptsFocus|AS_ClickToFocus)
+							 AS_AvoidCover|AS_AcceptsFocus|AS_ClickToFocus| \
+							 AS_UseCurrentViewport)
 
 
 /***********************************************************/
@@ -105,6 +108,7 @@ struct ScreenInfo;
 #define AS_DoesWmDeleteWindow 	(1<<1)
 #define AS_DoesWmPing           (1<<2)
 #define AS_NeedsVisibleName		(1<<3)  /* only if window has _NET_WM_NAME hint */
+#define AS_DoesKIPC				(1<<4)
 
 /***********************************************************/
 /* AS function masks :                                     */
@@ -152,8 +156,15 @@ typedef struct ASHints
   ASFlagType protocols ;
   ASFlagType function_mask ;
 
+#define AS_ClientIcon           (1<<0)
+#define AS_ClientIconPixmap     (1<<1)
+#define AS_ClientIconPosition   (1<<2)
+#define AS_ClientIconARGB		(1<<3)
+
+  ASFlagType client_icon_flags ;
   union { Window window; Pixmap pixmap; } icon ;
   Pixmap icon_mask ;
+  CARD32 *icon_argb ; 
   int icon_x, icon_y ;
   char *icon_file ;
 
@@ -185,6 +196,7 @@ typedef struct ASHints
 
   char *client_host ;    /* hostname of the computer on which client was executed */
   char *client_cmd  ;    /* preparsed command line of the client */
+  CARD32 window_opacity ;
 }
 ASHints;
 
@@ -208,7 +220,7 @@ typedef struct ASStatusHints
     unsigned int    width, height;
     unsigned int    border_width ; /* this border width is needed only to calculate
 								    * reference point when we are starting up */
-    unsigned int    viewport_x, viewport_y;
+    int    			viewport_x, viewport_y;
     int             desktop ;
 	int 			layer ;
 
@@ -250,6 +262,7 @@ typedef struct ASSupportedHints
  default :                                 break ; /* North/West */    \
 }}
 
+unsigned char get_hint_name_encoding( ASHints *hints, int name_idx );
 ASHints *merge_hints( struct ASRawHints *raw, struct ASDatabase *db, ASStatusHints *status,
                       ASSupportedHints *list, ASFlagType what, ASHints* reusable_memory, Window client );
 void merge_asdb_hints (ASHints * clean, struct ASRawHints * raw, struct ASDatabaseRecord * db_rec, ASStatusHints * status, ASFlagType what);
@@ -278,23 +291,26 @@ Bool compare_names( ASHints *old, ASHints *hints );
 ASFlagType compare_hints( ASHints *old, ASHints *hints );
 ASFlagType function2mask( int function );
 void constrain_size ( ASHints *hints, ASStatusHints *status,
-                 	  unsigned int max_width, unsigned int max_height );
+                 	  int max_width, int max_height );
 void get_gravity_offsets (ASHints *hints, int *xp, int *yp);
 int translate_asgeometry( struct ScreenInfo *scr, ASGeometry *asg, int *px, int *py, unsigned int *pwidth, unsigned int *pheight );
 void real2virtual (ASStatusHints *status, int *x, int *y, int vx, int vy );
 void virtual2real (ASStatusHints *status, int *x, int *y, int vx, int vy);
-int make_anchor_pos (ASStatusHints * status, int pos, unsigned int size, int vpos, int grav, int max_pos);
+int make_anchor_pos (ASStatusHints * status, int pos, int size, int vpos, int grav, int max_pos);
 void make_detach_pos (ASHints * hints, ASStatusHints * status, XRectangle *anchor, int *detach_x, int *detach_y);
 ASFlagType change_placement( struct ScreenInfo *scr, ASHints *hints, ASStatusHints *status, XPoint *anchor, ASStatusHints *new_status, int vx, int vy, ASFlagType what );
 
-void status2anchor( XRectangle *anchor, struct ASHints *hints, struct ASStatusHints *status, unsigned int vwidth, unsigned int vheight );
+void status2anchor( XRectangle *anchor, struct ASHints *hints, struct ASStatusHints *status, int vwidth, int vheight );
 void anchor2status( struct ASStatusHints *status, struct ASHints *hints, XRectangle *anchor );
 
-int calculate_viewport( int *pos, unsigned int size, unsigned int scr_vpos, unsigned int scr_size, int max_viewport );
+int calculate_viewport( int *pos, int size, int scr_vpos, int scr_size, int max_viewport );
 char *make_client_geometry_string (struct ScreenInfo * scr, ASHints *hints, ASStatusHints *status, XRectangle *anchor, int vx, int vy, char **pure_geometry);
 char *make_client_command( struct ScreenInfo *scr, ASHints *hints, ASStatusHints *status, XRectangle *anchor, int vx, int vy);
 
 Bool set_all_client_hints( Window w, ASHints *hints, ASStatusHints *status, Bool set_command );
+struct ASImage* get_client_icon_image( struct ScreenInfo * scr, ASHints *hints );
+CARD32 set_hints_window_opacity_percent( ASHints *clean, int opaque_percent );
+
 
 /* printing functions :
  * if func and stream are not specified - fprintf(stderr) is used ! */
