@@ -18,7 +18,7 @@
  *
  */
 
-#undef LOCAL_DEBUG
+#define LOCAL_DEBUG
 #undef DO_CLOCKING
 #undef UNKNOWN_KEYWORD_WARNING
 
@@ -44,6 +44,7 @@
 #include "parser.h"
 #include "freestor.h"
 #include "functions.h"
+#include "../libAfterBase/xml.h"
 
 char         *_disabled_keyword = DISABLED_KEYWORD;
 char         *_unknown_keyword = "unknown";
@@ -77,31 +78,35 @@ keyword_id2keyword( int id )
 	return _unknown_keyword;
 }
 
+
+void 
+flush_keyword_ids() 
+{
+	if( Keyword2IDHash != NULL )
+		destroy_ashash( &Keyword2IDHash );
+}
+
 void
 BuildHash (SyntaxDef * syntax)
 {
 	register int  i;
 
+	LOCAL_DEBUG_CALLER_OUT( "syntax = \"%s\"", syntax->display_name ); 		
+
 	if (syntax->term_hash_size <= 0)
 		syntax->term_hash_size = TERM_HASH_SIZE;
 	if (syntax->term_hash == NULL)
+	{
 		syntax->term_hash = create_ashash (syntax->term_hash_size, option_hash_value, option_compare, NULL);
+		LOCAL_DEBUG_OUT( "created hash %p", syntax->term_hash ); 		
+	}
+	LOCAL_DEBUG_OUT( "adding hash entries ... %s", "" ); 		
 	for (i = 0; syntax->terms[i].keyword; i++)
 	{	
 		add_hash_item (syntax->term_hash, AS_HASHABLE(syntax->terms[i].keyword), (void *)&(syntax->terms[i]));
 		register_keyword_id( syntax->terms[i].keyword, syntax->terms[i].id );
 	}
-}
-
-TermDef      *
-FindStatementTerm (char *tline, SyntaxDef * syntax)
-{
-    ASHashData hdata = {0};
-    LOCAL_DEBUG_OUT( "looking for pterm in hash table  %p of the syntax %s ", syntax->term_hash, syntax->display_name );
-    if (get_hash_item (syntax->term_hash, AS_HASHABLE(tline), &hdata.vptr)!=ASH_Success  )
-		hdata.vptr = NULL;
-    LOCAL_DEBUG_OUT( "FOUND pterm %p", hdata.vptr );
-    return hdata.vptr;
+	LOCAL_DEBUG_OUT( "%d hash entries added.", i ); 		
 }
 
 void
@@ -111,16 +116,17 @@ PrepareSyntax (SyntaxDef * syntax)
 	{
 		register int  i;
 
+ 		LOCAL_DEBUG_OUT( "syntax = \"%s\", recursion = %d", syntax->display_name, syntax->recursion ); 		
 		if (syntax->recursion > 0)
 			return;
 		syntax->recursion++;
 
-		BuildHash (syntax);
+		if (syntax->term_hash == NULL)
+			BuildHash (syntax);
+		
 		for (i = 0; syntax->terms[i].keyword; i++)
 			if (syntax->terms[i].sub_syntax)
-				/* this should prevent us from endless recursion */
-				if (syntax->terms[i].sub_syntax->term_hash == NULL)
-					PrepareSyntax (syntax->terms[i].sub_syntax);
+				PrepareSyntax (syntax->terms[i].sub_syntax);
 
 		syntax->recursion--;
 	}
@@ -133,6 +139,8 @@ FreeSyntaxHash (SyntaxDef * syntax)
 	{
 		register int  i;
 
+ 		LOCAL_DEBUG_CALLER_OUT( "syntax = \"%s\", recursion = %d", syntax->display_name, syntax->recursion ); 		
+
 		if (syntax->recursion > 0)
 			return;
 		syntax->recursion++;
@@ -163,6 +171,8 @@ PushSyntax (ConfigDef * config, SyntaxDef * syntax)
 		config->current_syntax->current_term = config->current_term;
 		config->current_syntax->current_flags = config->current_flags;
 	}
+	config->current_flags = CF_NONE ;
+	config->current_term = NULL ;
 	pnew->next = config->current_syntax;
 	pnew->syntax = syntax;
 	if (config->syntax && syntax->terminator == '\n')
@@ -191,10 +201,8 @@ PushSyntax (ConfigDef * config, SyntaxDef * syntax)
 		config->current_prepend_size += len;
 
 	}
-#ifdef DEBUG_PARSER
-	fprintf (stderr, "PushSyntax(%s, old is 0x%lX) current_prepend = [%s]\n",
-			 syntax->display_name, (unsigned long)config->syntax, config->current_prepend);
-#endif
+	LOCAL_DEBUG_OUT("%p: \"%s\", old is %p:  current_prepend = [%s]\n",
+			 syntax, syntax->display_name, config->syntax, config->current_prepend);
 	config->current_syntax = pnew;
 	config->syntax = syntax;
 }
@@ -212,18 +220,12 @@ PopSyntax (ConfigDef * config)
 		/* restoring our status */
 		config->current_term = config->current_syntax->current_term;
 		config->current_flags = config->current_syntax->current_flags;
-#ifdef DEBUG_PARSER
-		if (config->current_term)
-			fprintf (stderr,
-				 "PopSyntax(%s, flags = 0x%lX, term = [%s] ; old is 0x%lX)\n",
-				 config->syntax->display_name, config->current_flags,
-				 config->current_term->keyword, (unsigned long)(pold->syntax));
-		else
-			fprintf (stderr,
-				 "PopSyntax(%s, flags = 0x%lX, term = [%s] ; old is 0x%lX)\n",
- 				 config->syntax->display_name, config->current_flags,
- 				 "", (unsigned long)(pold->syntax));
-#endif
+		
+		LOCAL_DEBUG_OUT("%p: \"%s\", old is %p: term = \"%s\"\n",
+						 config->syntax, config->syntax->display_name, 
+						 pold->syntax,
+						 config->current_term?config->current_term->keyword:"");
+		
 		if (pold->syntax->terminator == '\n')
 		{
 			if ((config->current_prepend_size -= strlen (config->syntax->prepend_sub)) >= 0)
@@ -238,11 +240,11 @@ PopSyntax (ConfigDef * config)
 }
 
 void
-PushStorage (ConfigDef * config, FreeStorageElem ** tail)
+PushStorage (ConfigDef * config, void *storage)
 {
 	StorageStack *pnew = (StorageStack *) safecalloc (1, sizeof (StorageStack));
 
-	pnew->tail = tail;
+	pnew->storage = storage;
 	pnew->next = config->current_tail;
 	config->current_tail = pnew;
 }
@@ -264,7 +266,7 @@ PopStorage (ConfigDef * config)
 
 
 void
-config_error (ConfigDef * config, char *error)
+config_error (ConfigDef * config, char *err_text)
 {
 	if (config)
 	{
@@ -273,7 +275,7 @@ config_error (ConfigDef * config, char *error)
 		if (eol)
 			*eol = '\0';
 		show_error ("in %s (line %d):%s[%.50s]",
-				    config->current_syntax->syntax->display_name, config->line_count, error, config->tline);
+				    config->current_syntax->syntax->display_name, config->line_count, err_text, config->tline);
 		if (eol)
 			*eol = '\n';
   	}
@@ -478,6 +480,36 @@ void print_trimmed_str( char *prompt, char * str )
 #endif
 }
 
+void
+ProcessSubSyntax (ConfigDef * config, void *storage, SyntaxDef * syntax)
+{
+	PushStorage (config, storage);
+	LOCAL_DEBUG_OUT("Old_syntax = \"%s\", new_syntax = \"%s\"", config->syntax->display_name, syntax->display_name );
+	if (config->syntax->terminator == syntax->file_terminator)
+	{										   /* need to push back term's data into config buffer */
+		int skip_tokens = 0 ; 
+		config->cursor = config->tdata;
+		skip_tokens = GetTermUseTokensCount(config->current_term);
+		if ( get_flags(config->current_term->flags, TF_NAMED|TF_INDEXED|TF_DIRECTION_INDEXED) )
+			++skip_tokens ; 
+		if( skip_tokens > 0 )
+			config->cursor = tokenskip( config->cursor, skip_tokens );
+	} else if (config->syntax->terminator == syntax->terminator)
+	{										   /* need to push back entire term's line into config buffer */
+		config->cursor = config->tline_start;
+	}
+
+/*  fprintf( stderr, "\nprocessing as subsyntax: [%s]", config->cursor );
+*/
+	PushSyntax (config, syntax);
+}
+
+void ProcessStatement(ConfigDef *config)
+{
+	if( config && config->statement_handler ) 
+		config->statement_handler(config);
+}
+
 char         *
 GetToNextLine (ConfigDef * config)
 {
@@ -588,58 +620,58 @@ GetNextStatement (ConfigDef * config, int my_only)
 				{							   /* let's check for DISABLE keyword */
                     print_trimmed_str( "comments at", cur );
 					for (i = 1; i < DISABLED_KEYWORD_SIZE; i++)
-						if (*(cur + i) == '\0' || *(cur + i) != _disabled_keyword[i])
+						if (cur[i] == '\0' || cur[i] != _disabled_keyword[i])
 							break;
-					if (i < DISABLED_KEYWORD_SIZE)
+					if (i == DISABLED_KEYWORD_SIZE)
 					{						   /* comments - skip entire line */
-						while (*cur != '\n' && *cur != '\0')
-							cur++;
-						config->cursor = cur;
-						break;
-					}
-					config->current_flags |= CF_DISABLED_OPTION;
-					/* let's skip few spaces here */
-					cur = cur + i;
-					while (isspace ((int)*cur) && *cur != terminator)
-						cur++;
-					if (*cur == '\0' || *cur == terminator)
-						break;				   /* not a valid option */
+						config->current_flags |= CF_DISABLED_OPTION;
+						/* let's skip few spaces here */
+						while (isspace ((int)cur[i]) && cur[i] != terminator) ++i;
+						if (cur[i] == '\0' || cur[i] == terminator)
+							break;				   /* not a valid option */
+						cur = &cur[i];
+					}else
+						set_flags( config->current_flags, CF_PUBLIC_OPTION);/* comments are always public */
 				}
 
 				if (*cur == MYNAME_CHAR)
 				{							   /* check if we have MyName here */
 					register char *mname = config->myname;
-
+				
                     print_trimmed_str(  "private option at", cur );
-					while (*cur != '\0' && *mname != '\0')
-					{
-						if (tolower (*(mname++)) != tolower (*(++cur)))
+					++cur;
+					for( i = 0 ; mname[i] != '\0' && cur[i] != '\0' ; ++i ) 
+						if (tolower (mname[i]) != tolower (cur[i]) )
 							break;
-					}
-					if (*mname != '\0')
+					if (mname[i] != '\0')
 					{						   /* that was a foreign optiion - belongs to the other executable */
 						if (my_only)
 							break;
-						config->current_flags |= CF_FOREIGN_OPTION;
-					}
-					cur++;
+						set_flags( config->current_flags, CF_FOREIGN_OPTION );
+						/* keeping *MyName part of the token  without leading * for foreign options */
+					}else
+						cur+=i;  /* skipping *MyName part of the token */ 
 				} else
                 {
-
                     if( *cur == terminator || *cur == file_terminator )   /* public option keyword may not be empty ! */
                         break;
-                    config->current_flags |= CF_PUBLIC_OPTION;
+                    set_flags( config->current_flags, CF_PUBLIC_OPTION);
                     print_trimmed_str( "public option at", cur );
                 }
 				config->tline = cur;		   /*that will be the begginnig of the term */
 
 				/* now we should copy everything from after the first space to
-				   config->current_data and set current_data_len ; */
-                i = 0 ;
-                while (cur[i] && !isspace ((int)cur[i]) && cur[i] != terminator && cur[i] != file_terminator)
-                    ++i;
-                while ((*cur) && isspace ((int)cur[i]) && cur[i] != terminator && cur[i] != file_terminator)
-                    ++i;
+				   config->current_data and set current_data_len ; (unless its a comment) */
+				if( *cur != COMMENTS_CHAR )
+				{	
+                	i = 0 ;
+                	while (cur[i] && !isspace ((int)cur[i]) && cur[i] != terminator && cur[i] != file_terminator)
+                    	++i;
+                	while (isspace ((int)cur[i]) && cur[i] != terminator && cur[i] != file_terminator)
+                    	++i;
+				}else
+					i = 1;
+
                 cur = &(cur[i]);           /* that will be the beginning of our data */
                 config->tdata = cur;
                 print_trimmed_str( "data start at :", cur );
@@ -654,7 +686,7 @@ GetNextStatement (ConfigDef * config, int my_only)
 						if (config->current_data == NULL)
 						{
 							config_error (config, "Not enough memory to hold option's arguments");
-							exit (0);
+							return NULL;
 						}
                         data = config->current_data;
 					}
@@ -664,13 +696,13 @@ GetNextStatement (ConfigDef * config, int my_only)
                 cur = &(cur[i]);
 				/* now let's go back and remove trailing spaces */
 				if (config->tdata[0] == file_terminator)
-					config->current_flags |= CF_LAST_OPTION;
+					set_flags(config->current_flags, CF_LAST_OPTION);
 				else
 				{
-                    for (i--; i >= 0; i--)
+                    while( --i >= 0 )
 					{
 						if (config->tdata[i] == file_terminator)
-							config->current_flags |= CF_LAST_OPTION;
+							set_flags(config->current_flags, CF_LAST_OPTION);
 						if (!isspace ((int)data[i]))
 							break;
 					}
@@ -679,7 +711,7 @@ GetNextStatement (ConfigDef * config, int my_only)
 				/* since \0 is our normal data terminator we really should trigger
 				   end of the configuration when the file ends : */
 				if (file_terminator == '\0')
-					config->current_flags &= ~CF_LAST_OPTION;
+					clear_flags(config->current_flags, CF_LAST_OPTION);
                 LOCAL_DEBUG_OUT( "%d bytes of clean data stored", i );
 				config->current_data_len = i;
 				config->current_data[i] = '\0';
@@ -698,79 +730,37 @@ GetNextStatement (ConfigDef * config, int my_only)
 }
 
 
+TermDef _as_comments_term =	{TF_NO_MYNAME_PREPENDING, "#", 1, TT_COMMENT, TT_COMMENT, NULL};
+TermDef _as_unknown_term_public  =	{TF_NO_MYNAME_PREPENDING, "unknown", 7, TT_ANY, TT_ANY, NULL};
+TermDef _as_unknown_term_private =	{0, "unknown", 7, TT_ANY, TT_ANY, NULL};
 
-void
-ProcessSubSyntax (ConfigDef * config, FreeStorageElem ** tail, SyntaxDef * syntax)
+TermDef      *
+FindStatementTerm (char *tline, SyntaxDef * syntax)
 {
-	register char *ptr;
+    ASHashData hdata = {0};
+    LOCAL_DEBUG_OUT( "looking for pterm for [%c%c%c...]in hash table  %p of the syntax %s ", tline[0], tline[1], tline[2], syntax->term_hash, syntax->display_name );
+	if( tline[0] == COMMENTS_CHAR )
+		return &_as_comments_term;
 
-	PushStorage (config, tail);
-	if (config->syntax->terminator == syntax->file_terminator)
-	{										   /* need to push back term's data into config buffer */
-		config->cursor = config->tdata;
-		if (config->current_term->flags & TF_NAMED_SUBCONFIG)
-			/* we are supposed to skip single quoted text in here, or unquoted token */
-		{
-			if (*(ptr = config->cursor) == '"')
-			{
-				ptr = find_doublequotes (ptr);
-				if (!ptr)
-				{
-					config_error (config, "Unmatched doublequotes detected\n");
-					ptr = config->cursor;
-				} else
-					ptr++;					   /* skipping current doubleqoute */
-			} else							   /*simply skipping to the next space */
-				for (; *ptr && !isspace ((int)*ptr); ptr++);
-			config->cursor = ptr;
-		}
-	} else if (config->syntax->terminator == syntax->terminator)
-	{										   /* need to push back entire term's line into config buffer */
-		config->cursor = config->tline_start;
-	}
-
-/*  fprintf( stderr, "\nprocessing as subsyntax: [%s]", config->cursor );
-*/
-	PushSyntax (config, syntax);
-}
-
-void
-ProcessStatement (ConfigDef * config)
-{
-	FreeStorageElem *pNext;
-	TermDef      *pterm = config->current_term;
-LOCAL_DEBUG_OUT( "checking for foreign option ...%s", "" );
-	if (IsForeignOption (config))
-		return;
-
-LOCAL_DEBUG_OUT( "adding storage ...%s", "" );
-	if ((pNext = AddFreeStorageElem (config->syntax, config->current_tail->tail, pterm, ID_ANY)) == NULL)
-		return;
-
-LOCAL_DEBUG_OUT( "parsing stuff ...%s", "" );
-	pNext->flags = config->current_flags;
-
-	if (config->current_data_len > 0 && !(pterm->flags & TF_DONT_REMOVE_COMMENTS))
+	if( isspace(tline[0]) )
 	{
-		stripcomments (config->current_data);
-		config->current_data_len = strlen (config->current_data);
-	}
-    print_trimmed_str( "config->current_data", config->current_data );
-    LOCAL_DEBUG_OUT( "curr_data_len = %d", config->current_data_len);
-
-	args2FreeStorage (pNext, config->current_data, config->current_data_len);
-
-	config->current_tail->tail = &(pNext->next);
-	if (pterm->sub_syntax)
-		ProcessSubSyntax (config, &(pNext->sub), pterm->sub_syntax);
-	return;
+		int i = 0; 
+		while( isspace(tline[i]) ) 	   ++i;
+		if( tline[i] == '~') 
+			if( get_hash_item (syntax->term_hash, AS_HASHABLE(&tline[i]), &hdata.vptr)==ASH_Success  )
+				return hdata.vptr;
+	}	 
+    if( get_hash_item (syntax->term_hash, AS_HASHABLE(tline), &hdata.vptr)!=ASH_Success  )
+		hdata.vptr = NULL;
+    LOCAL_DEBUG_OUT( "FOUND pterm %p", hdata.vptr );
+    return hdata.vptr;
 }
 
 
 
 /* main parsing procedure */
 int
-ParseConfig (ConfigDef * config, FreeStorageElem ** tail)
+config2tree_storage (ConfigDef * config, ASTreeStorageModel **tail, Bool ignore_foreign)
 {
 	int           TopLevel = 0;
 	unsigned long flags;
@@ -779,50 +769,73 @@ ParseConfig (ConfigDef * config, FreeStorageElem ** tail)
 	/* get line */
 	while (!TopLevel)
 	{
-		while (GetNextStatement (config, 1))
+		while (GetNextStatement (config, ignore_foreign))
 		{									   /* untill not end of text */
 			flags = 0x00;
+			TermDef *pterm = NULL ; 
+
 #ifdef DEBUG_PARSER
 			fprintf (stderr, "\nSentence Found:[%.50s ...]\n,\tData=\t[%s]", config->tline, config->current_data);
 			fprintf (stderr, "\nLooking for the Term...");
 #endif
 			/* find term */
-			if ((config->current_term = FindStatementTerm (config->tline, config->syntax)))
+			if( get_flags( config->current_flags, CF_FOREIGN_OPTION ) ) 
 			{
-#ifdef DEBUG_PARSER
-				fprintf (stderr, "\nTerm Found:[%s]", config->current_term->keyword);
-#endif
-				if (get_flags( config->current_term->flags, TF_OBSOLETE))
-					config_error (config, "Heh, It seems that I've encountered obsolete config option. I'll ignore it for now, Ok ?!");
-				if (get_flags( config->current_term->flags, TF_PHONY))
-					set_flags( config->flags, CF_PHONY_OPTION );
-  				if (get_flags( config->current_term->flags, TF_SPECIAL_PROCESSING))
-				{
-					if (config->special)
-					{
-						FreeStorageElem **ctail = config->current_tail->tail;
+				int i = 0 ; 
+				do
+				{	
+					++i;       /* it's ok - we can start with 1, since myname should have at least 1 char */
+					pterm = FindStatementTerm (&(config->tline[i]), config->syntax);
+				}while( pterm == NULL && !isspace(config->tline[i]) && config->tline[i] != '\0' );
+			}else
+				pterm = FindStatementTerm (config->tline, config->syntax);
 
-						flags = (*(config->special)) (config, ctail);
-						if (get_flags (flags, SPECIAL_BREAK))
-							break;
-						if (get_flags (flags, SPECIAL_STORAGE_ADDED))
-						{
-							for (ctail = config->current_tail->tail; (*ctail); ctail = &((*ctail)->next));
-							tail = config->current_tail->tail = ctail;
-						}
+			if ( pterm == NULL )
+				pterm = get_flags( config->current_flags, CF_PUBLIC_OPTION)?
+				 			&_as_unknown_term_public : &_as_unknown_term_private ;
+			
+			config->current_term = pterm;
+			
+			LOCAL_DEBUG_OUT("Term:[%s]", config->current_term->keyword);
+			if( isspace(config->tline[0]) &&  pterm->keyword_len > 0 &&
+				mystrncasecmp(pterm->keyword, config->current_data, pterm->keyword_len) == 0 ) 
+			{              /* we need to skip one token in current_data :  */
+				char *src = tokenskip( config->current_data, 1 ) ;
+				char *dst = config->current_data ;
+				if( src != dst ) 
+				{
+					int i = 0 ; 
+					do{ dst[i] = src[i]; }while( src[++i] );
+					dst[i] = '\0';
+				}		
+			}	 
+			if (get_flags( pterm->flags, TF_OBSOLETE))
+				config_error (config, "Heh, It seems that I've encountered obsolete config option. I'll ignore it for now, Ok ?!");
+			if (get_flags( pterm->flags, TF_PHONY))
+				set_flags( config->flags, CF_PHONY_OPTION );
+  			if (get_flags( pterm->flags, TF_SPECIAL_PROCESSING))
+			{
+				if (config->special)
+				{
+					flags = (*(config->special)) (config);
+					if (get_flags (flags, SPECIAL_BREAK))
+						break;
+					if (get_flags (flags, SPECIAL_STORAGE_ADDED))
+					{
+						ASTreeStorageModel **ctail = config->current_tail->storage;
+						while(*ctail) ctail = &((*ctail)->next);
+						tail = config->current_tail->storage = ctail;
 					}
 				}
-				if (!get_flags (flags, SPECIAL_SKIP))
-					ProcessStatement (config);
-
-				if ((config->current_term->flags & TF_SYNTAX_TERMINATOR) || IsLastOption (config))
-					break;
-			} else
-			{
-#ifdef UNKNOWN_KEYWORD_WARNING
-				config_error (config, " unknown keyword encountered");
-#endif
-                if (IsLastOption (config))
+			}
+			if (!get_flags (flags, SPECIAL_SKIP))
+				ProcessStatement (config);
+			/* Process Statement may alter config's state as it may dive into subconfig - 
+			 * must make sure that did not happen : */
+			if( config->current_term ) 
+			{	
+				if ( get_flags(config->current_term->flags, TF_SYNTAX_TERMINATOR) || 
+						IsLastOption (config) )
 					break;
 			}
 		}									   /* end while( GetNextStatement() ) */
@@ -846,32 +859,6 @@ ParseConfig (ConfigDef * config, FreeStorageElem ** tail)
 }
 
 
-FreeStorageElem *
-file2free_storage(const char *filename, char *myname, SyntaxDef *syntax, SpecialFunc special, FreeStorageElem **foreign_options )
-{
-	ConfigData cd ;
-	ConfigDef    *config_reader ;
-	FreeStorageElem *storage = NULL ;
-	
-	cd.filename = filename ;
-	config_reader = InitConfigReader (myname, syntax, CDT_Filename, cd, special);
-	if (!config_reader)
-		return NULL;
-
-	PrintConfigReader (config_reader);
-	ParseConfig(config_reader, &storage);
-	PrintFreeStorage (storage);
-
-	/* getting rid of all the crap first */
-	if( foreign_options )
-		StorageCleanUp (&storage, foreign_options, CF_DISABLED_OPTION|CF_PHONY_OPTION|CF_FOREIGN_OPTION);
-
-	DestroyConfig (config_reader);   
-	return storage;
-}	 
-
-
-
 
 /****************************************************************************************/
 /*                        Assorted utility functions                                    */
@@ -951,41 +938,10 @@ InitConfigWriter (char *myname, SyntaxDef * syntax, ConfigDataType type, ConfigD
 }
 
 /***************************************************************************/
-/* functions used in writing config */
-#ifdef WITH_CONFIG_WRITER
-/* Finds element in FreeStorage that represents this particular term
- * it then removes it from the list of elems and returns pointer to it
- */
-FreeStorageElem *
-FindTermAndRemove (FreeStorageElem ** storage, TermDef * pterm)
-{
-	FreeStorageElem **ppCurr = storage, *pRes;
-
-	if (storage == NULL)
-		return NULL;
-	for (; *ppCurr != NULL; ppCurr = &((*ppCurr)->next))
-		if ((*ppCurr)->term == pterm)
-			break;
-
-	pRes = *ppCurr;
-	if (pRes != NULL)
-	{
-		*ppCurr = pRes->next;
-		pRes->next = NULL;
-	}
-
-	return pRes;
-}
-
 /* writes block of text in to the output buffer, enlarging it if needed.
  * it starts with block_start and end with block_end or '\0'
  * if block_end is NULL
  */
-struct WriteBuffer
-{
-	char         *buffer;
-	size_t        allocated, used;
-};
 
 void
 WriteBlock (struct WriteBuffer *t_buffer, char *block_start, char *block_end)
@@ -1017,335 +973,10 @@ WriteBlock (struct WriteBuffer *t_buffer, char *block_start, char *block_end)
 
 
 char         *
-terminate (char *ptr, char terminator)
+parser_add_terminator (char *ptr, char terminator)
 {
 	if (*ptr != terminator && (*ptr == '\0' && *(ptr - 1) != terminator))
 		*(ptr++) = terminator;
 	return ptr;
 }
 
-/*
- * creates string representing freeStorage's elem and writes it into the
- * output buffer
- */
-static void
-WriteFreeStorageElem (ConfigDef * config, struct WriteBuffer *t_buffer, FreeStorageElem * pElem, int level)
-{
-	static char  *elem_buf = NULL;
-	static long   buf_allocated = 0;
-
-	size_t        elem_buf_size = 0, params_size;
-	register SyntaxDef *csyntax;
-	register char *ptr;
-
-	if (pElem == NULL || config->syntax == NULL)
-	{
-		if (elem_buf)
-			free (elem_buf);
-		elem_buf = NULL;
-		buf_allocated = 0;
-		return;
-	}
-	if (pElem->term == NULL)
-		return;
-
-	csyntax = config->syntax;
-	/* now lets approximate our memory size needed to accomodate complete config line : */
-	/* ( allowing some slack just for simplier algorithm, since few bytes would not really matter ) */
-	/* first goes constants (denoted as C# below) : */
-	elem_buf_size += DISABLED_KEYWORD_SIZE	   /* C1 */
-		+ 1									   /* C2 */
-		+ 1									   /* C3 */
-		+ pElem->argc						   /* C4 */
-		+ 1;								   /* C5 */
-	/* now calculating variable lengths (denoted as V# below ): */
-	if ((ptr = config->current_prepend) != NULL)	/* V1 */
-		while (*(ptr++))
-			elem_buf_size++;
-
-	ptr = csyntax->prepend_one;				   /* V2 */
-	while (*(ptr++))
-		elem_buf_size++;
-
-	if ((ptr = config->myname) != NULL)		   /* V3 */
-		while (*(ptr++))
-			elem_buf_size++;
-
-	if ((ptr = pElem->term->keyword) != NULL)  /* V4 */
-		elem_buf_size += pElem->term->keyword_len;
-
-	params_size = GetStringArraySize (pElem->argc, pElem->argv);	/* V5 */
-	elem_buf_size += params_size;
-
-	if ((ptr = csyntax->prepend_sub) != NULL)  /* V6 */
-		while (*(ptr++))
-			elem_buf_size++;
-	elem_buf_size++;
-
-	if (elem_buf_size > 0)
-	{
-		register char *src;
-
-		if (buf_allocated < elem_buf_size)
-		{
-			buf_allocated = elem_buf_size + (elem_buf_size >> 3);	/* to anticipate future buffer grows */
-			if (elem_buf != NULL)
-				free (elem_buf);
-			elem_buf = (char *)safemalloc (buf_allocated);
-		}
-		/*below is what's actually is using our memory: */
-		ptr = elem_buf;
-		if (config->current_prepend)		   /*V1: config->current_prepend */
-		{
-			src = config->current_prepend;
-			while (*src)
-				*(ptr++) = *(src++);
-		}
-
-		src = csyntax->prepend_one;			   /*V2: csyntax->prepend_one */
-		while (*src)
-			*(ptr++) = *(src++);
-
-		if (pElem->flags & CF_DISABLED_OPTION) /*C1: DISABLED_KEYWORD_SIZE */
-		{
-			src = _disabled_keyword;
-			while (*src)
-				*(ptr++) = *(src++);
-			*(ptr++) = csyntax->token_separator;	/*C2: csyntax->token_separator */
-		}else if (pElem->flags & CF_COMMENTED_OPTION) /*C1: DISABLED_KEYWORD_SIZE */
-		{
-			*(ptr++) = COMMENTS_CHAR;
-			*(ptr++) = csyntax->token_separator;	/*C2: csyntax->token_separator */
-		}
-
-		if (!(pElem->term->flags & TF_NO_MYNAME_PREPENDING)
-			&& !(pElem->flags & CF_PUBLIC_OPTION) && config->myname != NULL)
-		{
-			*(ptr++) = MYNAME_CHAR;			   /*C3: MYNAME_CHAR */
-			src = config->myname;			   /*V3: config->myname */
-			while (*src)
-				*(ptr++) = *(src++);
-		}
-		if (pElem->term->keyword)			   /*V4: pElem->term->keyword */
-		{
-			src = pElem->term->keyword;
-			while (*src)
-				*(ptr++) = *(src++);
-		}
-		/* some beautification is in order here */
-		if (params_size)
-		{
-			int           i;
-
-			*(ptr++) = csyntax->token_separator;	/*C4: argc*csyntax->token_separator */
-			for (i = 0; i < pElem->argc; i++)
-				if ((src = pElem->argv[i]) != NULL)
-				{
-					if (i)
-						*(ptr++) = csyntax->token_separator;
-					while (*src)
-						*(ptr++) = *(src++);   /*V5: argv[i] */
-				}
-		}
-		*ptr = '\0';						   /* so that termination will work */
-		if (pElem->sub == NULL || pElem->term->sub_syntax == NULL)	/*C5: csyntax->terminator */
-			ptr = terminate (ptr, csyntax->terminator);
-		else if (pElem->term->sub_syntax != NULL)
-		{
-			if (pElem->term->sub_syntax->terminator == csyntax->terminator)
-				ptr = terminate (ptr, csyntax->terminator);
-			else if (csyntax->terminator == '\0' && pElem->term->sub_syntax->terminator != '\0')
-				ptr = terminate (ptr, pElem->term->sub_syntax->terminator);
-		}
-
-		if (*(ptr - 1) != csyntax->terminator) /* V6: min(csyntax->prepend_sub,1) */
-		{									   /* we don't want for tokens to get joined together */
-			src = csyntax->prepend_sub;
-			if (src != NULL && *src != '\0')
-				while (*src)
-					*(ptr++) = *(src++);
-			else
-				ptr = terminate (ptr, csyntax->token_separator);
-		}
-
-		WriteBlock (t_buffer, elem_buf, ptr);
-	}
-	if (pElem->sub)
-	{
-		FreeStorageElem *psub;
-
-		if (pElem->term->sub_syntax)
-			PushSyntax (config, pElem->term->sub_syntax);
-		for (psub = pElem->sub; psub; psub = psub->next)
-			WriteFreeStorageElem (config, t_buffer, psub, level + 1);
-		if (pElem->term->sub_syntax)
-		{
-			PopSyntax (config);
-			if ((pElem->term->sub_syntax->file_terminator) != '\0')
-			{
-				*elem_buf = pElem->term->sub_syntax->file_terminator;
-				WriteBlock (t_buffer, elem_buf, elem_buf + 1);
-			}
-		}
-	}
-}
-
-void
-WriteRemnants (ConfigDef * config, struct WriteBuffer *t_buffer, FreeStorageElem * pCurr)
-{
-/* > this function is writing options that are acually new for this config */
-/* > as the result we want to write them into the begining of the file */
-/* > so lets save current buffer contents, write remnants and then write buffer back */
-/* I've changed that back to have later options override previous options */
-/*                                                      Sasha
- */
-/*  struct WriteBuffer rem_buffer ;
- *  int remnants_num = CountFreeStorageElems( pCurr );
- *
- *    if( remnants_num<=0 ) return ;
- *    rem_buffer.allocated = remnants_num*NORMAL_CONFIG_LINE+t_buffer->used+2 ;
- *    rem_buffer.buffer = (char*)safemalloc( rem_buffer.allocated );
- *    rem_buffer.used = 0 ;
- */
-	for (; pCurr != NULL; pCurr = pCurr->next)
-	{
-		if (pCurr->term == NULL)
-			continue;
-/*      WriteFreeStorageElem( config, &rem_buffer, pCurr );
- */ WriteFreeStorageElem (config, t_buffer, pCurr, 0);
-	}
-
-/*    WriteBlock( &rem_buffer, t_buffer->buffer, t_buffer->buffer+t_buffer->used );
- *    free( t_buffer->buffer );
- *    t_buffer->allocated = rem_buffer.allocated ;
- *    t_buffer->used = rem_buffer.used ;
- *    t_buffer->buffer = rem_buffer.buffer ;
- */
-}
-
-void
-ScanAndWriteExistant (ConfigDef * config, FreeStorageElem ** storage, struct WriteBuffer *t_buffer, unsigned long flags)
-{
-	char         *noise_start = config->cursor;
-	FreeStorageElem *pCurr;
-
-	while (GetNextStatement (config, 0))
-	{										   /* until not end of text */
-		TermDef      *pterm;
-
-		if (!(flags & WF_DISCARD_COMMENTS))
-		{
-			WriteBlock (t_buffer, noise_start, config->tline_start);
-			noise_start = config->cursor;
-		}
-		if (IsForeignOption (config))
-		{
-			if (!(flags & WF_DISCARD_FOREIGN))
-				WriteBlock (t_buffer, config->tline_start, config->cursor);
-			continue;
-		}
-
-		pterm = FindStatementTerm (config->tline, config->syntax);
-		if (IsPublicOption (config))
-		{
-			int           write_it = 1;
-
-			if (pterm)
-				write_it = !(pterm->flags & TF_NO_MYNAME_PREPENDING);
-			if (write_it)
-			{
-				if (!(flags & WF_DISCARD_PUBLIC))
-					WriteBlock (t_buffer, config->tline_start, config->cursor);
-				continue;
-			}
-		}
-
-		/* find term */
-		if (pterm)
-		{
-			pCurr = FindTermAndRemove (storage, pterm);
-			if (pCurr)
-				WriteFreeStorageElem (config, t_buffer, pCurr, 0);
-		} else if (!(flags & WF_DISCARD_UNKNOWN))
-			WriteBlock (t_buffer, config->tline_start, config->cursor);
-
-		DestroyFreeStorage (&pCurr);		   /* no longer need it */
-	}
-
-	if (!(flags & WF_DISCARD_COMMENTS))
-		WriteBlock (t_buffer, noise_start, config->cursor);
-
-}
-
-#endif
-/***************************************************************************/
-/* main writing procedure ( returns size of the data written )             */
-
-long
-WriteConfig (ConfigDef * config, FreeStorageElem ** storage,
-			 ConfigDataType target_type, ConfigData *target, unsigned long flags)
-{
-#ifdef WITH_CONFIG_WRITER
-	struct WriteBuffer t_buffer;
-	int           t_fd = -1;
-
-	if (config == NULL || storage == NULL || target == NULL)
-		return -1;
-	if (*storage == NULL)
-		return -1;
-
-	if (config->buffer_size <= 0)
-	{
-		t_buffer.allocated = MAXLINELENGTH;
-		flags = WF_DISCARD_EVERYTHING;
-	} else
-		t_buffer.allocated = config->buffer_size;
-
-	t_buffer.buffer = (char *)safemalloc (t_buffer.allocated);
-	t_buffer.used = 0;
-	/* scanning old file first to preserv comments */
-	if (!(flags & WF_DISCARD_COMMENTS) ||
-		!(flags & WF_DISCARD_UNKNOWN) || !(flags & WF_DISCARD_FOREIGN) || !(flags & WF_DISCARD_PUBLIC))
-		ScanAndWriteExistant (config, storage, &t_buffer, flags);
-	/* now writing remaining elements */
-	WriteRemnants (config, &t_buffer, *storage);
-	DestroyFreeStorage (storage);
-
-	t_buffer.buffer[t_buffer.used] = config->syntax->file_terminator;
-	t_buffer.buffer[t_buffer.used + 1] = '\0';
-
-	if (config->syntax->file_terminator != '\0')
-		t_buffer.used++;
-
-	/* now saving buffer into file if we need to */
-	switch (target_type)
-	{
-	 case CDT_Filename:
-		 t_fd = open (target->filename, O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH);
-		 break;
-	 case CDT_FilePtr:
-     case CDT_FilePtrAndData:
-         t_fd = fileno (target->fileptr);
-		 break;
-	 case CDT_FileDesc:
-		 t_fd = *(target->filedesc);
-		 break;
-	 case CDT_Data:
-		 target->data = t_buffer.buffer;
-		 return t_buffer.used;
-	}
-	if (t_fd != -1)
-	{
-		write (t_fd, t_buffer.buffer, t_buffer.used);
-		if (target_type == CDT_Filename)
-			close (t_fd);
-	}
-
-	free (t_buffer.buffer);
-	WriteFreeStorageElem (NULL, NULL, NULL, 0);	/* freeing statically allocated buffer */
-
-	return t_buffer.used;
-#else
-	return 0;
-#endif
-}
