@@ -88,6 +88,8 @@ Atom  		_AS_BUTTON_SHADE_PRESSED	= None;
 Atom  		_AS_BUTTON_MENU 			= None;
 Atom  		_AS_BUTTON_MENU_PRESSED 	= None;
 
+Atom  		_GTK_READ_RCFILES = None;
+Atom  		_KIPC_COMM_ATOM = None;
 
 /* Crossreferences of atoms into flag value for
    different atom list type of properties :*/
@@ -152,6 +154,9 @@ AtomXref  _WMPropAtoms[] = {
     WMPROPS_ATOM_DESC(_AS_BUTTON_MENU),
     WMPROPS_ATOM_DESC(_AS_BUTTON_MENU_PRESSED),
 	
+	WMPROPS_ATOM_DESC(_GTK_READ_RCFILES),
+	WMPROPS_ATOM_DESC(_KIPC_COMM_ATOM),
+	
 	{NULL, NULL, 0, None}
 };
 
@@ -289,13 +294,23 @@ setup_compatibility_props (ScreenInfo * scr)
 {
 	/* Old Gnome Compatibility */
 	set_multi32bit_property (scr->Root, _XA_WIN_PROTOCOLS, XA_ATOM, 7,
-							 _XA_WIN_LAYER, _XA_WIN_STATE, _XA_WIN_WORKSPACE, _XA_WIN_HINTS,
-							 _XA_WIN_WORKSPACE_COUNT, _XA_WIN_CLIENT_LIST,_XA_WIN_DESKTOP_BUTTON_PROXY);
+							 _XA_WIN_LAYER, 				_XA_WIN_STATE, 				_XA_WIN_WORKSPACE, 
+							 _XA_WIN_HINTS, 				_XA_WIN_WORKSPACE_COUNT, 	_XA_WIN_CLIENT_LIST,
+							 _XA_WIN_DESKTOP_BUTTON_PROXY);
 	/* Ext WM compatibility */
-	set_multi32bit_property (scr->Root, _XA_NET_SUPPORTED, XA_ATOM, 28, _XA_NET_SUPPORTED, _XA_NET_CLIENT_LIST, _XA_NET_CLIENT_LIST_STACKING, _XA_NET_NUMBER_OF_DESKTOPS, _XA_NET_DESKTOP_GEOMETRY, _XA_NET_DESKTOP_VIEWPORT, _XA_NET_CURRENT_DESKTOP, _XA_NET_ACTIVE_WINDOW, _XA_NET_SUPPORTING_WM_CHECK, _XA_NET_VIRTUAL_ROOTS,	/* 11 */
+	set_multi32bit_property (scr->Root, _XA_NET_SUPPORTED, XA_ATOM, 29, 
+							 _XA_NET_SUPPORTED, 			_XA_NET_CLIENT_LIST, 		_XA_NET_CLIENT_LIST_STACKING, 
+							 _XA_NET_NUMBER_OF_DESKTOPS, 	_XA_NET_DESKTOP_GEOMETRY, 	_XA_NET_DESKTOP_VIEWPORT, 
+							 _XA_NET_CURRENT_DESKTOP, 		_XA_NET_ACTIVE_WINDOW, 		_XA_NET_SUPPORTING_WM_CHECK, 
+							 _XA_NET_VIRTUAL_ROOTS,	/* 10 */
 							 /* client window properties : */
-							 _XA_NET_WM_NAME, _XA_NET_WM_DESKTOP, _XA_NET_WM_WINDOW_TYPE, _XA_NET_WM_WINDOW_TYPE_DESKTOP, _XA_NET_WM_WINDOW_TYPE_DOCK, _XA_NET_WM_WINDOW_TYPE_TOOLBAR, _XA_NET_WM_WINDOW_TYPE_MENU, _XA_NET_WM_WINDOW_TYPE_DIALOG, _XA_NET_WM_WINDOW_TYPE_NORMAL, _XA_NET_WM_STATE,	/*10 */
-							 _XA_NET_WM_STATE_MODAL, _XA_NET_WM_STATE_STICKY, _XA_NET_WM_STATE_MAXIMIZED_VERT, _XA_NET_WM_STATE_MAXIMIZED_HORZ, _XA_NET_WM_STATE_SHADED, _XA_NET_WM_STATE_SKIP_TASKBAR, _XA_NET_WM_PID, _XA_NET_WM_PING);	/*19 */
+							 _XA_NET_WM_NAME, 					_XA_NET_WM_DESKTOP, 			_XA_NET_WM_WINDOW_TYPE, 
+							 _XA_NET_WM_WINDOW_TYPE_DESKTOP, 	_XA_NET_WM_WINDOW_TYPE_DOCK, 	_XA_NET_WM_WINDOW_TYPE_TOOLBAR, 
+							 _XA_NET_WM_WINDOW_TYPE_MENU, 		_XA_NET_WM_WINDOW_TYPE_DIALOG, 	_XA_NET_WM_WINDOW_TYPE_NORMAL, 
+							 _XA_NET_WM_STATE,					_XA_NET_WM_STATE_MODAL,    		_XA_NET_WM_STATE_STICKY,
+							 _XA_NET_WM_STATE_MAXIMIZED_VERT, 	_XA_NET_WM_STATE_MAXIMIZED_HORZ,_XA_NET_WM_STATE_SHADED, 		
+							 _XA_NET_WM_STATE_SKIP_TASKBAR, 	_XA_NET_WM_PID, 				_XA_NET_WM_PING, 				
+							 _XA_NET_WM_ICON);	/*19 */
 
 }
 
@@ -461,7 +476,7 @@ read_as_current_desk (ASWMProps * wmprops, Bool deleted)
 {
 	if (wmprops)
 	{
-        INT32        desk_no = None;
+        CARD32        desk_no = 0;
 
 		if (deleted)
 			return False;
@@ -926,6 +941,8 @@ destroy_wmprops (ASWMProps * wmprops, Bool reusable)
 			free (wmprops->as_socket_filename);
 		if (wmprops->as_desk_numbers)
 			free (wmprops->as_desk_numbers);
+		if (wmprops->as_tbar_props_data)
+			free (wmprops->as_tbar_props_data);
 
         /* we are being paranoid : */
         memset (wmprops, 0x00, sizeof (ASWMProps));
@@ -1112,7 +1129,7 @@ set_desktop_num_prop (ASWMProps * wmprops, INT32 new_desk, Window vroot, Bool ad
 		/* need to update crossreference table here : */
 		set_32bit_property (wmprops->scr->Root, _XA_NET_NUMBER_OF_DESKTOPS, XA_CARDINAL, wmprops->desktop_num);
 		set_32bit_property (wmprops->scr->Root, _XA_WIN_WORKSPACE_COUNT, XA_CARDINAL, wmprops->desktop_num);
-		set_32bit_proplist (wmprops->scr->Root, _AS_DESK_NUMBERS, XA_CARDINAL, &(wmprops->as_desk_numbers[0]),
+		set_32bit_proplist (wmprops->scr->Root, _AS_DESK_NUMBERS, XA_CARDINAL, (CARD32*)&(wmprops->as_desk_numbers[0]),
 							wmprops->as_desk_num);
 		set_32bit_proplist (wmprops->scr->Root, _XA_NET_VIRTUAL_ROOTS, XA_WINDOW, &(wmprops->virtual_roots[0]),
 							wmprops->desktop_num);
