@@ -86,7 +86,7 @@ find_super_section( xml_elem_t* owner, const char *id )
 			if( attr_curr->tag_id == DOCBOOK_id_ID ) 
 				break;
 		match_found = ( attr_curr!= NULL && strncmp( attr_curr->parm, id, strlen(attr_curr->parm)) == 0 ) ;
-		LOCAL_DEBUG_OUT( "xml_elem_delete for attr %p", attr );
+/*		LOCAL_DEBUG_OUT( "xml_elem_delete for attr %p", attr ); */
 		xml_elem_delete(NULL, attr);					
 		
 		if( match_found ) 
@@ -380,7 +380,14 @@ close_link( ASXMLInterpreterState *state )
 			fwrite( "</A>", 1, 4, state->dest_fp );	   
 		}else if( state->doc_type == DocType_PHP )
 		{
-			fprintf( state->dest_fp, "\",\"%s\",$srcunset,$subunset) ?>", state->curr_url_page);	   
+			if ( TopicIndexName == NULL )
+			{
+			    fprintf( state->dest_fp, "\",\"%s\",\"%s\",$subunset) ?>", state->curr_url_page, state->display_name);
+			}
+			else
+			{
+			    fprintf( state->dest_fp, "\",\"%s\",$srcunset,$subunset) ?>", state->curr_url_page);
+			}
 			 							
 		}	 
 		clear_flags( state->flags, ASXMLI_InsideLink|ASXMLI_LinkIsLocal|ASXMLI_LinkIsURL );
@@ -459,6 +466,11 @@ add_local_link( xml_elem_t *attr, ASXMLInterpreterState *state )
 				fwrite( "\">", 1, 2, state->dest_fp );
 			}else if( state->doc_type == DocType_PHP ) 
 			{
+			    if (TopicIndexName == NULL) /* added for php part of data catalogue */
+			    {
+				fprintf( state->dest_fp, "<? local_doc_url(\"graphics.php\",\"" );
+			    }
+			    else
 				fprintf( state->dest_fp, "<? local_doc_url(\"visualdoc.php\",\"" );
 			}
 			set_flags( state->flags, ASXMLI_InsideLink|ASXMLI_LinkIsURL  );
@@ -682,7 +694,7 @@ void
 end_term_tag( xml_elem_t *doc, xml_elem_t *parm, ASXMLInterpreterState *state )
 {
 	if( state->doc_type == DocType_HTML || state->doc_type == DocType_PHP	 )
-		fwrite( "</B></DT>", 1, 9, state->dest_fp );	
+		fwrite( "</B></DT>", 1, 9, state->dest_fp );
 	else if( state->doc_type == DocType_NROFF )
 	{	
 		fprintf( state->dest_fp, "\"\n");
@@ -727,17 +739,22 @@ start_imagedata_tag( xml_elem_t *doc, xml_elem_t *parm, ASXMLInterpreterState *s
 			}	 
 			parm = parm->next ;
 		}		
-
-		fprintf( state->dest_fp, "<IMG src=\"%s\"", url );	
-		if( align != NULL ) 
-			fprintf( state->dest_fp, " align=\"%s\"", align );	 
-		if( valign != NULL ) 
-			fprintf( state->dest_fp, " valign=\"%s\"", valign );	 
-		if( width != NULL ) 
-			fprintf( state->dest_fp, " width=\"%s\"", width );	 
-		if( height != NULL ) 
-			fprintf( state->dest_fp, " height=\"%s\"", height );	 
-		fwrite( ">", 1, 1, state->dest_fp );	  
+		if( url ) 
+		{	
+			if (  state->doc_type == DocType_PHP ) 
+		    	fprintf( state->dest_fp, "<IMG SRC=%s/%s", state->dest_dir, url );
+			else
+		    	fprintf( state->dest_fp, "<IMG src=\"%s\"", url );
+			if( align != NULL ) 
+				fprintf( state->dest_fp, " align=\"%s\"", align );	 
+			if( valign != NULL ) 
+				fprintf( state->dest_fp, " valign=\"%s\"", valign );	 
+			if( width != NULL ) 
+				fprintf( state->dest_fp, " width=\"%s\"", width );	 
+			if( height != NULL ) 
+				fprintf( state->dest_fp, " height=\"%s\"", height );	 
+			fwrite( ">", 1, 1, state->dest_fp );	  
+		}
 	}
 }
 
@@ -745,8 +762,10 @@ void
 end_imagedata_tag( xml_elem_t *doc, xml_elem_t *parm, ASXMLInterpreterState *state )
 {
 	close_link(state);
-	if( state->doc_type == DocType_HTML || state->doc_type == DocType_PHP	  	)
-		fwrite( "</IMG>", 1, 6, state->dest_fp );	
+	if( state->doc_type == DocType_HTML || state->doc_type == DocType_PHP	)
+		fwrite( "</IMG>", 1, 6, state->dest_fp );
+//		fprintf( state->dest_fp, "\n", NULL );
+		fputc( '\n', state->dest_fp );
 }
 
 
