@@ -158,7 +158,7 @@ SelectDecor (ASWindow * t)
  *
  ***********************************************************************/
 ASWindow     *
-AddWindow (Window w)
+AddWindow (Window w, Bool from_map_request)
 {
 	ASWindow     *tmp_win;					   /* new afterstep window structure */
 	ASRawHints    raw_hints ;
@@ -188,7 +188,7 @@ AddWindow (Window w)
     {
         if( is_output_level_under_threshold(OUTPUT_LEVEL_HINTS) )
             print_hints( NULL, NULL, &raw_hints );
-        hints = merge_hints( &raw_hints, Database, &status, Scr.Look.supported_hints, HINT_ANY, NULL );
+        hints = merge_hints( &raw_hints, Database, &status, Scr.Look.supported_hints, HINT_ANY, NULL, w );
         destroy_raw_hints( &raw_hints, True );
         if( hints )
         {
@@ -218,7 +218,6 @@ AddWindow (Window w)
     tmp_win->wm_state_transition = get_flags(status.flags, AS_Iconic)?ASWT_Withdrawn2Iconic:ASWT_Withdrawn2Normal ;
 
     pending_placement = init_aswindow_status( tmp_win, &status );
-
 #ifdef SHAPE
     XShapeSelectInput (dpy, tmp_win->w, ShapeNotifyMask);
 #endif
@@ -231,10 +230,8 @@ AddWindow (Window w)
 	 * gotten one for anything up to here, however.
 	 */
     ASSync( False );
-	if( !pending_placement )
-	{
-	    grab_server();
-	}
+    grab_server();
+/*vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv Start Grab vvvvvvvvvvvvvvvvvvvvv */
     if (validate_drawable(tmp_win->w, NULL, NULL) == None)
 	{
 		destroy_hints(tmp_win->hints, False);
@@ -247,6 +244,8 @@ AddWindow (Window w)
 	/* add the window into the afterstep list */
     enlist_aswindow( tmp_win );
     redecorate_window  ( tmp_win, False );
+    /* saving window management properties : */
+    set_client_desktop( tmp_win->w, ASWIN_DESK(tmp_win) );
 
 	/* we have to set shape on frame window. If window has title - 
 	 * on_window_title_changed will take care of it - otherwise we force it
@@ -257,11 +256,19 @@ AddWindow (Window w)
 	    on_window_title_changed ( tmp_win, False );
 	else 
 		SetShape( tmp_win, 0 );
+	/* Must do it now or else Java will freak out !!! */
+    XMapRaised (dpy, tmp_win->w);
+	RaiseWindow(tmp_win);
 
+/*^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ End Grab ^^^^^^^^^^^^^^^^^^^^ */
+	ungrab_server();
+
+	/* doing that for any window seems to resolve java sizing bugs */
+	/* this will get called from set_window_wm_state() again just few lines below
+	   which is a bit of a excess, but we don't seem to have much choice */
+    on_window_status_changed( tmp_win, False, True );
     if( pending_placement )
     {
-		RaiseWindow(tmp_win);
-        on_window_status_changed( tmp_win, False, True );
         if( !place_aswindow( tmp_win ) )
         {
             LOCAL_DEBUG_OUT( "window status initialization failed for %lX - will put it where it is", w );
@@ -270,10 +277,9 @@ AddWindow (Window w)
 			 */
         }
     }
-    /* saving window management properties : */
-    set_client_desktop( tmp_win->w, ASWIN_DESK(tmp_win) );
-    set_window_wm_state( tmp_win, get_flags(status.flags, AS_Iconic) );
-	RaiseWindow(tmp_win);
+
+
+    set_window_wm_state( tmp_win, get_flags(status.flags, AS_Iconic), from_map_request );
  
 	if( ASWIN_HFLAGS( tmp_win, AS_AvoidCover )  )
 		enforce_avoid_cover( tmp_win );
@@ -284,12 +290,10 @@ AddWindow (Window w)
 	 * WithdrawnState in HandleUnmapNotify.  Map state gets set correctly
 	 * again in HandleMapNotify.
      */
-	if( !pending_placement )
-		ungrab_server();
     broadcast_config (M_ADD_WINDOW, tmp_win);
-    broadcast_window_name( tmp_win );
     broadcast_res_names( tmp_win );
     broadcast_icon_name( tmp_win );
+    broadcast_window_name( tmp_win );
 
     InstallWindowColormaps (tmp_win);
 	set_flags( tmp_win->internal_flags, ASWF_WindowComplete) ;
@@ -348,7 +352,7 @@ AddInternalWindow (Window w, ASInternalWindow **pinternal, ASHints **phints, ASS
 
     redecorate_window       ( tmp_win, False );
     on_window_title_changed ( tmp_win, False );
-    set_window_wm_state( tmp_win, get_flags(status->flags, AS_Iconic) );
+    set_window_wm_state( tmp_win, get_flags(status->flags, AS_Iconic), False );
 	RaiseWindow( tmp_win );
 /*    activate_aswindow( tmp_win, False, False ); */
 
@@ -359,9 +363,9 @@ AddInternalWindow (Window w, ASInternalWindow **pinternal, ASHints **phints, ASS
 	 * again in HandleMapNotify.
      */
     broadcast_config (M_ADD_WINDOW, tmp_win);
-    broadcast_window_name( tmp_win );
     broadcast_res_names( tmp_win );
     broadcast_icon_name( tmp_win );
+    broadcast_window_name( tmp_win );
 
 #if 0
 /* TODO : */
@@ -526,34 +530,34 @@ LOCAL_DEBUG_CALLER_OUT("%p, %d", asw, restart );
                startups are transparent for it */
             map_too = True ;
         }
-    }
-    /*
-     * Prevent the receipt of an UnmapNotify, since that would
-     * cause a transition to the Withdrawn state.
-     */
-    if( !ASWIN_GET_FLAGS(asw, AS_Dead) )
-    {
-        XSelectInput (dpy, asw->w, NoEventMask);
-        if( get_parent_window( asw->w ) == asw->frame )
-        {
-            ASStatusHints withdrawn_state ;
-            /* !!! Most of it has been done in datach_basic_widget : */
-            XReparentWindow (dpy, asw->w, Scr.Root, x, y);
-            withdrawn_state.flags = AS_Withdrawn ;
-            withdrawn_state.icon_window = None ;
-            set_client_state( asw->w, &withdrawn_state );
+    	/*
+     	* Prevent the receipt of an UnmapNotify, since that would
+     	* cause a transition to the Withdrawn state.
+     	*/
+    	if( !ASWIN_GET_FLAGS(asw, AS_Dead) )
+    	{
+        	XSelectInput (dpy, asw->w, NoEventMask);
+        	if( get_parent_window( asw->w ) == asw->frame )
+        	{
+            	ASStatusHints withdrawn_state ;
+            	/* !!! Most of it has been done in datach_basic_widget : */
+            	XReparentWindow (dpy, asw->w, Scr.Root, x, y);
+            	withdrawn_state.flags = AS_Withdrawn ;
+            	withdrawn_state.icon_window = None ;
+            	set_client_state( asw->w, &withdrawn_state );
 
-            if( width > 0 && height > 0 )
-                XResizeWindow( dpy, asw->w, width, height );
-            XSetWindowBorderWidth (dpy, asw->w, bw);
+            	if( width > 0 && height > 0 )
+                	XResizeWindow( dpy, asw->w, width, height );
+            	XSetWindowBorderWidth (dpy, asw->w, bw);
 
-            if( map_too )
-                XMapWindow (dpy, asw->w);
-            XSync( dpy, False );
-        }
-        /* signaling client that we no longer handle it : */
-        set_multi32bit_property (asw->w, _XA_WM_STATE, _XA_WM_STATE, 2, WithdrawnState, None);
-    }
+            	if( map_too )
+                	XMapWindow (dpy, asw->w);
+            	XSync( dpy, False );
+        	}
+        	/* signaling client that we no longer handle it : */
+        	set_multi32bit_property (asw->w, _XA_WM_STATE, _XA_WM_STATE, 2, WithdrawnState, None);
+    	}
+	}
 }
 
 /**********************************************************************/
