@@ -42,8 +42,6 @@
 ASFlagType AfterStepState = 0; /* default status */
 
 /* Config : */
-struct ASDatabase    *Database = NULL;
-
 ASVector     *Modules       = NULL;
 int           Module_fd     = 0;
 int           Module_npipes = 8;
@@ -74,6 +72,18 @@ LOCAL_DEBUG_OUT( "pid(%d),closing x_fd...", getpid() );
 LOCAL_DEBUG_OUT( "pid(%d),complete...", getpid() );
 }
 #endif
+#ifdef DEBUG_TRACE_X
+const char *window_id2name (Display * dpy, Window w, int *how_to_free)
+{
+	ASWindow *asw = window2ASWindow( w );
+	if( asw ) 
+	{
+		*how_to_free = 1 ;
+		return mystrdup(ASWIN_NAME(asw));
+	}
+	return NULL;
+}
+#endif
 
 
 /**************************************************************************/
@@ -257,7 +267,6 @@ SHOW_CHECKPOINT;
 
     /* Load config ... */
     /* read config file, set up menus, colors, fonts */
-    InitDatabase (False);
     LoadASConfig (0, PARSE_EVERYTHING);
 
     /* Reparent all the windows and setup pan frames : */
@@ -406,6 +415,9 @@ CleanupScreen()
         destroy_aswindow_list( &(Scr.Windows), True );
         ungrab_server();
     }
+
+	release_all_old_background( True );
+
     DestroyManagementWindows();
     CleanupColormaps();
 
@@ -534,7 +546,7 @@ CaptureAllWindows (ScreenInfo *scr)
     for (i = 0; i < nchildren; i++)
         if (children[i] )
         {
-            unsigned long nitems = 0;
+            long nitems = 0;
             CARD32 *state_prop = NULL ;
             int wm_state = DontCareState ;
             int k ;
@@ -705,28 +717,39 @@ LOCAL_DEBUG_CALLER_OUT( "%s restart, cmd=\"%s\"", restart?"Do":"Don't", command?
                      None, C_NO_CONTEXT, False, restart_self, NULL );
     } else
 	{
+
 	    XCloseDisplay (dpy);
 		dpy = NULL ;
 
 		/* freeing up memory */
+		DestroyPendingFunctionsQueue();
+		DestroyCategories();
+
+	    balloon_init (True);
+		destroy_asdatabase();
     	destroy_assession( Session );
 		destroy_asenvironment( &Environment );
-    	InitDatabase(True);
-    	free_func_hash ();
     	/* pixmap references */
     	build_xpm_colormap (NULL);
 
         restack_window_list(INVALID_DESK, True);
+
         clientprops_cleanup ();
         wmprops_cleanup ();
+
         free_func_hash();
+		flush_keyword_ids();
         purge_asimage_registry();
+	
 		asxml_var_cleanup();
 		custom_color_cleanup(); 
+
+        free_as_app_args();
+		free( ASDefaultScr );
+
 		flush_default_asstorage();
-        flush_ashash_memory_pool();
         flush_asbidirlist_memory_pool();
-        free( MyArgs.saved_argv );
+        flush_ashash_memory_pool();
 #ifdef DEBUG_ALLOCS
         print_unfreed_mem ();
 #endif /*DEBUG_ALLOCS */
