@@ -50,6 +50,8 @@ typedef struct ASInternalWindow
     void (*on_hilite_changed)( struct ASInternalWindow *asiw, ASMagic *data, Bool focused );
     /* ButtonPress/Release event on one of the contexts : */
     void (*on_pressure_changed)( struct ASInternalWindow *asiw, int pressed_context );
+    /* Mouse wheel scrolling event : */
+    void (*on_scroll_event)( struct ASInternalWindow *asiw, struct ASEvent *event );
     /* Motion notify : */
     void (*on_pointer_event)( struct ASInternalWindow *asiw, struct ASEvent *event );
     /* KeyPress/Release : */
@@ -106,6 +108,7 @@ typedef struct ASWindow
 #define ASWIN_CLASS(t)      ((t)->hints->res_class)
 #define ASWIN_RES_NAME(t)   ((t)->hints->res_name)
 #define ASWIN_ICON_NAME(t)  ((t)->hints->icon_name)
+#define ASWIN_ICON_NAME_ENC(t)  (get_hint_name_encoding((t)->hints, (t)->hints->icon_name_idx))
 #define ASWIN_DESK(t)       ((t)->status->desktop)
 #define ASWIN_LAYER(t)      ((t)->status->layer)
 #define ASWIN_HFLAGS(t,f) 	get_flags((t)->hints->flags,(f))
@@ -313,6 +316,7 @@ typedef struct module_t
   int                   fd;
   int                   active;
   char                 *name;
+  char                 *cmd_line;
   CARD32                mask;
   CARD32                lock_on_send_mask;
   queue_buff_struct    *output_queue;
@@ -452,6 +456,7 @@ void on_window_moveresize( ASWindow *asw, Window w );
 void on_icon_changed( ASWindow *asw );
 void on_window_title_changed( ASWindow *asw, Bool update_display );
 void on_window_hints_changed( ASWindow *asw );
+void on_window_opacity_changed( ASWindow *asw );
 void on_window_status_changed( ASWindow *asw, Bool update_display, Bool reconfigured );
 void on_window_hilite_changed( ASWindow *asw, Bool focused );
 void on_window_pressure_changed( ASWindow *asw, int pressed_context );
@@ -521,6 +526,7 @@ ASFlagType compile_titlebuttons_mask (ASHints *hints);
 void estimate_titlebar_size( ASHints *hints, unsigned int *width_ret, unsigned int *height_ret );
 void disable_titlebuttons_with_function (ASWindow * t, int function);
 Bool hints2decorations( ASWindow *asw, ASHints *old_hints );
+void invalidate_window_mystyles( ASWindow *asw );
 
 /*************************** events.c ********************************/
 const char *context2text(int ctx);
@@ -570,6 +576,7 @@ void ExecuteFunctionForClient(struct FunctionData *data, Window client);
 void ExecuteFunctionExt (struct FunctionData *data, struct ASEvent *event, int module, Bool defered);
 /* execute all the scheduled functions from the queue */
 void ExecutePendingFunctions();
+void DestroyPendingFunctionsQueue();
 /* non-window specific, non-defferrable functions are run : */
 void ExecuteBatch ( ComplexFunction *batch );
 
@@ -645,13 +652,20 @@ void broadcast_config (send_data_type event_type, ASWindow * t);
 void HandleModuleInOut(unsigned int channel, Bool has_input, Bool has_output);
 
 void KillModuleByName (char *name);
+void KillAllModulesByName (char *name);
+
 int FindModuleByName (char *name);
+char *GetModuleCmdLineByName(char *name);
 void DeadPipe (int nonsense);
 void ShutdownModules(Bool dont_free_memory);
 
 void RunCommand (FunctionData * fdata, unsigned int channel, Window w);
 
 void FlushAllQueues();
+MenuData *make_module_menu( FunctionCode func, const char *title, int sort_order );
+#define make_stop_module_menu(sort_order)  make_module_menu(  F_KILLMODULEBYNAME, "Stop Running Module", sort_order )
+#define make_restart_module_menu(sort_order)  make_module_menu(  F_RESTARTMODULEBYNAME, "Restart Running Module", sort_order )
+
 
 /******************************* outline.c ********************************/
 void MoveOutline( struct MoveResizeData * pdata );
@@ -670,6 +684,7 @@ void change_desktop_background( int desk );
 void HandleBackgroundRequest( struct ASEvent *event );
 Bool is_background_xfer_ximage( unsigned long id );
 void stop_all_background_xfer();
+void release_all_old_background( Bool forget );
 
 
 
