@@ -75,6 +75,7 @@ struct SaveWindowAuxData
     char  this_host[MAXHOSTNAME];
     FILE *f;
 	ASHashTable *res_name_counts;
+	Bool only_modules ;
 };
 
 int
@@ -135,6 +136,132 @@ char *pid2cmd ( int pid )
 	return cmd;
 }
 
+static 
+char *filter_out_geometry( char *cmd_args, char **geom_keyword, char **original_size )
+{
+	char *clean = mystrdup(cmd_args); 
+	char *token_start ; 
+	char *token_end = clean;
+	
+	do
+	{
+		token_start = token_end ;
+  
+		if( token_start[0] == '-' ) 
+		{	
+			if( strncmp( token_start, "-g ", 3 ) == 0 ||
+				strncmp( token_start, "-geometry ", 10 ) == 0 ||
+				strncmp( token_start, "--geometry ", 11 ) == 0 )
+			{
+				unsigned int width = 0 ;
+  				unsigned int height = 0 ;
+				int flags = 0 ;
+				char *geom_start = token_start ; 
+				if( geom_keyword ) 
+					*geom_keyword = tokencpy (geom_start);
+		 		token_end = tokenskip( token_start, 1 );
+				if( token_end > token_start ) 
+				{	
+					token_start = token_end ;
+					token_end = parse_geometry(token_start, NULL, NULL, &width, &height, &flags ); 
+					if( token_end > token_start ) 
+					{
+						int i = 0; 
+						for( i = 0 ; token_end[i] != '\0' ; ++i)
+							geom_start[i] = token_end[i] ;
+						geom_start[i]	= '\0' ;
+					}
+				}
+				if( original_size && get_flags(flags, WidthValue|HeightValue) )
+				{
+					char *tmp = *original_size = safemalloc(64);
+					if( get_flags(flags, WidthValue) && width > 0 )
+					{	
+						sprintf( tmp, "%d", width );
+						while( *tmp ) ++tmp;
+					}
+					if( get_flags(flags, HeightValue) && height > 0 )
+						sprintf( tmp, "x%d", height );
+				}	 
+				break;
+			}
+		}
+		token_end = tokenskip( token_start, 1 );
+	}while( token_end > token_start );
+	return clean;
+}	 
+
+static void 
+stripreplace_geometry_size( char **pgeom, char *original_size )
+{
+	char *geom = *pgeom ;
+	if( geom ) 
+	{
+		
+		int i = 0 ; 
+		if( isdigit(geom[0]) )
+			while( geom[++i] ) if( geom[i] == '+' || geom[i] == '-' ) break;
+		if( original_size != NULL || i > 0 )
+		{
+			int size = strlen( &geom[i] );
+			if( original_size ) 
+				size += strlen(original_size);
+			*pgeom = safemalloc( size + 1 );
+			if( original_size ) 
+				sprintf( *pgeom, "%s%s", original_size, &geom[i] );
+			else
+				strcpy( *pgeom, &geom[i] );
+			free( geom );		 	  
+		}	 
+	}		  
+}
+
+static char *
+make_application_name( ASWindow *asw, char *rclass, char *rname )
+{
+	char *name = ASWIN_NAME(asw);
+	/* need to check if we can use window name in the pattern. It has to be :
+		* 1) Not changed since window initial mapping
+		* 2) all ASCII
+		* 3) shorter then 80 chars
+		* 4) must not match class or res_name
+		* 5) Unique
+		*/
+	if( name == rname || name == rclass ) 
+		name = NULL ;
+	else if( name != NULL && get_flags( asw->internal_flags, ASWF_NameChanged ) ) 
+	{/* allow changed names only for terms, as those could launch sub app inside */
+		int rclass_len = rclass?strlen(rclass):0;
+		if( rclass_len != 5 )
+			name = NULL ;
+		else if( strstr( rclass+1, "term" ) != 0 ) 	  
+			name = NULL ;
+	}
+	if(	name )
+	{
+		int i  = 0;
+		while( name[i] != '\0' )
+		{   /* we do not want to have path in names as well */
+			if( !isalnum(name[i]) && !isspace(name[i]))
+				break;
+			if( ++i >= 80 )
+				break;
+		}
+		if( name[i] != '\0' )
+			name = NULL ;
+		else
+		{
+			if( strcmp( rclass, name ) == 0 ||
+				strcmp( rname, name ) == 0 	)
+				name = NULL ;
+			else                   /* check that its unique */
+				if( !check_aswindow_name_unique( name, asw ) )
+					name = NULL ;
+		}
+	}
+	return name;
+}	  
+
 Bool
 make_aswindow_cmd_iter_func(void *data, void *aux_data)
 {
@@ -148,6 +275,10 @@ make_aswindow_cmd_iter_func(void *data, void *aux_data)
 
 		/* don't want to save modules - those are started from autoexec anyways */
 		if( ASWIN_HFLAGS(asw,AS_Module) ) 
+		{	
+			if( !swad->only_modules	)
+				return True;
+		}else if( swad->only_modules	)
 			return True;
 
 		if( asw->hints->client_cmd == NULL && same_host )
@@ -167,40 +298,16 @@ make_aswindow_cmd_iter_func(void *data, void *aux_data)
 			int app_no = get_res_name_count( swad->res_name_counts, asw->hints->res_name );
 			char *rname = asw->hints->res_name?asw->hints->res_name:"*" ;
 			char *rclass = asw->hints->res_class?asw->hints->res_class:"*" ;
-			char *name = get_flags( asw->internal_flags, ASWF_NameChanged )?NULL:ASWIN_NAME(asw);
-			int i = 0;
+			char *name = ASWIN_NAME(asw);
 			char *app_name = "*" ;
 			char *cmd_app = NULL, *cmd_args ;
-			/* need to check if we can use window name in the pattern. It has to be :
-				* 1) Not changed since window initial mapping
-				* 2) all ASCII
-				* 3) shorter then 80 chars
-				* 4) must not match class or res_name
-				* 5) Unique
-				*/
-			if( name == rname || name == rclass ) 
-				name = NULL ;
-			if(	name )
-			{
-				while( name[i] != '\0' )
-				{
-					if( !isascii(name[i]) )
-						break;
-					if( ++i >= 80 )
-						break;
-				}
-				if( name[i] != '\0' )
-					name = NULL ;
-				else
-				{
-					if( strcmp( rclass, name ) == 0 ||
-						strcmp( rname, name ) == 0 	)
-						name = NULL ;
-					else                   /* check that its unique */
-						if( !check_aswindow_name_unique( name, asw ) )
-							name = NULL ;
-				}
-			}
+			Bool supports_geometry = False ;
+			char *geometry_keyword = NULL ; 
+			char *clean_cmd_args = NULL ;
+			char *original_size = NULL ;
+
+			name = make_application_name( asw, rclass, rname );
+			
 			if( name == NULL )
 			{
 				app_name = safemalloc( strlen(rclass)+1+strlen(rname)+1+1+15+1 );
@@ -212,47 +319,90 @@ make_aswindow_cmd_iter_func(void *data, void *aux_data)
 			}
 				
 			cmd_args = parse_token(asw->hints->client_cmd, &cmd_app );
+			if( cmd_app )  /* we want -geometry to be the first arg, so that terms could correctly launch app with -e arg */
+				clean_cmd_args = filter_out_geometry( cmd_args, &geometry_keyword, &original_size ) ;
+			
+			if( geometry_keyword == NULL ) 
+				geometry_keyword = mystrdup(ASWIN_HFLAGS(asw,AS_Module)?"--geometry":"-geometry"); 
+			else
+				supports_geometry = True ;
+			
+			if( !ASWIN_HFLAGS(asw,AS_Handles) )
+			{   /* we want to remove size from geometry here, 
+				 * unless it was requested in original cmd-line geometry */
+				stripreplace_geometry_size( &geom, original_size );
+			}	 
+			
 			if( cmd_app ) 
-			{   /* we want -geometry to be the first arg, so that terms could correctly launch app with -e arg */
-				fprintf( swad->f, 	"\tExec \"I:%s\" %s -geometry %s %s &\n", app_name, cmd_app, geom, cmd_args );
-				free( cmd_app );	
+			{   
+				fprintf( swad->f, 	"\tExec \"I:%s\" %s %s %s %s &\n", app_name, cmd_app, geometry_keyword, geom, clean_cmd_args );
+				destroy_string(&cmd_app);	
 			}else
-	            fprintf( swad->f, 	"\tExec \"I:%s\" %s -geometry %s &\n", app_name, asw->hints->client_cmd, geom );
-			if( ASWIN_GET_FLAGS(asw,AS_Sticky) )
-			{	
-				fprintf( swad->f, 	"\tWait \"I:%s\" DefaultGeometry %s"
-							        ", Layer %d"
-									", Sticky"
-									", StartsOnDesk %d"
-									", %s"
-									"\n",
-						     		app_name, pure_geometry,
-							 		ASWIN_LAYER(asw),
-									ASWIN_DESK(asw),
-									ASWIN_GET_FLAGS(asw,AS_Iconic)?"StartIconic":"StartNormal");
+	            fprintf( swad->f, 	"\tExec \"I:%s\" %s %s %s &\n", app_name, asw->hints->client_cmd, geometry_keyword, geom );
+			
+			destroy_string(&clean_cmd_args);
+			destroy_string(&geometry_keyword);
+			destroy_string(&original_size);
+
+			if( ASWIN_HFLAGS(asw,AS_Module) ) 
+			{
+				fprintf( swad->f, "\tWait \"I:%s\" Layer %d\n", app_name, ASWIN_LAYER(asw));
+			}else if( ASWIN_GET_FLAGS(asw,AS_Sticky) )
+			{
+				if( supports_geometry ) 	  
+					fprintf( swad->f, 	"\tWait \"I:%s\" Layer %d"
+										", Sticky"
+										", StartsOnDesk %d"
+										", %s"
+										"\n",
+						     			app_name, ASWIN_LAYER(asw),
+										ASWIN_DESK(asw),
+										ASWIN_GET_FLAGS(asw,AS_Iconic)?"StartIconic":"StartNormal");
+				else
+					fprintf( swad->f, 	"\tWait \"I:%s\" DefaultGeometry %s"
+							        	", Layer %d"
+										", Sticky"
+										", StartsOnDesk %d"
+										", %s"
+										"\n",
+						     			app_name, pure_geometry,
+							 			ASWIN_LAYER(asw),
+										ASWIN_DESK(asw),
+										ASWIN_GET_FLAGS(asw,AS_Iconic)?"StartIconic":"StartNormal");
 			}else
 			{
-				
-				fprintf( swad->f, 	"\tWait \"I:%s\" DefaultGeometry %s"
-							        ", Layer %d"
-									", Slippery"
-									", StartsOnDesk %d"
-									", ViewportX %d"
-									", ViewportY %d"
-									", %s"
-									"\n",
-						     		app_name, pure_geometry,
-							 		ASWIN_LAYER(asw),
-									ASWIN_DESK(asw),
-									((asw->status->x + asw->status->viewport_x) / Scr.MyDisplayWidth)*Scr.MyDisplayWidth,
-									((asw->status->y + asw->status->viewport_y) / Scr.MyDisplayHeight)*Scr.MyDisplayHeight,
-									ASWIN_GET_FLAGS(asw,AS_Iconic)?"StartIconic":"StartNormal");
+				if( supports_geometry ) 
+					fprintf( swad->f, 	"\tWait \"I:%s\" Layer %d"
+										", Slippery"
+										", StartsOnDesk %d"
+										", ViewportX %d"
+										", ViewportY %d"
+										", %s"
+										"\n",
+						     			app_name, ASWIN_LAYER(asw),
+										ASWIN_DESK(asw),
+										((asw->status->x + asw->status->viewport_x) / Scr.MyDisplayWidth)*Scr.MyDisplayWidth,
+										((asw->status->y + asw->status->viewport_y) / Scr.MyDisplayHeight)*Scr.MyDisplayHeight,
+										ASWIN_GET_FLAGS(asw,AS_Iconic)?"StartIconic":"StartNormal");
+				else										   
+					fprintf( swad->f, 	"\tWait \"I:%s\" DefaultGeometry %s"
+							        	", Layer %d"
+										", Slippery"
+										", StartsOnDesk %d"
+										", ViewportX %d"
+										", ViewportY %d"
+										", %s"
+										"\n",
+						     			app_name, pure_geometry,
+							 			ASWIN_LAYER(asw),
+										ASWIN_DESK(asw),
+										((asw->status->x + asw->status->viewport_x) / Scr.MyDisplayWidth)*Scr.MyDisplayWidth,
+										((asw->status->y + asw->status->viewport_y) / Scr.MyDisplayHeight)*Scr.MyDisplayHeight,
+										ASWIN_GET_FLAGS(asw,AS_Iconic)?"StartIconic":"StartNormal");
 			}	 
-			if( pure_geometry ) 
-				free( pure_geometry );
-			if( geom )
-				free( geom );
-			free( app_name );
+			destroy_string(&pure_geometry);
+			destroy_string(&geom);
+			destroy_string(&app_name);
         }
         return True;
     }
@@ -293,6 +443,13 @@ save_aswindow_list( ASWindowList *list, char *file )
 	if( swad.f )
     {
 		fprintf( swad.f, "Function \"WorkspaceState\"\n" );
+		swad.only_modules = False ;
+		swad.res_name_counts = create_ashash(0, string_hash_value, string_compare, string_destroy);
+        iterate_asbidirlist( list->clients, make_aswindow_cmd_iter_func, &swad, NULL, False );
+		destroy_ashash( &(swad.res_name_counts) );
+		fprintf( swad.f, "EndFunction\n\n" );
+		fprintf( swad.f, "Function \"WorkspaceModules\"\n" );
+		swad.only_modules = True ;
 		swad.res_name_counts = create_ashash(0, string_hash_value, string_compare, string_destroy);
         iterate_asbidirlist( list->clients, make_aswindow_cmd_iter_func, &swad, NULL, False );
 		destroy_ashash( &(swad.res_name_counts) );
@@ -1543,12 +1700,18 @@ LOCAL_DEBUG_OUT("checking i(%d)->end_i(%d)->dir(%d)->AutoReverse(%d)", i, end_i,
 /********************************************************************************/
 /* window list menus regeneration :                                             */
 /********************************************************************************/
-static int
-make_desk_winlist_menu_strcmp(const void *a, const void *b) {
-	FunctionData *fda = *(FunctionData **)a;
-	FunctionData *fdb = *(FunctionData **)b;
-
-	return strcmp(fda->name ? fda->name : "", fdb->name ? fdb->name : "");
+static inline void
+ASWindow2func_data( FunctionCode func, ASWindow *asw, FunctionData *fdata, char *scut, Bool icon_name ) 
+{
+    fdata->func = F_RAISE_IT;
+    fdata->name = mystrdup(icon_name ? ASWIN_ICON_NAME(asw) : ASWIN_NAME(asw));
+	if( !icon_name )
+	 	fdata->name_encoding = ASWIN_NAME_ENCODING(asw) ;
+    fdata->func_val[0] = (long) asw;
+    fdata->func_val[1] = (long) asw->w;
+	if (++(*scut) == ('9' + 1))
+		(*scut) = 'A';		/* Next shortcut key */
+    fdata->hotkey = (*scut);
 }
 
 MenuData *
@@ -1556,79 +1719,67 @@ make_desk_winlist_menu(  ASWindowList *list, int desk, int sort_order, Bool icon
 {
     char menu_name[256];
     MenuData *md ;
+    MenuDataItem *mdi ;
     FunctionData  fdata ;
     char          scut = '0';                  /* Current short cut key */
+    ASWindow **clients;
+    int i, max_i;
 
     if( list == NULL )
         return NULL;;
+    
+	clients = PVECTOR_HEAD(ASWindow*,list->circulate_list);
+    max_i = PVECTOR_USED(list->circulate_list);
 
     if( IsValidDesk( desk ) )
         sprintf( menu_name, "Windows on Desktop #%d", desk );
     else
         sprintf( menu_name, "Windows on All Desktops" );
 
-    if( (md = CreateMenuData (menu_name)) == NULL )
+    if( (md = create_menu_data (menu_name)) == NULL )
         return NULL;
 
-    purge_menu_data_items( md );
-
     memset(&fdata, 0x00, sizeof(FunctionData));
     fdata.func = F_TITLE ;
     fdata.name = mystrdup(menu_name);
     add_menu_fdata_item( md, &fdata, NULL, NULL );
 
-    if( sort_order == ASO_Circulation )
-    {
-        ASWindow **clients = PVECTOR_HEAD(ASWindow*,list->circulate_list);
-        int i = -1, max_i = PVECTOR_USED(list->circulate_list);
-        while( ++i < max_i )
+	if( sort_order == ASO_Alpha ) 
+	{
+		FunctionData **menuitems = safecalloc(sizeof(FunctionData *), max_i);
+		int numitems = 0;
+        
+		for( i = 0 ; i < max_i ; ++i ) 
+		{
+            if ((ASWIN_DESK(clients[i]) == desk || !IsValidDesk(desk)) && !ASWIN_HFLAGS(clients[i], AS_SkipWinList)) 
+			{
+				menuitems[numitems] = safecalloc(1, sizeof(FunctionData));
+				ASWindow2func_data( F_RAISE_IT, clients[i], menuitems[numitems], &scut, icon_name ); 
+				++numitems;
+            }
+        }
+		qsort(menuitems, numitems, sizeof(FunctionData *), compare_func_data_name);
+		for( i = 0 ; i < numitems ; ++i ) 
+		{
+			if( (mdi = add_menu_fdata_item( md, menuitems[i], NULL, 
+								 get_flags( Scr.Feel.flags, WinListHideIcons) ? NULL : 
+								 get_client_icon_image(ASDefaultScr, clients[i]->hints))) != NULL )
+				set_flags( mdi->flags, MD_ScaleMinipixmapDown );
+			safefree(menuitems[i]); /* scrubba-dub-dub */
+		}
+		safefree(menuitems);
+    } else /* if( sort_order == ASO_Circulation || sort_order == ASO_Stacking ) */
+	{
+        for( i = 0 ; i < max_i ; ++i ) 
         {
             if ((ASWIN_DESK(clients[i]) == desk || !IsValidDesk(desk)) && !ASWIN_HFLAGS(clients[i], AS_SkipWinList))
 			{
-                fdata.func = F_RAISE_IT ;
-                fdata.name = mystrdup(icon_name? ASWIN_ICON_NAME(clients[i]) : ASWIN_NAME(clients[i]));
-				if( !icon_name )
-	 				fdata.name_encoding = ASWIN_NAME_ENCODING(clients[i]) ;
-				LOCAL_DEBUG_OUT( "item's encoding = %d, name = \"%s\"", fdata.name_encoding,
-				fdata.name );
-                fdata.func_val[0] = (long) clients[i];
-                fdata.func_val[1] = (long) clients[i]->w;
-		if (++scut == ('9' + 1))
-			scut = 'A';		   /* Next shortcut key */
-                fdata.hotkey = scut;
-                add_menu_fdata_item( md, &fdata, NULL, get_flags( Scr.Feel.flags, WinListHideIcons)? NULL : get_window_icon_image(clients[i]));
+				ASWindow2func_data( F_RAISE_IT, clients[i], &fdata, &scut, icon_name ); 
+                if( (mdi = add_menu_fdata_item( md, &fdata, NULL, get_flags( Scr.Feel.flags, WinListHideIcons)? NULL : 
+									 get_client_icon_image( ASDefaultScr, clients[i]->hints))) != NULL ) 
+					set_flags( mdi->flags, MD_ScaleMinipixmapDown );									 	
             }
         }
-    } else if( sort_order == ASO_Alpha ) {
-        ASWindow **clients = PVECTOR_HEAD(ASWindow*,list->circulate_list);
-        int i = -1, max_i = PVECTOR_USED(list->circulate_list);
-	FunctionData **menuitems = safecalloc(sizeof(FunctionData *), max_i);
-    	FunctionData *sfdata;
-	int numitems = 0;
-        while( ++i < max_i ) {
-            if ((ASWIN_DESK(clients[i]) == desk || !IsValidDesk(desk)) && !ASWIN_HFLAGS(clients[i], AS_SkipWinList)) {
-		sfdata = safecalloc(1, sizeof(FunctionData));
-                sfdata->func = F_RAISE_IT;
-                sfdata->name = mystrdup(icon_name ? ASWIN_ICON_NAME(clients[i]) : ASWIN_NAME(clients[i]));
-				if( !icon_name )
-	 				sfdata->name_encoding = ASWIN_NAME_ENCODING(clients[i]) ;
-                sfdata->func_val[0] = (long) clients[i];
-                sfdata->func_val[1] = (long) clients[i]->w;
-		if (++scut == ('9' + 1))
-			scut = 'A';		/* Next shortcut key */
-                sfdata->hotkey = scut;
-		menuitems[numitems++] = sfdata;
-            }
-        }
-	qsort(menuitems, numitems, sizeof(FunctionData *), make_desk_winlist_menu_strcmp);
-	i = 0;
-	while (i < numitems) {
-		add_menu_fdata_item( md, menuitems[i], NULL, get_flags( Scr.Feel.flags, WinListHideIcons) ? NULL : get_window_icon_image(clients[i]));
-		safefree(menuitems[i++]); /* scrubba-dub-dub */
-	}
-	safefree(menuitems);
-    } else if( sort_order == ASO_Stacking ) {
-    } else {
     }
     return md;
 }
