@@ -26,6 +26,7 @@
 #include "../../configure.h"
 
 #include "asinternals.h"
+#include "../../libAfterConf/afterconf.h"
 
 /********************************************************************/
 /* ASWindow frame decorations :                                     */
@@ -364,7 +365,8 @@ LOCAL_DEBUG_OUT( "--DESTR Client(%lx(%s))->CLIENT->canvas(%p)->window(%lx)", asw
          * since that would cause a transition to the Withdrawn state.
 		 */
 #if 1
-        if( !ASWIN_GET_FLAGS(asw, AS_Dead) && get_parent_window( w ) == asw->frame )
+        if( (asw->status == NULL || !get_flags(asw->status->flags, AS_Dead)) && 
+			get_parent_window( w ) == asw->frame )
         {
             if( get_flags( AfterStepState, ASS_Shutdown ) )
                 quietly_reparent_window( w, Scr.Root, xwc.x, xwc.y, AS_CLIENT_EVENT_MASK );
@@ -399,7 +401,7 @@ check_icon_canvas( ASWindow *asw, Bool required )
             attributes.border_pixel = Scr.asv->black_pixel;
             attributes.cursor = Scr.Feel.cursors[ASCUR_Default];
 
-            if ((ASWIN_HFLAGS(asw, AS_ClientIcon|AS_ClientIconPixmap) != AS_ClientIcon) ||
+            if ((get_flags(asw->hints->client_icon_flags, AS_ClientIcon|AS_ClientIconPixmap) != AS_ClientIcon) ||
                  asw->hints == NULL || asw->hints->icon.window == None ||
                  !get_flags(Scr.Feel.flags, KeepIconWindows))
             { /* create windows */
@@ -422,6 +424,8 @@ LOCAL_DEBUG_OUT( "++CREAT Client(%lx(%s))->ICON->canvas(%p)->window(%lx)", asw->
     {                                          /* destroy canvas here */
         w = canvas->w ;
 LOCAL_DEBUG_OUT( "--DESTR Client(%lx(%s))->ICON->canvas(%p)->window(%lx)", asw->w, ASWIN_NAME(asw)?ASWIN_NAME(asw):"noname", canvas, canvas->w );
+		if( asw->icon_title_canvas == canvas ) 
+			asw->icon_title_canvas = NULL ;
         destroy_ascanvas( &canvas );
         if( asw->hints && asw->hints->icon.window == w )
             unregister_aswindow( w );
@@ -478,42 +482,6 @@ LOCAL_DEBUG_OUT( "++CREAT Client(%lx(%s))->ICONT->canvas(%p)->window(%lx)", asw-
     return canvas;
 }
 
-ASImage*
-get_window_icon_image( ASWindow *asw )
-{
-	ASImage *im = NULL ;
-    if( asw )
-    {
-        if( ASWIN_HFLAGS( asw, AS_ClientIcon|AS_ClientIconPixmap ) == (AS_ClientIcon|AS_ClientIconPixmap) &&
-            get_flags( Scr.Feel.flags, KeepIconWindows )&&
-            asw->hints->icon.pixmap != None )
-        {/* convert client's icon into ASImage */
-            unsigned int width, height ;
-            get_drawable_size( asw->hints->icon.pixmap, &width, &height );
-            im = picture2asimage( Scr.asv, asw->hints->icon.pixmap,
-                                        asw->hints->icon_mask,
-                                        0, 0, width, height,
-                                        0xFFFFFFFF, False, 100 );
-            LOCAL_DEBUG_OUT( "converted client's pixmap into an icon %dx%d %p", width, height, im );
-        }
-        /* TODO: we also need to check for newfashioned ARGB icon from
-        * extended WM specs here
-        */
-        if( im == NULL )
-        {
-            if( asw->hints->icon_file )
-            {
-                im = get_asimage( Scr.image_manager, asw->hints->icon_file, 0xFFFFFFFF, 100 );
-                LOCAL_DEBUG_OUT( "loaded icon from \"%s\" into %dx%d %p", asw->hints->icon_file, im?im->width:0, im?im->height:0, im );
-            }else
-            {
-                LOCAL_DEBUG_OUT( "no icon to use %s", "" );
-            }
-        }
-    }
-	return im;
-}
-
 static ASTBarData*
 check_tbar( ASTBarData **tbar, Bool required, const char *mystyle_name,
             ASImage *img, unsigned short back_w, unsigned short back_h,
@@ -568,6 +536,22 @@ invalidate_window_icon( ASWindow *asw )
         check_icon_canvas( asw, False );
 }
 
+void 
+invalidate_window_mystyles( ASWindow *asw )
+{
+	int i ; 
+	
+	for( i = 0 ; i < FRAME_PARTS ; ++i ) 
+		if( asw->frame_bars[i] ) 
+			invalidate_astbar_style (asw->frame_bars[i], -1);
+    if( asw->tbar )
+		invalidate_astbar_style (asw->tbar, -1);
+    if( asw->icon_button )
+		invalidate_astbar_style (asw->icon_button, -1);
+	if( asw->icon_title )
+		invalidate_astbar_style (asw->icon_title, -1);
+}
+
 /***********************************************************************
  *  grab_aswindow_buttons - grab needed buttons for all of the windows
  *  for specified client
@@ -776,19 +760,26 @@ check_allowed_function (FunctionData *fdata, ASHints *hints)
     int i ;
     ComplexFunction *cfunc ;
 
-    if (func != F_FUNCTION)
+    if ( func != F_FUNCTION && func != F_CATEGORY )
         return check_allowed_function2 (func, hints);
 
-    if( (cfunc = get_complex_function( fdata->name ) ) == NULL )
-        return 0;
+	if( func == F_FUNCTION )
+	{	
+    	if( (cfunc = get_complex_function( fdata->name ) ) == NULL )
+        	return 0;
 
-    for( i = 0 ; i < cfunc->items_num ; ++i )
-        if( cfunc->items[i].func == F_FUNCTION )
-        {
-            if( check_allowed_function (&(cfunc->items[i]), hints) == 0 )
-                return 0;
-        }else if( check_allowed_function2 (cfunc->items[i].func, hints) == 0 )
-            return 0;
+    	for( i = 0 ; i < cfunc->items_num ; ++i )
+        	if( cfunc->items[i].func == F_FUNCTION || cfunc->items[i].func == F_CATEGORY )
+        	{
+            	if( check_allowed_function (&(cfunc->items[i]), hints) == 0 )
+                	return 0;
+        	}else if( check_allowed_function2 (cfunc->items[i].func, hints) == 0 )
+            	return 0;
+	}else if( func == F_CATEGORY ) 
+	{
+		if( name2desktop_category( fdata->text?fdata->text:fdata->name, NULL ) == NULL ) 
+			return 0;
+	}	 
     return 1;
 }
 
@@ -898,7 +889,7 @@ hints2decorations( ASWindow *asw, ASHints *old_hints )
     ASOrientation *od = get_orientation_data(asw);
     char *mystyle_name = Scr.Look.MSWindow[BACK_FOCUSED]->name;
     char *frame_mystyle_name = NULL ;
-    int *frame_contexts  = &(od->frame_contexts[0]);
+    unsigned int *frame_contexts  = &(od->frame_contexts[0]);
 	Bool tbar_created = False;
 	Bool status_changed = False ;
 
@@ -908,6 +899,7 @@ hints2decorations( ASWindow *asw, ASHints *old_hints )
     check_icon_canvas( asw, (ASWIN_HFLAGS( asw, AS_Icon) && !get_flags(Scr.Feel.flags, SuppressIcons)) );
 
 	/* 4) we need to prepare icon title window : */
+	LOCAL_DEBUG_CALLER_OUT( "checking icon title canvas(%p)", asw->icon_title_canvas );
 	check_icon_title_canvas( asw, (ASWIN_HFLAGS( asw, AS_IconTitle) &&
                                    get_flags(Scr.Feel.flags, IconTitle) &&
 								  !get_flags(Scr.Feel.flags, SuppressIcons)),
@@ -965,7 +957,7 @@ hints2decorations( ASWindow *asw, ASHints *old_hints )
 	{
 		if( old_hints )
 		{
-			icon_image_changed = (ASWIN_HFLAGS( asw, AS_ClientIcon|AS_ClientIconPixmap ) != get_flags(old_hints->flags, AS_ClientIcon|AS_ClientIconPixmap));
+			icon_image_changed = (get_flags(asw->hints->client_icon_flags, AS_ClientIcon|AS_ClientIconPixmap ) != get_flags(old_hints->client_icon_flags, AS_ClientIcon|AS_ClientIconPixmap));
 			if( !icon_image_changed )
         		icon_image_changed = (asw->hints->icon.pixmap != old_hints->icon.pixmap );
 			if( !icon_image_changed )
@@ -974,7 +966,7 @@ hints2decorations( ASWindow *asw, ASHints *old_hints )
 
 		if( icon_image_changed )
 		{
-			ASImage *icon_image = get_window_icon_image( asw );
+			ASImage *icon_image = get_client_icon_image( ASDefaultScr, asw->hints ); 
     		check_tbar( &(asw->icon_button), (asw->icon_canvas != NULL), AS_ICON_MYSTYLE,
             		    icon_image, Scr.Look.ButtonWidth, Scr.Look.ButtonHeight,/* scaling icon image */
                 		0, Scr.Look.ButtonAlign,
@@ -1007,8 +999,7 @@ hints2decorations( ASWindow *asw, ASHints *old_hints )
 	    add_astbar_label( asw->icon_title, 0, 0, 0, frame->title_align,
 							DEFAULT_TBAR_HSPACING, DEFAULT_TBAR_VSPACING,
 						    ASWIN_ICON_NAME(asw),
-							(asw->hints->icon_name_idx < 0)?AS_Text_ASCII :
-							asw->hints->names_encoding[asw->hints->icon_name_idx]);
+							ASWIN_ICON_NAME_ENC(asw));
 	    set_astbar_style( asw->icon_title, BAR_STATE_FOCUSED, AS_ICON_TITLE_MYSTYLE );
   	    set_astbar_style( asw->icon_title, BAR_STATE_UNFOCUSED, AS_ICON_TITLE_UNFOCUS_MYSTYLE );
     }
@@ -1141,7 +1132,7 @@ hints2decorations( ASWindow *asw, ASHints *old_hints )
 			{
 				ASCanvas *canvas = ASWIN_HFLAGS(asw, AS_VerticalTitle)?asw->frame_sides[FR_W]:asw->frame_sides[FR_N];
 	        	/* label ( goes on top of above pixmap ) */
-	        	if( change_astbar_first_label( asw->tbar, asw->hints->names[0], asw->hints->names_encoding[0] ) )
+	        	if( change_astbar_first_label( asw->tbar, ASWIN_NAME(asw), ASWIN_NAME_ENCODING(asw) ) )
   		        	if( canvas )
       		    	{
 	                	render_astbar( asw->tbar, canvas );
@@ -1214,7 +1205,7 @@ hints2decorations( ASWindow *asw, ASHints *old_hints )
                           tbar_layout_row[ASO_TBAR_ELEM_LBL],
                           od->flip,
                           title_align, frame->title_h_spacing, frame->title_v_spacing,
-                          asw->hints->names[0], asw->hints->names_encoding[0]);
+                          ASWIN_NAME(asw), ASWIN_NAME_ENCODING(asw));
 		
 			/* all the buttons go after the label to be rendered over it */
 			/* left buttons : */
@@ -1250,6 +1241,12 @@ hints2decorations( ASWindow *asw, ASHints *old_hints )
       		}
 		}
 	}
+	
+	if( ASWIN_HFLAGS( asw, AS_WindowOpacity ) )
+	{
+		set_32bit_property (asw->frame, _XA_NET_WM_WINDOW_OPACITY, XA_CARDINAL, asw->hints->window_opacity );
+	}	 
+
 
     /* we also need to setup label, unfocused/sticky style and tbar sizes -
      * it all is done when we change windows state, or move/resize it */
@@ -1268,6 +1265,7 @@ hints2decorations( ASWindow *asw, ASHints *old_hints )
 			map_canvas_window( asw->icon_title_canvas, False );
 		}
 	}
+
 	return	status_changed ;
 }
 
