@@ -60,7 +60,7 @@
  *  DispatchEvent - calls appropriate handler for the event
  ************************************************************************/
 void DigestEvent    ( ASEvent *event );
-void afterstep_wait_pipes_input ();
+void afterstep_wait_pipes_input (int timeout_sec);
 
 void
 HandleEvents ()
@@ -88,7 +88,7 @@ HandleEvents ()
             }
             ExecutePendingFunctions();
         }
-        afterstep_wait_pipes_input ();
+        afterstep_wait_pipes_input (0);
         ExecutePendingFunctions();
     }
 }
@@ -121,7 +121,7 @@ LOCAL_DEBUG_OUT( "checking masked events ...%s", "" );
             if( Scr.moveresize_in_progress == NULL )
                 return;
         }
-        afterstep_wait_pipes_input ();
+        afterstep_wait_pipes_input (0);
     }
 }
 
@@ -254,6 +254,7 @@ WaitWindowLoop( char *pattern, long timeout )
 	text = safemalloc(64+strlen(pattern)+1);
 	sprintf( text, "Waiting for window matching \"%s\" ... Press button to cancel.", pattern );
 	hint = create_ashint_window( ASDefaultScr, &(Scr.Look), text);
+	free( text );
 	while (!done)
 	{
 		do
@@ -293,7 +294,7 @@ WaitWindowLoop( char *pattern, long timeout )
 
 		if( time(NULL) > end_time )
 			break;
-		afterstep_wait_pipes_input ();
+		afterstep_wait_pipes_input (1);
 	}
 	destroy_ashint_window( &hint );
     return NULL;
@@ -362,8 +363,15 @@ DigestEvent( ASEvent *event )
 			event->client = NULL ;
 		}else
 		{
-			event->widget = NULL ;	
-			event->client = window2ASWindow( event->w );
+			if( (event->eclass & ASE_POINTER_EVENTS) != 0 && is_balloon_click( &(event->x) ) )
+			{	
+				event->client = NULL ;
+				event->widget = NULL ;	 
+			}else
+			{	
+				event->widget = NULL ;	
+				event->client = window2ASWindow( event->w );
+			}
 		}	 
 	}
 
@@ -423,7 +431,8 @@ DigestEvent( ASEvent *event )
 	        if (ASWIN_GET_FLAGS(asw, AS_Shaded) && canvas != asw->frame_sides[tbar_side] )
 			{
 				event->context = C_NO_CONTEXT ;				
-				XRaiseWindow( dpy, asw->frame_sides[tbar_side]->w );
+				if( asw->frame_sides[tbar_side] )
+					XRaiseWindow( dpy, asw->frame_sides[tbar_side]->w );
 			}else if( w != asw->frame )
             {
                 if( event->w == asw->frame )
@@ -599,7 +608,13 @@ DispatchEvent ( ASEvent *event, Bool deffered )
             * indicates that the warp is done */
             if( get_flags( AfterStepState, ASS_WarpingMode ) )
                 EndWarping();
-            HandleButtonPress (event, deffered);
+			if( event->x.xbutton.button > Button3 )
+			{/* buttons 4 and 5 are for scrollwheel */
+				ASInternalWindow *internal = event->client?event->client->internal:NULL ;
+				if( internal && internal->on_scroll_event )
+					internal->on_scroll_event( internal, event );
+			}else
+		        HandleButtonPress (event, deffered);
             break;
         case ButtonRelease:
             /* if warping, a button press, non-warp keypress, or pointer motion
@@ -695,8 +710,10 @@ DispatchEvent ( ASEvent *event, Bool deffered )
 void
 HandleFocusIn ( ASEvent *event )
 {
-    while (ASCheckTypedEvent (FocusIn, &event->x));
-    DigestEvent( event );
+    int events_count = 0 ;
+	while (ASCheckTypedEvent (FocusIn, &event->x)) events_count++;
+	if( events_count > 0 ) 
+    	DigestEvent( event );
 
     if( get_flags( AfterStepState, ASS_WarpingMode ) )
         ChangeWarpingFocus( event->client );
@@ -800,6 +817,7 @@ HandlePropertyNotify (ASEvent *event)
     ASWindow       *asw;
     XPropertyEvent *xprop = &(event->x.xproperty);
     Atom atom = xprop->atom ;
+	XEvent prop_xev ; 
 
 	/* force updates for "transparent" windows */
     if (atom == _XROOTPMAP_ID && event->w == Scr.Root)
@@ -813,7 +831,7 @@ HandlePropertyNotify (ASEvent *event)
         }
         if(Scr.RootBackground && Scr.RootBackground->im != NULL )
         {
-            if( Scr.RootBackground->pmap && Scr.wmprops->root_pixmap == Scr.RootBackground->pmap );
+            if( Scr.RootBackground->pmap && Scr.wmprops->root_pixmap == Scr.RootBackground->pmap )
                 Scr.RootImage = dup_asimage(Scr.RootBackground->im) ;
         }
 		if( Scr.wmprops->as_root_pixmap != Scr.wmprops->root_pixmap ) 
@@ -833,10 +851,15 @@ HandlePropertyNotify (ASEvent *event)
 					move_menu (menu, (*menu).x, (*menu).y);
         }
 #endif
+		return;
     }
 
-    if( (asw = event->client) == NULL )
+    if( (asw = event->client) == NULL || ASWIN_GET_FLAGS( asw, AS_Dead ))
+	{
+		if( event->w != Scr.Root )
+			while (XCheckTypedWindowEvent (dpy, event->w, PropertyNotify, &prop_xev));
         return ;
+	}
     else
     {
 		char *prop_name = NULL;
@@ -847,12 +870,24 @@ HandlePropertyNotify (ASEvent *event)
     if( IsNameProp(atom))
     {
 		char *old_name = get_flags( asw->internal_flags, ASWF_NameChanged )?NULL:mystrdup( ASWIN_NAME(asw) );
+		
+		/* we want to check if there were some more events generated 
+		 * as window names tend to change multiple properties : */
+		while (XCheckTypedWindowEvent (dpy, asw->w, PropertyNotify, &prop_xev))
+			if( !IsNameProp(prop_xev.xproperty.atom) ) 
+			{	
+				XPutBackEvent( dpy, &prop_xev );
+				break;
+			}
+		
 		/*ASFlagType old_hflags = asw->hints->flags ; */
 		show_debug( __FILE__, __FUNCTION__, __LINE__, "name prop changed..." );
-        if( update_property_hints_manager( asw->w, xprop->atom,
-                                        Scr.Look.supported_hints,
-										Database,
-                                        asw->hints, asw->status ) )
+        if( get_flags( Scr.Feel.flags, FollowTitleChanges) )
+    	    on_window_hints_changed( asw );
+    	else if( update_property_hints_manager( asw->w, xprop->atom,
+             	                           		Scr.Look.supported_hints,
+												Database,
+                                        		asw->hints, asw->status ) )
         {
             if( ASWIN_GET_FLAGS( asw, AS_Dead ) )
 				return;
@@ -862,15 +897,11 @@ HandlePropertyNotify (ASEvent *event)
 			if( old_name && strcmp( old_name, ASWIN_NAME(asw) ) != 0 )
 				set_flags( asw->internal_flags, ASWF_NameChanged );
             /* fix the name in the title bar */
-            if( get_flags( Scr.Feel.flags, FollowTitleChanges) )
-	    	    on_window_hints_changed( asw );
-	    	else
-	    	{
-				if (!ASWIN_GET_FLAGS(asw, AS_Iconic))
-           	    	on_window_title_changed( asw, True );
-	        	broadcast_window_name( asw );
-        		broadcast_icon_name( asw );
-		    }
+			if (!ASWIN_GET_FLAGS(asw, AS_Iconic))
+       	    	on_window_title_changed( asw, True );
+			broadcast_res_names( asw );
+        	broadcast_window_name( asw );
+       		broadcast_icon_name( asw );
         }
 		if( old_name )
 			free( old_name );
@@ -879,8 +910,13 @@ HandlePropertyNotify (ASEvent *event)
     }else if( atom == XA_WM_COMMAND || atom == XA_WM_CLIENT_MACHINE )
 	{
 		update_cmd_line_hints (asw->w, atom, asw->hints, asw->status );
+	}else if( atom == _XA_NET_WM_WINDOW_OPACITY )
+	{	
+	 	if( event->context == C_WINDOW && event->w != asw->frame ) 
+			on_window_opacity_changed( asw );
 	}else if( NeedToTrackPropChanges(atom) )
         on_window_hints_changed( asw );
+
     /* we have to do the complete refresh of hints, since we have to override WH_HINTS with database, etc. */
 }
 
@@ -904,7 +940,7 @@ HandleClientMessage (ASEvent *event)
         (event->x.xclient.data.l[0] == IconicState) &&
         !ASWIN_GET_FLAGS(event->client, AS_Iconic))
 	{
-        set_window_wm_state( event->client, True );
+        set_window_wm_state( event->client, True, False );
 #ifdef ENABLE_DND
 		/* Pass the event to the client window */
         if (event->x.xclient.window != event->client->w)
@@ -921,16 +957,29 @@ HandleClientMessage (ASEvent *event)
 		ASFlagType extwm_flags = 0, as_flags = 0;
 		CARD32 props[2] ;
 		XClientMessageEvent *xcli = &(event->x.xclient) ;
+		
 		props[0] = xcli->data.l[1] ;
 		props[1] = xcli->data.l[2] ;
-
+		
 		translate_atom_list (&extwm_flags, EXTWM_State, &props[0], 2);
 		/* now we need to translate EXTWM flags into AS flags : */
 		as_flags = extwm_state2as_state_flags( extwm_flags );
 		if( xcli->data.l[0] == EXTWM_StateRemove ) 
+		{
 			as_flags = ASWIN_GET_FLAGS(event->client,as_flags);
-		else if( xcli->data.l[0] == EXTWM_StateAdd ) 
+		}else if( xcli->data.l[0] == EXTWM_StateAdd ) 
 			as_flags = as_flags&(~ASWIN_GET_FLAGS(event->client,as_flags));
+
+		if( props[0] == _XA_NET_WM_STATE_DEMANDS_ATTENTION ||
+			props[1] == _XA_NET_WM_STATE_DEMANDS_ATTENTION  ) 
+		{/* requires special treatment as it competes with ICCCM HintUrgency in WM_HINTS */
+			Bool set = True ; 
+			if( xcli->data.l[0] == EXTWM_StateRemove || 
+				(xcli->data.l[0] == EXTWM_StateToggle && ASWIN_GET_FLAGS(event->client,AS_Urgent)))  
+				set = False ; 
+			set_extwm_urgency_state( event->client->w, set );
+		}		
+		
 		if( as_flags != 0 ) 
 			toggle_aswindow_status(event->client, as_flags );
 	}	 
@@ -975,10 +1024,10 @@ HandleMapRequest (ASEvent *event )
     /* If the window has never been mapped before ... */
     if (event->client == NULL)
     {
-        if( (event->client = AddWindow (event->w)) == NULL )
+        if( (event->client = AddWindow (event->w, True)) == NULL )
             return;
     }else /* If no hints, or currently an icon, just "deiconify" */
-        set_window_wm_state( event->client, False );
+        set_window_wm_state( event->client, False, True );
 }
 
 /***********************************************************************
@@ -997,6 +1046,7 @@ HandleMapNotify ( ASEvent *event )
     if ( asw == NULL || event->w == Scr.Root )
         return;
 
+    LOCAL_DEBUG_OUT( "asw->w = %lX, event->w = %lX", asw->w, event->w );
     if( event->w != asw->w )
     {
         if( asw->wm_state_transition == ASWT_Withdrawn2Iconic && event->w == asw->status->icon_window )
@@ -1007,11 +1057,18 @@ HandleMapNotify ( ASEvent *event )
     }
 
 	if( asw->wm_state_transition == ASWT_Withdrawn2Normal )
-		no_focus = (ASWIN_HFLAGS(asw, AS_FocusOnMap) == 0) ;
-    if( asw->wm_state_transition == ASWT_StableState )
+	{
+		if( ASWIN_HFLAGS(asw, AS_FocusOnMap) )
+			force_activation = True;
+		else
+			no_focus = True ;
+	}
+    LOCAL_DEBUG_OUT( "asw->wm_state_transition = %d", asw->wm_state_transition );
+
+	if( asw->wm_state_transition == ASWT_StableState )
     {
         if( ASWIN_GET_FLAGS( asw, AS_Iconic ) )
-            set_window_wm_state( asw, False );  /* client has requested deiconification */
+            set_window_wm_state( asw, False, False );  /* client has requested deiconification */
         return ;                                /* otherwise it is redundand event */
     }
     if( get_flags(asw->wm_state_transition, ASWT_FROM_ICONIC ) )
@@ -1022,7 +1079,8 @@ HandleMapNotify ( ASEvent *event )
     ASWIN_CLEAR_FLAGS(asw, AS_IconMapped);
     ASWIN_CLEAR_FLAGS(asw, AS_Iconic);
     complete_wm_state_transition( asw, NormalState );
-	if( !no_focus )
+	LOCAL_DEBUG_OUT( "no_focus = %d, force_activation = %d, AcceptsFocus = %ld", no_focus, force_activation, ASWIN_HFLAGS(asw,AS_AcceptsFocus) );
+	if( !no_focus && ASWIN_HFLAGS(asw,AS_AcceptsFocus) )
     	activate_aswindow (asw, force_activation, False);
     broadcast_config( M_MAP, asw );
     /* finally reaches Normal state */
@@ -1076,12 +1134,12 @@ HandleUnmapNotify (ASEvent *event )
 
     grab_server();
     destroyed = ASCheckTypedWindowEvent ( event->w, DestroyNotify, &dummy) ;
-	LOCAL_DEBUG_OUT("wm_state_transition = 0x%X", event->client->wm_state_transition );
-	if( !get_flags( event->client->wm_state_transition, ASWT_FROM_WITHDRAWN ) )
-    	event->client->wm_state_transition = ASWIN_GET_FLAGS(event->client, AS_Iconic)?ASWT_Iconic2Withdrawn:ASWT_Normal2Withdrawn ;
+	LOCAL_DEBUG_OUT("wm_state_transition = 0x%X", asw->wm_state_transition );
+	if( !get_flags( asw->wm_state_transition, ASWT_FROM_WITHDRAWN ) )
+    	asw->wm_state_transition = ASWIN_GET_FLAGS(asw, AS_Iconic)?ASWT_Iconic2Withdrawn:ASWT_Normal2Withdrawn ;
 	else
-		event->client->wm_state_transition = ASWT_Withdrawn2Withdrawn ;
-    Destroy (event->client, destroyed);               /* do not need to mash event before */
+		asw->wm_state_transition = ASWT_Withdrawn2Withdrawn ;
+    Destroy (asw, destroyed);               /* do not need to mash event before */
     ungrab_server();
     ASFlush ();
 }
@@ -1330,7 +1388,8 @@ HandleConfigureRequest ( ASEvent *event )
 		return;
 	}
 
-	if( ASWIN_HFLAGS(asw, AS_IgnoreConfigRequest ) )
+	if( ASWIN_HFLAGS(asw, AS_IgnoreConfigRequest ) ||
+		ASWIN_GET_FLAGS( asw, AS_Fullscreen ) )
 	{	
 		LOCAL_DEBUG_OUT( "Ignoring ConfigureRequest for client %p as required by hints", asw );
 		send_canvas_configure_notify(asw->frame_canvas, asw->client_canvas);
@@ -1455,7 +1514,7 @@ void HandleShmCompletion(ASEvent *event)
  *
  ****************************************************************************/
 void
-afterstep_wait_pipes_input()
+afterstep_wait_pipes_input(int timeout_sec)
 {
 	fd_set        in_fdset, out_fdset;
 	int           retval;
@@ -1498,6 +1557,12 @@ afterstep_wait_pipes_input()
 	/* watch for timeouts */
 	if (timer_delay_till_next_alarm ((time_t *) & tv.tv_sec, (time_t *) & tv.tv_usec))
 		t = &tv;
+	else if( timeout_sec > 0 )
+	{
+		t = &tv ; 
+		tv.tv_sec = timeout_sec ; 
+		tv.tv_usec = 0 ;
+	}
     retval = PORTABLE_SELECT(min (max_fd + 1, fd_width),&in_fdset,&out_fdset,NULL,t);
 
 	if (retval > 0)
