@@ -38,11 +38,12 @@
  *
  ***************************************************************************/
 
-#undef LOCAL_DEBUG
+#define LOCAL_DEBUG
 
 #include "../../configure.h"
 
 #include "asinternals.h"
+#include "../../libAfterStep/desktop_category.h"
 
 #include <signal.h>
 #include <stdarg.h>
@@ -84,9 +85,12 @@ FindPopup (const char *name, int quiet)
 MenuData     *
 CreateMenuData (char *name)
 {
+	MenuData *md;
     if( Scr.Feel.Popups == NULL )
         init_list_of_menus(&(Scr.Feel.Popups), True);
-    return new_menu_data( Scr.Feel.Popups, name );
+    md = new_menu_data( Scr.Feel.Popups, name );
+	md->recent_items = Scr.Feel.recent_submenu_items ;
+	return md;
 }
 
 void print_func_data(const char *file, const char *func, int line, FunctionData *data);
@@ -137,7 +141,7 @@ FunctionItemParse (void *data, const char *buf)
         return False;
     if (fdata.func != F_ENDPOPUP && fdata.func != F_ENDFUNC)
 	{
-        if (fdata.func != F_MINIPIXMAP)
+        if (!IsMinipixmapFunc(fdata.func))
         {
             func->items = realloc(func->items, sizeof(FunctionData)*(func->items_num+1));
             func->items[func->items_num] = fdata ;
@@ -328,6 +332,19 @@ ParseKeyEntry (char *tline, FILE * fd, char **junk, int *junk2)
 
  /* we assume buf is at least MAXLINELENGTH bytes */
 
+void 
+add_minipixmap_from_dirtree_item( dirtree_t * tree, MenuData *menu )
+{
+	FunctionData *fdata = NULL ;
+	if( tree->de != NULL && tree->de->fulliconname != NULL ) 
+        fdata = create_named_function( F_SMALL_MINIPIXMAP, tree->de->fulliconname);
+    else if (tree->icon != NULL) /* should default to: "mini-menu.xpm" */
+        fdata = create_named_function(  get_flags(tree->flags, DIRTREE_ICON_IS_SMALL)?F_SMALL_MINIPIXMAP:F_MINIPIXMAP, 
+										tree->icon);
+	if( fdata ) 
+		menu_data_item_from_func (menu, fdata, False);
+}	 
+
 MenuData     *
 dirtree_make_menu2 (dirtree_t * tree, char *buf, Bool reload_submenus)
 {
@@ -351,12 +368,9 @@ dirtree_make_menu2 (dirtree_t * tree, char *buf, Bool reload_submenus)
 	fdata = create_named_function( F_TITLE, tree->name);
 	/* We exploit that scan_for_hotkey removes & (marking hotkey) from name */
 	scan_for_hotkey (fdata->name);
-    MenuDataItemFromFunc (menu, fdata);
-    if (tree->icon != NULL)
-    {/* should default to: "mini-menu.xpm" */
-        fdata = create_named_function( F_MINIPIXMAP, tree->icon);
-        MenuDataItemFromFunc (menu, fdata);
-	}
+    menu_data_item_from_func (menu, fdata, False);
+
+	add_minipixmap_from_dirtree_item( tree, menu );
 
 	for (t = tree->child; t != NULL; t = t->next)
 	{
@@ -379,15 +393,28 @@ dirtree_make_menu2 (dirtree_t * tree, char *buf, Bool reload_submenus)
 			else
 				fdata->text = string_from_int (t->flags & DIRTREE_ID);
 
-            MenuDataItemFromFunc (menu, fdata);
-            if (t->icon)
-			{
-                /* should default to "mini-folder.xpm" */
-                fdata = create_named_function( F_MINIPIXMAP, t->icon);
-                MenuDataItemFromFunc (menu, fdata);
-			}
+            menu_data_item_from_func (menu, fdata, False);
+
+ 			add_minipixmap_from_dirtree_item( t, menu );
 /************* Done creating Popup Title entry : ************************/
-		} else if (t->command.func != F_NOP)
+		} else if( t->de ) 
+		{
+			if( t->de->type == ASDE_TypeApplication ) 
+			{	
+				FunctionCode func = F_EXEC ;
+				if( get_flags( t->de->flags, ASDE_Unavailable ) )
+					func = F_NOP ;
+				if( get_flags( t->de->flags, ASDE_Terminal ) )
+					func = F_ExecInTerm;	   
+				else if( get_flags( t->de->flags, ASDE_ASModule ) )
+					func = F_MODULE;	   
+		 			
+				fdata = create_named_function(func, t->stripped_name);	   
+            	fdata->text = mystrdup( t->de->clean_exec );
+            	menu_data_item_from_func (menu, fdata, False);
+ 				add_minipixmap_from_dirtree_item( t, menu );
+			}
+		}else if (t->command.func != F_NOP)
 		{
 			fdata = create_named_function(t->command.func, t->stripped_name);
 			if (t->command.text)
@@ -401,12 +428,19 @@ dirtree_make_menu2 (dirtree_t * tree, char *buf, Bool reload_submenus)
                     quotestr (fdata->text + cmd_len + 1, t->path, 2 * path_len + 1);
 			} else
 				fdata->text = mystrdup (t->path);
-            MenuDataItemFromFunc (menu, fdata);
-
-            if (t->icon != NULL)
-            {  /* no defaults !!! */
-				fdata = create_named_function(F_MINIPIXMAP, t->icon);
-                MenuDataItemFromFunc (menu, fdata);
+						
+#ifndef NO_AVAILABILITYCHECK
+			if( !check_fdata_availability( fdata ) ) 
+				fdata->func = F_NOP ; 
+#endif		
+			LOCAL_DEBUG_OUT( "fdata->text = \"%s\", func = %ld", fdata->text, fdata->func );			
+			if( fdata->func == F_NOP && !get_flags(tree->flags, DIRTREE_SHOW_UNAVAILABLE) )
+			{	
+				destroy_func_data( &fdata );
+			}else
+			{
+	            menu_data_item_from_func (menu, fdata, False);
+	 			add_minipixmap_from_dirtree_item( t, menu );
 			}
         } else
 		{
@@ -433,14 +467,11 @@ dirtree_make_menu2 (dirtree_t * tree, char *buf, Bool reload_submenus)
 					if( fdata->name == NULL )
 		                fdata->name = mystrdup( t->stripped_name );
 #ifndef NO_AVAILABILITYCHECK
-					if ( fdata->func!= F_ExecInTerm && (IsSwallowFunc(fdata->func) || IsExecFunc(fdata->func)))
-					{	
-						available = is_executable_in_path (fdata->text);
-						if( !available )
-							fdata->func = F_NOP;
-					}
+					available = check_fdata_availability( fdata );
+					if( !available )
+						fdata->func = F_NOP;
 #endif					
-					if( fdata->func == F_MINIPIXMAP )
+					if( IsMinipixmapFunc(fdata->func) )
 					{
 						if( minipixmap == NULL ) 
 						{
@@ -469,19 +500,19 @@ dirtree_make_menu2 (dirtree_t * tree, char *buf, Bool reload_submenus)
 				if( available || get_flags(tree->flags, DIRTREE_SHOW_UNAVAILABLE))
 				{	
 					if( valid_func  ) 
-						MenuDataItemFromFunc (menu, valid_func);
+						menu_data_item_from_func (menu, valid_func, False);
 					else
 					{
 						fdata = create_named_function(F_NOP, name?name:t->stripped_name);
 	  					fdata->text = mystrdup (t->name);
-	          			MenuDataItemFromFunc (menu, fdata);
+	          			menu_data_item_from_func (menu, fdata, False);
 					}
 					if( minipixmap ) 
-						MenuDataItemFromFunc (menu, minipixmap);
+						menu_data_item_from_func (menu, minipixmap, False);
 					else if( t->icon != NULL )
 					{
 						fdata = create_named_function(F_MINIPIXMAP, t->icon);
-	          			MenuDataItemFromFunc (menu, fdata);
+	          			menu_data_item_from_func (menu, fdata, False);
 					}
 				}else
 				{
@@ -500,13 +531,13 @@ dirtree_make_menu2 (dirtree_t * tree, char *buf, Bool reload_submenus)
 				fdata = create_named_function(F_EXEC, t->stripped_name);
 				fdata->text = mystrdup (t->name);
 #ifndef NO_AVAILABILITYCHECK
-				available = is_executable_in_path (fdata->text);
+				available = check_fdata_availability( fdata );
 				if( !available )		   
 					fdata->func = F_NOP;
 #endif			
 				if( available || get_flags(tree->flags, DIRTREE_SHOW_UNAVAILABLE))
 				{	
-					MenuDataItemFromFunc (menu, fdata);
+					menu_data_item_from_func (menu, fdata, False);
 					if( t->icon != NULL )
 					{
 						fdata = create_named_function(F_MINIPIXMAP, t->icon);
