@@ -43,6 +43,7 @@
 #define STOP_LONG_DRAW_OPERATION    ungrab_server()
 
 
+
 static ASMenu *ASTopmostMenu = NULL;
 
 ASHints *make_menu_hints( ASMenu *menu );
@@ -86,7 +87,9 @@ free_asmenu_item( ASMenuItem *item )
 {
     if( item->bar )
         destroy_astbar(&(item->bar) );
-    item->submenu = NULL ;
+	if( item->icon )
+	    safe_asimage_destroy( item->icon );
+	free_func_data (&(item->fdata));
 }
 
 void destroy_asmenu(ASMenu **pmenu);
@@ -192,6 +195,8 @@ LOCAL_DEBUG_CALLER_OUT( "top(%p)->supermenu(%p)->menu(%p)->submenu(%p)", ASTopmo
     }
 }
 
+#define FDataPopupName(fdata)   ((fdata).text)
+
 static void
 set_asmenu_item_data( ASMenuItem *item, MenuDataItem *mdi )
 {
@@ -211,17 +216,25 @@ set_asmenu_item_data( ASMenuItem *item, MenuDataItem *mdi )
         item->icon = NULL ;
     }
 	/* we can only use images that are reference counted */
-    if( mdi->minipixmap_image && mdi->minipixmap_image->imageman != NULL )
+    if( mdi->minipixmap_image )
         icon_im = mdi->minipixmap_image ;
     else if( mdi->minipixmap )
         icon_im = GetASImageFromFile( mdi->minipixmap );
+	else if( mdi->fdata->func == F_CHANGE_BACKGROUND_FOREIGN ) 
+		icon_im = GetASImageFromFile( mdi->fdata->text );
+
     if( icon_im )
 	{
-		item->icon = check_scale_menu_pmap( icon_im );
+		item->icon = check_scale_menu_pmap( icon_im, mdi->flags );
 		if( item->icon != icon_im && icon_im != mdi->minipixmap_image )
 			safe_asimage_destroy( icon_im );
-		if( item->icon == mdi->minipixmap_image )
-            item->icon = dup_asimage( item->icon );
+		if( item->icon && item->icon == mdi->minipixmap_image )
+		{	
+			if( item->icon->imageman != NULL )
+            	item->icon = dup_asimage( item->icon );
+			else
+		    	item->icon = clone_asimage( item->icon, 0xFFFFFFFF );
+		}
     }
 LOCAL_DEBUG_OUT( "item(\"%s\")->minipixmap(\"%s\")->icon(%p)", mdi->item?mdi->item:"NULL", mdi->minipixmap?mdi->minipixmap:"NULL", item->icon );
 
@@ -239,21 +252,30 @@ LOCAL_DEBUG_OUT( "item(\"%s\")->minipixmap(\"%s\")->icon(%p)", mdi->item?mdi->it
     /* optional menu items : */
     /* add label */
     if( mdi->item )
+	{	
         add_astbar_label( item->bar, 3, 0, 0, ALIGN_LEFT|ALIGN_VCENTER, 0, 0, mdi->item,
 		mdi->fdata->name_encoding );
+		item->first_sym = mdi->item[0] ;
+	}
     /* add hotkey */
     if( mdi->item2 )
         add_astbar_label( item->bar, 4, 0, 0, ALIGN_RIGHT|ALIGN_VCENTER, 0, 0, mdi->item2, mdi->fdata->name_encoding );
 
     item->flags = 0 ;
-    if( mdi->fdata->func == F_POPUP )
+
+	if( IsDynamicPopup(mdi->fdata->func) )
+	{	
+		set_flags( item->flags, AS_MenuItemHasSubmenu );
+	}else if( mdi->fdata->func == F_POPUP )
     {
-        if( (item->submenu = FindPopup (mdi->fdata->text, True)) == NULL )
+		MenuData *submenu = FindPopup (FDataPopupName(*(mdi->fdata)), True);
+        if( submenu == NULL )
             set_flags( item->flags, AS_MenuItemDisabled );
 		else
 		{
-			MenuDataItem *smdi = item->submenu->first ;
+			MenuDataItem *smdi = submenu->first ;
 
+			set_flags( item->flags, AS_MenuItemHasSubmenu );
 			while( smdi ) 
 			{
 				if( smdi->fdata->func != F_TITLE )
@@ -266,7 +288,7 @@ LOCAL_DEBUG_OUT( "item(\"%s\")->minipixmap(\"%s\")->icon(%p)", mdi->item?mdi->it
     }else if( get_flags( mdi->flags, MD_Disabled ) || mdi->fdata->func == F_NOP )
         set_flags( item->flags, AS_MenuItemDisabled );
 
-    item->fdata = mdi->fdata ;
+	dup_func_data (&(item->fdata), mdi->fdata);
 }
 
 static Bool
@@ -315,7 +337,7 @@ LOCAL_DEBUG_OUT( "item.bar(%p)->look(%p)", item->bar, look );
     /* delete tile from Popup arrow cell : */
     delete_astbar_tile( item->bar, MI_POPUP_IDX );
     /* now readd it as proper type : */
-    if( look->MenuArrow && item->fdata->func == F_POPUP )
+    if( look->MenuArrow && get_flags( item->flags, AS_MenuItemHasSubmenu ) )
         add_astbar_icon( item->bar, 7, 0, 0, ALIGN_VCENTER, look->MenuArrow->image );
     else
         add_astbar_spacer( item->bar, 7, 0, 0, NO_ALIGN, arrow_space, 1 );
@@ -425,9 +447,10 @@ render_asmenu_bars( ASMenu *menu, Bool force )
 /* midium level ASMenu functionality :                                      */
 /*************************************************************************/
 static unsigned int
-extract_recent_subitems( MenuData *md, MenuDataItem **subitems, unsigned int max_subitems )
+extract_recent_subitems( char *submenu_name, MenuDataItem **subitems, unsigned int max_subitems )
 {
-    int used = 0 ;
+	MenuData *md = FindPopup (submenu_name, True);
+	int used = 0 ;
     MenuDataItem *smdi = md->first ;
     while( smdi != NULL )
     {
@@ -492,9 +515,9 @@ set_asmenu_data( ASMenu *menu, MenuData *md, Bool first_time )
                 set_asmenu_item_data( item, mdi );
 
                 ++real_items_num;
-                if( mdi->fdata->func == F_POPUP && item->submenu != NULL && subitems != NULL )
+                if( item->fdata.func == F_POPUP && !get_flags( item->flags, AS_MenuItemDisabled ) && subitems != NULL )
                 {
-                    int used = extract_recent_subitems( item->submenu, subitems, md->recent_items );
+                    int used = extract_recent_subitems( FDataPopupName(item->fdata), subitems, md->recent_items );
                     if( used > 0 )
                     {
                         items_num += used ;
@@ -611,7 +634,10 @@ set_asmenu_look( ASMenu *menu, MyLook *look )
         if( (bar= menu->items[i].bar) != NULL )
         {
             width = calculate_astbar_width( bar );
-            height = calculate_astbar_height( bar );
+			if( get_flags( menu->items[i].flags, AS_MenuItemSubitem ) )
+				height = max_height;
+			else
+            	height = calculate_astbar_height( bar );
 LOCAL_DEBUG_OUT( "i(%d)->bar(%p)->size(%ux%u)", i, bar, width, height );
             if( width > max_width )
                 max_width = width ;
@@ -690,9 +716,11 @@ set_menu_item_used( ASMenu *menu, MenuDataItem *mdi )
 void set_asmenu_scroll_position( ASMenu *menu, int pos );
 
 void
-select_menu_item( ASMenu *menu, int selection )
+select_menu_item( ASMenu *menu, int selection, Bool render )
 {
+	Bool needs_scrolling ;
 LOCAL_DEBUG_CALLER_OUT( "%p,%d", menu, selection );
+	
     if( AS_ASSERT(menu) || menu->items_num == 0 )
         return;
     if( selection < 0 )
@@ -700,20 +728,22 @@ LOCAL_DEBUG_CALLER_OUT( "%p,%d", menu, selection );
     else if( selection >= (int)menu->items_num )
         selection = menu->items_num - 1 ;
 
+	needs_scrolling = ( selection < menu->top_item || selection >= menu->top_item + menu->visible_items_num ) ;
     if( selection != menu->selected_item )
     {
         close_asmenu_submenu( menu );
 		if( menu->selected_item >= 0 )
-        	set_astbar_focused( menu->items[menu->selected_item].bar, menu->main_canvas, False );
+        	set_astbar_focused( menu->items[menu->selected_item].bar, NULL/*needs_scrolling?NULL:menu->main_canvas*/, False );
     }
-    set_astbar_focused( menu->items[selection].bar, menu->main_canvas, True );
+    set_astbar_focused( menu->items[selection].bar, NULL/*needs_scrolling?NULL:menu->main_canvas*/, True );
     menu->selected_item = selection ;
 
     if( selection < menu->top_item )
-        set_asmenu_scroll_position( menu, MAX(selection-1, 0) );
+        set_asmenu_scroll_position( menu, MAX(selection, 0) );
     else if( selection >= menu->top_item + menu->visible_items_num )
         set_asmenu_scroll_position( menu, (selection-menu->visible_items_num)+1);
-    render_asmenu_bars(menu, False);
+	else if( render )
+	    render_asmenu_bars(menu, False);
 }
 
 void
@@ -764,41 +794,63 @@ LOCAL_DEBUG_OUT("adj_pos(%d)->curr_y(%d)->items_num(%d)->vis_items_num(%d)->sel_
 	menu->top_item = first_item ;
 
 	if( menu->selected_item < menu->top_item )
-        select_menu_item( menu, menu->top_item );
+        select_menu_item( menu, menu->top_item, False );
     else if( menu->selected_item >= 0 )
 	{
 		if( menu->visible_items_num > 0 && menu->selected_item >= (int)menu->top_item + (int)menu->visible_items_num )
-        select_menu_item( menu, menu->top_item + menu->visible_items_num - 1 );
+        select_menu_item( menu, menu->top_item + menu->visible_items_num - 1, False );
 	}
     render_asmenu_bars(menu, False);
 }
 
 static inline void
-run_item_submenu( ASMenu *menu, int item )
+run_item_submenu( ASMenu *menu, int item_no )
 {
-LOCAL_DEBUG_CALLER_OUT( "%p, %d, submenu(%p)", menu, item, menu->items[item].submenu );
-    if( menu->items[item].submenu && !get_flags( menu->items[item].flags, AS_MenuItemDisabled ) )
-    {
-        int x = menu->main_canvas->root_x+(int)menu->main_canvas->bw ;
-        int y ;
-        ASQueryPointerRootXY( &x, &y );
-        x -= 5 ;
+	ASMenuItem *item = &(menu->items[item_no]);
+LOCAL_DEBUG_CALLER_OUT( "%p, %d", menu, item_no );
+    if( !get_flags( item->flags, AS_MenuItemDisabled ) )
+	{
+		MenuData *submenu = NULL ;
+		if( item->fdata.func == F_POPUP )
+			submenu = FindPopup( FDataPopupName(item->fdata), True );
+		else if( item->fdata.func == F_STOPMODULELIST ) 
+			submenu = make_stop_module_menu(  Scr.Feel.winlist_sort_order );
+		else if( item->fdata.func == F_RESTARTMODULELIST ) 
+			submenu = make_restart_module_menu(  Scr.Feel.winlist_sort_order );
+#ifndef NO_WINDOWLIST
+		else if( item->fdata.func == F_WINDOWLIST ) 
+		{	
+			submenu = make_desk_winlist_menu( Scr.Windows, 
+											  (item->fdata.text == NULL) ? 
+											  Scr.CurrentDesk: item->fdata.func_val[0], 
+											  Scr.Feel.winlist_sort_order, False );
+		}
+#endif		
+		if( submenu )
+    	{
+        	int x = menu->main_canvas->root_x+(int)menu->main_canvas->bw ;
+        	int y ;
+        	ASQueryPointerRootXY( &x, &y );
+        	x -= 5 ;
 #if 0
-        if( x  < menu->main_canvas->root_x + (int)(menu->item_width/3) )
-        {
-            int max_dx = Scr.MyDisplayWidth / 40 ;
-            if( menu->main_canvas->root_x + (int)(menu->item_width/3) - x  < max_dx )
-                x = menu->main_canvas->root_x + (int)(menu->item_width/3) ;
-            else
-                x += max_dx ;
-        }
+        	if( x  < menu->main_canvas->root_x + (int)(menu->item_width/3) )
+        	{
+            	int max_dx = Scr.MyDisplayWidth / 40 ;
+            	if( menu->main_canvas->root_x + (int)(menu->item_width/3) - x  < max_dx )
+                	x = menu->main_canvas->root_x + (int)(menu->item_width/3) ;
+            	else
+                	x += max_dx ;
+        	}
 #endif
-        y = menu->main_canvas->root_y+(menu->item_height*(item+1-(int)menu->top_item)) - 10 ;
-/*	if( x > menu->main_canvas->root_x+menu->item_width-5 )
-	    x = menu->main_canvas->root_x+menu->item_width-5 ;
-*/      close_asmenu_submenu( menu );
-        menu->submenu = run_submenu( menu, menu->items[item].submenu, x, y);
-    }
+        	y = menu->main_canvas->root_y+(menu->item_height*(item_no+1-(int)menu->top_item)) - 10 ;
+	/*	if( x > menu->main_canvas->root_x+menu->item_width-5 )
+	    	x = menu->main_canvas->root_x+menu->item_width-5 ;
+	*/      close_asmenu_submenu( menu );
+        	menu->submenu = run_submenu( menu, submenu, x, y);
+			if( item->fdata.func != F_POPUP )
+    			destroy_menu_data( &submenu );
+		}
+	}
 }
 
 void
@@ -824,17 +876,17 @@ LOCAL_DEBUG_CALLER_OUT( "%p,%d", menu, pressed );
 			if( keyboard_event ) 
 			{   /* don't press if keyboard selection */
 				if( pressed != menu->selected_item )
-					select_menu_item( menu, pressed );
+					select_menu_item( menu, pressed, False );
 
 			}else if( pressed != menu->selected_item )
             {
                 set_astbar_pressed( menu->items[pressed].bar, NULL, True );/* don't redraw yet */
-                select_menu_item( menu, pressed );  /* this one updates display already */
+                select_menu_item( menu, pressed, False );  /* this one updates display already */
             }else
                 set_astbar_pressed( menu->items[pressed].bar, menu->main_canvas, True );
             set_menu_item_used( menu, menu->items[pressed].source );
 			
-			if( menu->items[pressed].submenu != NULL ) 
+			if( get_flags( menu->items[pressed].flags, AS_MenuItemHasSubmenu) ) 
 			{
 				if( keyboard_event ) 
 				{
@@ -857,11 +909,10 @@ LOCAL_DEBUG_OUT( "pressed(%d)->old_pressed(%d)->focused(%d)", pressed, menu->pre
 		if( item_no >= 0 && (pressed < 0 || keyboard_event))   
     	{
         	ASMenuItem *item = &(menu->items[item_no]);
-        	if( !get_flags(item->flags, AS_MenuItemDisabled) &&
-            	 item->fdata->func != F_POPUP )
+        	if( get_flags(item->flags, AS_MenuItemDisabled|AS_MenuItemHasSubmenu) == 0 )
         	{
             	set_menu_item_used( menu, item->source );
-            	ExecuteFunctionForClient( item->fdata, menu->client_window );
+            	ExecuteFunctionForClient( &(item->fdata), menu->client_window );
             	if( !menu->pinned )
                 	close_asmenu( &ASTopmostMenu );
         	}
@@ -983,6 +1034,70 @@ LOCAL_DEBUG_OUT( "pointer(%d,%d)", px, py );
     }
 }
 
+static void 
+handle_menu_selection( ASMenu *menu, int button_state, int px, int selection, Bool render)
+{
+    if( selection != menu->selected_item )
+    {
+        if( button_state & (Button1Mask|Button2Mask|Button3Mask) )
+            press_menu_item( menu, selection, False );
+        else
+            select_menu_item( menu, selection, render );
+    }
+    if( px > menu->main_canvas->width - ( menu->arrow_space + DEFAULT_MENU_SPACING ) &&
+        menu->submenu == NULL )
+        run_item_submenu( menu, selection );
+
+}
+
+void
+on_menu_scroll_event( ASInternalWindow *asiw, ASEvent *event )
+{
+    ASMenu   *menu = (ASMenu*)(asiw->data) ;
+    if( menu != NULL && menu->magic == MAGIC_ASMENU && event)
+    {
+		if(  menu->items_num >= menu->visible_items_num )
+		{
+			XButtonEvent *xbtn = &(event->x.xbutton);
+			int selection = menu->selected_item ; 
+			int top = menu->top_item ;
+			int px_offset = menu->main_canvas->root_x+(int)menu->main_canvas->bw ;
+			int px, py ;
+			Bool needs_scrolling ;
+			
+			ASQueryPointerRootXY( &px, &py );
+			if( xbtn->button == Button4 )		
+			{
+				if( selection > 0 ) 
+				{
+					--selection ;
+					py -= menu->item_height ;
+				}
+				if( selection < menu->top_item )
+					--top ;
+			}else
+			{
+				if( selection+1 < menu->items_num ) 
+				{
+					++selection ;
+					py += menu->item_height ;
+				}
+				if( selection > menu->top_item + menu->visible_items_num - 1 )
+					++top ;
+			}
+			needs_scrolling = (top != menu->top_item);
+			if( selection != menu->selected_item )
+			{
+				handle_menu_selection( menu, xbtn->state, px-px_offset, selection, !needs_scrolling);
+				if( !needs_scrolling ) 
+					XWarpPointer (dpy, None, Scr.Root, 0, 0, 0, 0, px, py);
+			}
+//			if( needs_scrolling ) 
+//				set_asmenu_scroll_position( menu, top );
+		}
+	}
+}
+
 /* Motion notify : */
 void
 on_menu_pointer_event( ASInternalWindow *asiw, ASEvent *event )
@@ -1046,16 +1161,7 @@ on_menu_pointer_event( ASInternalWindow *asiw, ASEvent *event )
 				selection = py/menu->item_height ;
 			selection += menu->top_item ;
 
-            if( selection != menu->selected_item )
-            {
-                if( xmev->state & ButtonAnyMask )
-                    press_menu_item( menu, selection, False );
-                else
-                    select_menu_item( menu, selection );
-            }
-            if( px > canvas->width - ( menu->arrow_space + DEFAULT_MENU_SPACING ) &&
-                menu->submenu == NULL )
-                run_item_submenu( menu, selection );
+			handle_menu_selection( menu, xmev->state, px, selection, True);
         }
     }
 }
@@ -1102,17 +1208,14 @@ on_menu_keyboard_event( ASInternalWindow *asiw, ASEvent *event )
 			LOCAL_DEBUG_OUT( "processing keysym [%c]", (char)keysym );
 			/* Search menu for matching hotkey */
 			for( i = 0 ; i < menu->items_num ; i++ )
-				if( menu->items[i].fdata->hotkey == keysym )
+				if( menu->items[i].fdata.hotkey == keysym )
 				{
 					press_menu_item( menu, i, True );
 					return ;
 				}
 			for( i = 0 ; i < menu->items_num ; i++ )
 			{
-				char *n = menu->items[i].source->item ;
-		
-				LOCAL_DEBUG_OUT( "name = %s", n?n:"(null)" );	  
-				if( n != NULL && n[0] == keysym )
+				if( menu->items[i].first_sym == keysym )
 				{
 					press_menu_item( menu, i, True );
 					return ;
@@ -1130,22 +1233,22 @@ on_menu_keyboard_event( ASInternalWindow *asiw, ASEvent *event )
 				case XK_k:
 				case XK_p:
 					if( menu->selected_item > 0 )
-	                	select_menu_item( menu, menu->selected_item-1 );
+	                	select_menu_item( menu, menu->selected_item-1, True );
                     break;
 				case XK_Tab :
 				case XK_Down:
 				case XK_n:
 				case XK_j:
 					if( menu->selected_item < menu->items_num )
-	                	select_menu_item( menu, menu->selected_item+1 );
+	                	select_menu_item( menu, menu->selected_item+1, True );
                     break;
 				case XK_Right:
 				case XK_Return :
 				case XK_space :
 					if( menu->selected_item	>= 0 ) 
 					{	
-						if( menu->items[menu->selected_item].submenu || keysym != XK_Right ) 
-		 				press_menu_item( menu, menu->selected_item, True );		
+						if( get_flags( menu->items[menu->selected_item].flags, AS_MenuItemHasSubmenu) || keysym != XK_Right ) 
+		 					press_menu_item( menu, menu->selected_item, True );		
 					}
 			}	 
 		}	
@@ -1307,6 +1410,7 @@ show_asmenu( ASMenu *menu, int x, int y )
     asiw->on_hilite_changed = on_menu_hilite_changed ;
     asiw->on_pressure_changed = on_menu_pressure_changed;
     asiw->on_pointer_event = on_menu_pointer_event;
+	asiw->on_scroll_event = on_menu_scroll_event;
     asiw->on_keyboard_event = on_menu_keyboard_event;
     asiw->on_look_feel_changed = on_menu_look_feel_changed;
 	asiw->on_root_background_changed = on_menu_root_background_changed ;
@@ -1425,9 +1529,9 @@ run_submenu( ASMenu *supermenu, MenuData *md, int x, int y )
     ASMenu *menu = NULL ;
     if( md )
     {
-        menu = create_asmenu(md->name);
-		menu->rendered = False ;
-        set_asmenu_data( menu, md, True );
+		menu = create_asmenu(md->name);
+        menu->rendered = False ;
+		set_asmenu_data( menu, md, True );
         set_asmenu_look( menu, &Scr.Look );
         /* will set scroll position when ConfigureNotify arrives */
         menu->supermenu = supermenu;
