@@ -54,6 +54,7 @@
 
 #include <X11/keysym.h>
 
+#include "../../libAfterStep/desktop_category.h"
 #include "../../libAfterStep/module.h"
 #include "../../libAfterStep/wmprops.h"
 
@@ -120,15 +121,16 @@ LOCAL_DEBUG_OUT( "module name \"%s\"", module->name );
     	while (module->output_queue != NULL)
         	DeleteQueueBuff (module);
     	if (module->name != NULL)
-        	free (module->name);
-    	if (module->ibuf.text != NULL)
-        	free (module->ibuf.text);
+        destroy_string( &(module->name));
+        destroy_string( &(module->cmd_line));
+    	destroy_string( &(module->ibuf.text));
 
     	if (module->ibuf.func != NULL)
 		{
         	free_func_data (module->ibuf.func);
         	free (module->ibuf.func);
 		}
+		
 
     	memset( module, 0x00, sizeof(module_t) );
 	}else
@@ -707,53 +709,70 @@ DeadPipe (int nonsense)
 }
 
 
-void
-KillModuleByName (char *name)
-{
-    wild_reg_exp  *wrexp = compile_wild_reg_exp( name );
-
-    if ( wrexp != NULL && Modules)
-    {
-        register int i = MODULES_NUM;
-        register module_t *list = MODULES_LIST ;
-
-        while( --i >= 0 )
-            if (list[i].fd > 0)
-            {
-                LOCAL_DEBUG_OUT( "checking to kill module %d \"%s\", regexp \"%s\"", i, list[i].name, name);
-                if (match_wild_reg_exp( list[i].name, wrexp ) == 0 )
-                    KillModule (&(list[i]), False);
-            }
-        destroy_wild_reg_exp( wrexp );
-    }
-	return;
-}
-
 int FindModuleByName (char *name)
 {
-    wild_reg_exp  *wrexp = compile_wild_reg_exp( name );
 	int module = -1;
+	if( Modules ) 
+	{	
+    	wild_reg_exp  *wrexp = compile_wild_reg_exp( name );
 
-    if ( wrexp != NULL && Modules)
-    {
-        register int i = MODULES_NUM;
-        register module_t *list = MODULES_LIST ;
+    	if ( wrexp != NULL )
+    	{
+        	register int i = MODULES_NUM;
+        	register module_t *list = MODULES_LIST ;
 
-        while( --i >= 0 )
-            if (list[i].fd > 0)
-            {
-                LOCAL_DEBUG_OUT( "checking if module %d \"%s\" matches regexp \"%s\"", i, list[i].name, name);
-                if (match_wild_reg_exp( list[i].name, wrexp ) == 0 )
-				{	
-                    module = i ; 
-					break;
-				}
-            }
-        destroy_wild_reg_exp( wrexp );
+        	while( --i >= 0 )
+            	if (list[i].fd > 0)
+            	{
+                	LOCAL_DEBUG_OUT( "checking if module %d \"%s\" matches regexp \"%s\"", i, list[i].name, name);
+                	if (match_wild_reg_exp( list[i].name, wrexp ) == 0 )
+					{	
+                    	module = i ; 
+						break;
+					}
+            	}
+        	destroy_wild_reg_exp( wrexp );
+		}
     }
 	return module;
 }
 
+char *
+GetModuleCmdLineByName(char *name)
+{
+	int module = FindModuleByName (name);
+	if( module >= 0 ) 
+	{	
+		register module_t *list = MODULES_LIST ;	
+		if( list[module].cmd_line ) 
+			return mystrdup( list[module].cmd_line ) ;
+		else
+			return mystrdup( list[module].name );
+	}
+	return NULL ;
+}
+
+void
+KillModuleByName (char *name)
+{
+	int module = FindModuleByName (name);
+	if( module >= 0 ) 
+	{	
+		register module_t *list = MODULES_LIST ;	
+        KillModule (&(list[module]), False);
+	}
+}
+
+void
+KillAllModulesByName (char *name)
+{
+	int module ;
+	while( (module = FindModuleByName (name)) >= 0 )
+	{	
+		register module_t *list = MODULES_LIST ;	
+        KillModule (&(list[module]), False);
+	}
+}
 
 void
 SendPacket ( int channel, send_data_type msg_type, send_data_type num_datum, ...)
@@ -821,10 +840,10 @@ SendConfig (int module, send_data_type event_type, ASWindow * t)
     }
 
 	asw_id.asw = t ;
-    SendPacket (module, event_type, 25,
+    SendPacket (module, event_type, 26,
                 t->w,                 t->frame,              asw_id.id,
                 frame_x,              frame_y,               frame_width,         frame_height,
-                ASWIN_DESK(t),        t->status->flags,      t->hints->flags,
+                ASWIN_DESK(t),        t->status->flags,      t->hints->flags,     t->hints->client_icon_flags,
                 t->hints->base_width, t->hints->base_height, t->hints->width_inc, t->hints->height_inc,
                 t->hints->min_width,  t->hints->min_height,  t->hints->max_width,  t->hints->max_height,
                 t->hints->gravity,    icon_title_w,          icon_pixmap_w,
@@ -919,10 +938,13 @@ RunCommand (FunctionData * fdata, unsigned int channel, Window w)
 		 module->lock_on_send_mask = fdata->func_val[1];
 		 break;
 	 case F_SET_NAME:
-        if (module->name != NULL)
-            free (module->name);
-        module->name = fdata->name;
+        set_string( &(module->name), fdata->name );
         fdata->name = NULL;
+		if( fdata->text ) 
+		{	
+	        set_string( &(module->cmd_line), fdata->text );
+    	    fdata->text = NULL;
+		}
         break;
      case F_UNLOCK:
 		 toret = 66;
@@ -1017,7 +1039,7 @@ broadcast_window_name( ASWindow *asw )
     if( asw )
 	{	
         SendString( -1, M_WINDOW_NAME, asw->w, asw->frame,
-                    asw, ASWIN_NAME(asw), asw->hints->names_encoding[0]);
+                    asw, ASWIN_NAME(asw), get_hint_name_encoding(asw->hints, 0));
         SendString( -1, M_WINDOW_NAME_MATCHED, asw->w, asw->frame,
                     asw, asw->hints->matched_name0, asw->hints->matched_name0_encoding );
 	}
@@ -1027,8 +1049,10 @@ void
 broadcast_icon_name( ASWindow *asw )
 {
     if( asw )
+	{
         SendString( -1, M_ICON_NAME, asw->w, asw->frame,
-                    asw, ASWIN_ICON_NAME(asw), asw->hints->names_encoding[asw->hints->icon_name_idx]);
+                    asw, ASWIN_ICON_NAME(asw), get_hint_name_encoding(asw->hints, asw->hints->icon_name_idx));
+	}
 }
 
 void
@@ -1037,9 +1061,9 @@ broadcast_res_names( ASWindow *asw )
     if( asw )
     {
         SendString( -1, M_RES_CLASS, asw->w, asw->frame,
-                    asw, asw->hints->res_class, asw->hints->names_encoding[asw->hints->res_class_idx]);
+                    asw, asw->hints->res_class, get_hint_name_encoding(asw->hints, asw->hints->res_class_idx));
         SendString( -1, M_RES_NAME, asw->w, asw->frame,
-                    asw, asw->hints->res_name, asw->hints->names_encoding[asw->hints->res_name_idx]);
+                    asw, asw->hints->res_name, get_hint_name_encoding(asw->hints, asw->hints->res_name_idx));
     }
 }
 
@@ -1063,3 +1087,83 @@ broadcast_config (send_data_type event_type, ASWindow * t)
     SendConfig( -1, event_type, t );
 }
 
+/********************************************************************************/
+/* module list menus regeneration :                                             */
+/********************************************************************************/
+
+static inline void
+module_t2func_data( FunctionCode func, module_t *module, FunctionData *fdata, char *scut ) 
+{
+	fdata->func = func;
+    fdata->name = mystrdup(module->name);
+    fdata->text = mystrdup(module->name);
+	if (++(*scut) == ('9' + 1))
+		(*scut) = 'A';		/* Next shortcut key */
+    fdata->hotkey = (*scut);
+}	   
+
+MenuData *
+make_module_menu( FunctionCode func, const char *title, int sort_order )
+{
+    MenuData *md ;
+    MenuDataItem *mdi ;
+    FunctionData  fdata ;
+    char          scut = '0';                  /* Current short cut key */
+	module_t *modules ;
+	int i, max_i ;
+
+	if ( Modules == NULL ) 
+        return NULL;
+    
+	if( (md = create_menu_data ("@#%module_menu%#@")) == NULL )
+        return NULL;
+
+	modules = MODULES_LIST ;
+	max_i = MODULES_NUM ;
+
+    memset(&fdata, 0x00, sizeof(FunctionData));
+    fdata.func = F_TITLE ;
+    fdata.name = mystrdup(title);
+    add_menu_fdata_item( md, &fdata, NULL, NULL );
+
+	if( sort_order == ASO_Alpha ) 
+	{
+		FunctionData **menuitems = safecalloc(sizeof(FunctionData *), max_i);
+        
+		for( i = 0 ; i < max_i ; ++i) 
+		{
+			menuitems[i] = safecalloc(1, sizeof(FunctionData));
+			module_t2func_data( func, &(modules[i]), menuitems[i], &scut ); 
+        }
+		qsort(menuitems, i, sizeof(FunctionData *), compare_func_data_name);
+		for( i = 0 ; i < max_i ; ++i) 
+		{
+			ASDesktopEntry *de ;
+			char *minipixmap = NULL ; 
+			de = fetch_desktop_entry( AfterStepCategories, menuitems[i]->name );
+			if( de && de->fulliconname ) 
+				minipixmap = de->fulliconname ;
+
+			if( (mdi = add_menu_fdata_item( md, menuitems[i], minipixmap, NULL)) != NULL ) 
+				set_flags( mdi->flags, MD_ScaleMinipixmapDown );
+			safefree( menuitems[i] ); /* scrubba-dub-dub */
+		}
+		safefree(menuitems);
+    }else /* if( sort_order == ASO_Circulation || sort_order == ASO_Stacking ) */
+	{
+		for( i = 0 ; i < max_i ; ++i) 
+        {
+			ASDesktopEntry *de ;
+			char *minipixmap = NULL ; 
+            module_t2func_data( func, &(modules[i]), &fdata, &scut ); 
+			de = fetch_desktop_entry( AfterStepCategories, fdata.name );
+			if( de && de->fulliconname ) 
+				minipixmap = de->fulliconname ;
+            if( (mdi = add_menu_fdata_item( md, &fdata, minipixmap, NULL )) != NULL ) 
+				set_flags( mdi->flags, MD_ScaleMinipixmapDown );
+        }
+    }
+    return md;
+}
+
+
