@@ -1,4 +1,6 @@
 /*
+ * Copyright (C) 2005 Fabian Yamaguchi
+ * Copyright (C) 2004-2005 Sasha Vasko
  * Copyright (C) 1996 Frank Fejes
  * Copyright (C) 1995 Bo Yang
  * Copyright (C) 1993 Robert Nation
@@ -72,7 +74,10 @@ get_free_rectangles_iter_func(void *data, void *aux_data)
     {
         int x, y;
         unsigned int width, height, bw;
-        if( !ASWIN_HFLAGS(asw, AS_AvoidCover) && fr_data->min_layer > ASWIN_LAYER(asw))
+        if( ASWIN_GET_FLAGS(asw, AS_Fullscreen ) )
+			return True;
+
+		if( !ASWIN_HFLAGS(asw, AS_AvoidCover) && fr_data->min_layer > ASWIN_LAYER(asw))
             return True;
 
         if( ASWIN_GET_FLAGS(asw, AS_Iconic ) )
@@ -741,6 +746,10 @@ do_maximized_placement( ASWindow *asw, ASWindowBox *aswbox, ASGeometry *area)
     int selected = -1 ;
     unsigned int w = asw->status->width;
     unsigned int h = asw->status->height;
+#ifdef HAVE_XINERAMA
+    unsigned int x = asw->status->x;
+    unsigned int y = asw->status->y;
+#endif    
     ASVector *free_space_list = NULL;
     XRectangle *rects = NULL;
     int i ;
@@ -755,19 +764,25 @@ do_maximized_placement( ASWindow *asw, ASWindowBox *aswbox, ASGeometry *area)
     while( --i >= 0 )
         if( rects[i].width >= w && rects[i].height >=  h )
         {
-            if( selected > 0 )
+          /* if a rect has been selected */  
+	  if( selected > 0 )
             {
-                if( rects[i].width * rects[i].height < rects[selected].width * rects[selected].height )
+              /* if this rect is smaller than the selected one */  
+	      if( rects[i].width * rects[i].height < rects[selected].width * rects[selected].height )
                     continue;
             }
-            selected = i ;
+          /* select this rectangle because it's bigger */  
+	  selected = i ;
         }
-	if( selected  < 0 ) 
+        
+
+        /* if a rect has NOT been selected*/
+        if( selected  < 0 ) 
 	{	                       /* we simply select the biggest area available : */
 	    i = PVECTOR_USED(free_space_list);
 	    while( --i >= 0 )
-       	{
-           	if( selected > 0 )
+	    {
+		if( selected > 0 )
            	{
                	if( rects[i].width * rects[i].height < rects[selected].width * rects[selected].height )
                    	continue;
@@ -775,13 +790,89 @@ do_maximized_placement( ASWindow *asw, ASWindowBox *aswbox, ASGeometry *area)
             selected = i ;
         }
 	}
+
 		 
     if( selected >= 0 )
     {
-        ASFlagType flags = 0 ;
-		int max_width = rects[selected].width ;
-		int max_height = rects[selected].height ;
-        save_aswindow_anchor( asw, ASWIN_GET_FLAGS( asw, AS_MaximizedX), ASWIN_GET_FLAGS( asw, AS_MaximizedY) );
+	    ASFlagType flags = 0 ;
+	    int max_width = rects[selected].width ;
+	    int max_height = rects[selected].height ;
+
+#ifdef HAVE_XINERAMA
+	    /* the following block makes sure windows
+	      are not maximized over multiply heads.*/
+	
+	    int i;
+	    XRectangle *s = Scr.xinerama_screens;
+	    int dest_size = -1;
+	    int dest_rect =  0;
+	    int inter_width;
+	    int inter_height;
+	    
+	    if( s != NULL)
+	    {
+
+	    /* select the xinerama-screen holding most of the
+	      window as it is before maximizing it */
+	    for(i = 0; i< Scr.xinerama_screens_num; ++i)
+	    {
+		    /* if window is not on this xin-rect at all. */
+		    if( ( x < s[i].x && x > s[i].x + s[i].width )||
+			( y < s[i].y && y > s[i].y + s[i].height) )
+			    continue;
+	    
+		    /* window starts left of screen */
+		    if(s[i].x  > x)
+			    /* and ends on the screen */
+			    if( s[i].x + s[i].width > x + w)
+				    inter_width = x + w - s[i].x;
+			    else
+				    inter_width = s[i].width;
+		    else
+			    if( s[i].x + s[i].width > x + w )
+				    inter_width = s[i].width ;
+			    else
+				    inter_width = s[i].x + s[i].width - x;
+		    
+		    
+		    /* window starts above of screen */
+		    if(s[i].y  > y)
+			    /* and ends on the screen */
+			    if( s[i].y + s[i].height > y + h)
+				    inter_height = y + h - s[i].y;
+			    else
+				    inter_height = s[i].height;
+		    else
+			    if( s[i].y + s[i].height > y + h )
+				    inter_height = s[i].height ;
+			    else
+				    inter_height = s[i].y + s[i].height - y;
+		    
+		    if(inter_width * inter_height > dest_size )
+		    {
+			    /* I like this rect better than the last. */
+			    dest_rect = i;
+			    dest_size = inter_width * inter_height;
+		    }
+			    
+		    
+	    }
+    
+	    
+	    if(rects[selected].x < s[dest_rect].x )
+		    rects[selected].x = s[dest_rect].x;
+	    if(rects[selected].y < s[dest_rect].y )
+		    rects[selected].y = s[dest_rect].y;
+	    if( max_width > s[dest_rect].width)
+		    max_width = s[dest_rect].width;
+	    if( max_height > s[dest_rect].height)
+		    max_height = s[dest_rect].height;
+	    }
+
+#endif /* XINERAMA */
+
+	    
+	    save_aswindow_anchor( asw, ASWIN_GET_FLAGS( asw, AS_MaximizedX), ASWIN_GET_FLAGS( asw, AS_MaximizedY) );
 
         if( ASWIN_GET_FLAGS( asw, AS_MaximizedX)  )
             set_flags( flags, XValue|WidthValue );
@@ -859,8 +950,12 @@ static Bool do_tile_placement( ASWindow *asw, ASWindowBox *aswbox, ASGeometry *a
 
     if( selected >= 0 )
     {
-        int spacer_x = (rects[selected].width > w)? 1: 0;
-        int spacer_y = (rects[selected].height > h)? 1: 0;
+        int spacer_x = aswbox->x_spacing ;
+        int spacer_y = aswbox->y_spacing ;
+		if(rects[selected].width < w+spacer_x)
+			spacer_x = rects[selected].width - w ;
+		if(rects[selected].height < h+spacer_y)
+			spacer_y = rects[selected].height - h ;
         apply_placement_result_asw( asw, XValue|YValue, rects[selected].x+spacer_x, rects[selected].y+spacer_y, 0, 0 );
         LOCAL_DEBUG_OUT( "success: status(%+d%+d), anchor(%+d,%+d)", asw->status->x, asw->status->y, asw->anchor.x, asw->anchor.y );
     }else
@@ -1131,38 +1226,72 @@ Bool place_aswindow( ASWindow *asw )
         }
     }
 
+    /* search for a window-box if none was specified.*/
     if( aswbox == NULL )
     {
-        int i;
-        aswbox = &(Scr.Feel.window_boxes[0]);
+        int i, t;
+	ASWindowBox **aswbox_sorted = safemalloc(sizeof(ASWindowBox *) * Scr.Feel.window_boxes_num);
+	aswbox = &(Scr.Feel.window_boxes[0]);
+	
+	/* the following code will make sure window-boxes which have the
+	   virtual-flag set and are not on the current viewport will
+	   be considered for placement after all others. */
+	t = Scr.Feel.window_boxes_num;
+	for( i = 0; i < Scr.Feel.window_boxes_num; ++i)
+	{
+		if( get_flags(aswbox[i].flags, ASA_Virtual)
+		    && (( asw->status->viewport_x / Scr.MyDisplayWidth
+			  != aswbox[i].area.x / Scr.MyDisplayWidth ) ||
+			( asw->status->viewport_y / Scr.MyDisplayHeight
+			  != aswbox[i].area.y / Scr.MyDisplayHeight) )
+		  )
+			/* place window-box at the end */
+			aswbox_sorted[--t] = &aswbox[i];
+		else
+			/* place window-box at the front */
+			aswbox_sorted[i - Scr.Feel.window_boxes_num + t ] = &aswbox[i];
+	}
+	
         for( i = 0 ; i < Scr.Feel.window_boxes_num ; ++i )
         {
 			LOCAL_DEBUG_OUT("window_box \"%s\": main_strategy = %d, backup_strategy = %d", 
-		   					aswbox[i].name, aswbox[i].main_strategy, aswbox[i].backup_strategy);
+		   					aswbox_sorted[i]->name, aswbox_sorted[i]->main_strategy, aswbox_sorted[i]->backup_strategy);
 			
-            if( IsValidDesk(aswbox[i].desk) && aswbox[i].desk != asw->status->desktop )
+            if( IsValidDesk(aswbox_sorted[i]->desk) && aswbox_sorted[i]->desk != asw->status->desktop )
                 continue;
-            if( aswbox[i].min_layer > asw->status->layer || aswbox[i].max_layer < asw->status->layer )
+            if( aswbox_sorted[i]->min_layer > asw->status->layer || aswbox_sorted[i]->max_layer < asw->status->layer )
                 continue;
-            if( aswbox[i].min_width > asw->status->width || (aswbox[i].max_width > 0 && aswbox[i].max_width < asw->status->width) )
-                continue;
-            if( aswbox[i].min_height > asw->status->height || (aswbox[i].max_height > 0 && aswbox[i].max_height < asw->status->height) )
+            if( aswbox_sorted[i]->min_width > asw->status->width || (aswbox_sorted[i]->max_width > 0 && aswbox_sorted[i]->max_width < asw->status->width) )
                 continue;
+            if( aswbox_sorted[i]->min_height > asw->status->height || (aswbox_sorted[i]->max_height > 0 && aswbox_sorted[i]->max_height < asw->status->height) )
+	        continue;
+
 
 		    if( ASWIN_GET_FLAGS(asw, AS_MaximizedX|AS_MaximizedY ) )
 			{
-	            if( aswbox[i].area.x > asw->status->x+(int)(asw->status->width) || 
-					aswbox[i].area.y > asw->status->y+(int)(asw->status->height)||
-					aswbox[i].area.x+(int)aswbox[i].area.width < asw->status->x ||
-					aswbox[i].area.y+(int)aswbox[i].area.height < asw->status->y )
-    	            continue;
+			  int win_x = get_flags(aswbox_sorted[i]->flags, ASA_Virtual) ? asw->status->viewport_x: asw->status->x;
+			  int win_y = get_flags(aswbox_sorted[i]->flags, ASA_Virtual) ? asw->status->viewport_y: asw->status->y;
+			  
+			  
+			  if( aswbox_sorted[i]->area.x > win_x +(int)(asw->status->width) || 
+					aswbox_sorted[i]->area.y > win_y +(int)(asw->status->height)||
+					aswbox_sorted[i]->area.x+(int)aswbox[i].area.width < win_x ||
+					aswbox_sorted[i]->area.y+(int)aswbox[i].area.height < win_y )
+					continue;
 			}	 
 
 		    if( ASWIN_GET_FLAGS(asw, AS_MaximizedX|AS_MaximizedY ) )
-        		return place_aswindow_in_windowbox( asw, &(aswbox[i]), ASP_UseBackupStrategy, True );
-			else if( place_aswindow_in_windowbox( asw, &(aswbox[i]), ASP_UseMainStrategy , False ))
-                return True;
-        }
+		    {
+			    Bool retval = place_aswindow_in_windowbox( asw, aswbox_sorted[i], ASP_UseBackupStrategy, True );
+			    free( aswbox_sorted );
+			    return retval;
+		    }			
+		    else if( place_aswindow_in_windowbox( asw, aswbox_sorted[i], ASP_UseMainStrategy , False )){
+			    free( aswbox_sorted );
+			    return True;
+		    }        
+	}
+	free( aswbox_sorted );
     }
     return place_aswindow_in_windowbox( asw, Scr.Feel.default_window_box, ASP_UseBackupStrategy, True );
 }
@@ -1437,6 +1566,7 @@ PlaceWindow (ASWindow * tmp_win, unsigned long tflag, int Desk)
 
 		for (i = 0; i < Scr.xinerama_screens_num; ++i)
 		{
+			/* if window is completely in this xinerama-screen */
 			if (s[i].x < x + width && s[i].x + s[i].width > x &&
 				s[i].y < y + height && s[i].y + s[i].height > y)
 			{
