@@ -65,47 +65,51 @@ update_window_transparency( ASWindow *asw, Bool force )
 	int i ;
 	ASCanvas *changed_canvases[6] = {NULL, NULL, NULL, NULL, NULL, NULL };
 
-	for( i = 0 ; i < FRAME_PARTS ; ++i )
-		if( asw->frame_bars[i] )
-		{
-			update_astbar_transparency (asw->frame_bars[i], asw->frame_sides[od->tbar2canvas_xref[i]], force);
-			if( DoesBarNeedsRendering(asw->frame_bars[i]) )
+    if( !ASWIN_GET_FLAGS( asw, AS_Iconic ) )
+	{
+		for( i = 0 ; i < FRAME_PARTS ; ++i )
+			if( asw->frame_bars[i] )
 			{
-				ASCanvas *c = asw->frame_sides[od->tbar2canvas_xref[i]];
-				changed_canvases[od->tbar2canvas_xref[i]] = c ;
-				render_astbar( asw->frame_bars[i], c );
+				update_astbar_transparency (asw->frame_bars[i], asw->frame_sides[od->tbar2canvas_xref[i]], force);
+				if( DoesBarNeedsRendering(asw->frame_bars[i]) )
+				{
+					ASCanvas *c = asw->frame_sides[od->tbar2canvas_xref[i]];
+					changed_canvases[od->tbar2canvas_xref[i]] = c ;
+					render_astbar( asw->frame_bars[i], c );
+				}
+			}
+
+		if( asw->tbar )
+		{
+			update_astbar_transparency (asw->tbar, asw->frame_sides[od->tbar_side], force);
+			if( DoesBarNeedsRendering(asw->tbar) )
+			{
+				changed_canvases[od->tbar_side] = asw->frame_sides[od->tbar_side] ;
+				render_astbar( asw->tbar, asw->frame_sides[od->tbar_side] );
 			}
 		}
-
-	if( asw->tbar )
+	}else
 	{
-		update_astbar_transparency (asw->tbar, asw->frame_sides[od->tbar_side], force);
-		if( DoesBarNeedsRendering(asw->tbar) )
+		if( asw->icon_button )
 		{
-			changed_canvases[od->tbar_side] = asw->frame_sides[od->tbar_side] ;
-			render_astbar( asw->tbar, asw->frame_sides[od->tbar_side] );
-		}
-	}
-
-	if( asw->icon_button )
-	{
-		update_astbar_transparency (asw->icon_button, asw->icon_canvas, force);
-		if( DoesBarNeedsRendering(asw->icon_button) )
-		{
-			changed_canvases[4] = asw->icon_canvas ;
-			render_astbar(asw->icon_button, asw->icon_canvas);
-		}
-	}
-	if( asw->icon_title )
-	{
-		update_astbar_transparency (asw->icon_title, asw->icon_title_canvas?asw->icon_title_canvas:asw->icon_canvas, force);
-		if( DoesBarNeedsRendering( asw->icon_title ) )
-		{
-			if( asw->icon_title_canvas != NULL && asw->icon_title_canvas != asw->icon_canvas )
-				changed_canvases[5] = asw->icon_title_canvas ;
-			else
+			update_astbar_transparency (asw->icon_button, asw->icon_canvas, force);
+			if( DoesBarNeedsRendering(asw->icon_button) )
+			{
 				changed_canvases[4] = asw->icon_canvas ;
-			render_astbar (asw->icon_title, asw->icon_title_canvas?asw->icon_title_canvas:asw->icon_canvas );
+				render_astbar(asw->icon_button, asw->icon_canvas);
+			}
+		}
+		if( asw->icon_title )
+		{
+			update_astbar_transparency (asw->icon_title, asw->icon_title_canvas?asw->icon_title_canvas:asw->icon_canvas, force);
+			if( DoesBarNeedsRendering( asw->icon_title ) )
+			{
+				if( asw->icon_title_canvas != NULL && asw->icon_title_canvas != asw->icon_canvas )
+					changed_canvases[5] = asw->icon_title_canvas ;
+				else
+					changed_canvases[4] = asw->icon_canvas ;
+				render_astbar (asw->icon_title, asw->icon_title_canvas?asw->icon_title_canvas:asw->icon_canvas );
+			}
 		}
 	}
 	for( i = 0 ; i < 6 ; ++i )
@@ -843,7 +847,7 @@ on_window_title_changed( ASWindow *asw, Bool update_display )
     if( asw->tbar )
     {
         ASCanvas *canvas = ASWIN_HFLAGS(asw, AS_VerticalTitle)?asw->frame_sides[FR_W]:asw->frame_sides[FR_N];
-        if( change_astbar_first_label( asw->tbar, asw->hints->names[0], asw->hints->names_encoding[0] ) )
+        if( change_astbar_first_label( asw->tbar, ASWIN_NAME(asw), ASWIN_NAME_ENCODING(asw) ) )
             if( canvas && update_display )
             {
 				invalidate_canvas_save( canvas );
@@ -867,7 +871,7 @@ on_window_title_changed( ASWindow *asw, Bool update_display )
     LOCAL_DEBUG_OUT( "icon_title = %p, icon_name = \"%s\"", asw->icon_title, ASWIN_ICON_NAME(asw)?ASWIN_ICON_NAME(asw):"(null)" );
 	if( asw->icon_title )
     {
-        if( change_astbar_first_label( asw->icon_title, ASWIN_ICON_NAME(asw), (asw->hints->icon_name_idx<0)?AS_Text_ASCII:asw->hints->names_encoding[asw->hints->icon_name_idx] ) )
+        if( change_astbar_first_label( asw->icon_title, ASWIN_ICON_NAME(asw), ASWIN_ICON_NAME_ENC(asw) ) )
             on_icon_changed( asw );
     }
 }
@@ -878,16 +882,24 @@ on_window_hints_changed( ASWindow *asw )
     static ASRawHints raw_hints ;
     ASHints *hints = NULL, *old_hints = NULL;
 	Bool tie_changed = False;
+	ASStatusHints scratch_status ; /* all we need from it is AS_Urgent state change */
+	Bool status_changed = False;
+	
 
     if( AS_ASSERT(asw) )
         return ;
-
+    if( ASWIN_GET_FLAGS( asw, AS_Dead ) )
+		return;
     if( !collect_hints( ASDefaultScr, asw->w, HINT_ANY, &raw_hints ) )
         return ;
     if( is_output_level_under_threshold(OUTPUT_LEVEL_HINTS) )
         print_hints( NULL, NULL, &raw_hints );
-    hints = merge_hints( &raw_hints, Database, NULL, Scr.Look.supported_hints, HINT_ANY, NULL, asw->w );
-    destroy_raw_hints( &raw_hints, True );
+
+	memset( &scratch_status, 0x00, sizeof( scratch_status ) );
+			
+    hints = merge_hints( &raw_hints, Database, &scratch_status, Scr.Look.supported_hints, HINT_ANY, NULL, asw->w );
+    
+	destroy_raw_hints( &raw_hints, True );
     if( hints )
     {
         show_debug( __FILE__, __FUNCTION__, __LINE__,  "Window management hints collected and merged for window %X", asw->w );
@@ -917,18 +929,77 @@ on_window_hints_changed( ASWindow *asw )
     /* we do not want to do a complete refresh of decorations -
 	 * we want to do only what is neccessary: */
 
+	if( get_flags( scratch_status.flags, AS_Urgent ) )
+	{
+		if( !get_flags( asw->status->flags, AS_Urgent ) )
+		{
+			set_flags( asw->status->flags, AS_Urgent );
+			status_changed = True ;				
+		}
+	}else if( get_flags( asw->status->flags, AS_Urgent ) )
+	{
+		ASFlagType extwm_state = 0 ;
+		
+		if( !get_extwm_state_flags (asw->w, &extwm_state) )
+			extwm_state = 0 ;/* just in case */
+		if ( !get_flags (extwm_state, EXTWM_StateDemandsAttention))
+		{
+			clear_flags( asw->status->flags, AS_Urgent );
+			status_changed = True ;				
+		}
+	}
 	if( hints2decorations( asw, old_hints ) )
+		status_changed = True ;
+	
+	if( status_changed) 
 	    on_window_status_changed( asw, False, True );
 
 	if( mystrcmp( old_hints->res_name, hints->res_name ) != 0 || 
 		mystrcmp( old_hints->res_class, hints->res_class ) != 0)
   		broadcast_res_names( asw );
 	if( mystrcmp( old_hints->names[0], hints->names[0] ) != 0 )
+	{	
 	    broadcast_window_name( asw );
+		set_flags( asw->internal_flags, ASWF_NameChanged );
+	}
 	if( mystrcmp( old_hints->icon_name, hints->icon_name ) != 0 )
 	    broadcast_icon_name( asw );
 
-	destroy_hints (old_hints, True);
+	destroy_hints (old_hints, False);
+}
+
+void 
+on_window_opacity_changed( ASWindow *asw )
+{
+    CARD32 new_opacity = NET_WM_WINDOW_OPACITY_OPAQUE ;
+	Bool set = False ;
+	ASDatabaseRecord db_rec ; 
+	
+	if( AS_ASSERT(asw) )
+        return ;
+    if( ASWIN_GET_FLAGS( asw, AS_Dead ) )
+		return;
+	set = read_32bit_property (asw->w, _XA_NET_WM_WINDOW_OPACITY, &new_opacity);
+	if ( set && new_opacity == asw->hints->window_opacity && ASWIN_HFLAGS(asw, AS_WindowOpacity) )
+		return;
+	
+	if( fill_asdb_record (Database, asw->hints->names, &db_rec, False ) )
+	{
+		if( get_flags( db_rec.set_data_flags, STYLE_WINDOW_OPACITY ) )
+		{	
+			new_opacity = set_hints_window_opacity_percent( NULL, db_rec.window_opacity);		   
+			set = True ;
+		}
+	}	 
+
+	if( !set )
+		XDeleteProperty( dpy, asw->frame, _XA_NET_WM_WINDOW_OPACITY	);
+	else if( new_opacity != asw->hints->window_opacity || !ASWIN_HFLAGS(asw, AS_WindowOpacity) )
+	{
+		set_flags( asw->hints->flags, AS_WindowOpacity);
+		asw->hints->window_opacity = new_opacity ;
+		set_32bit_property (asw->frame, _XA_NET_WM_WINDOW_OPACITY, XA_CARDINAL, asw->hints->window_opacity );
+	}	 
 }
 
 int
@@ -1008,6 +1079,8 @@ on_window_status_changed( ASWindow *asw, Bool update_display, Bool reconfigured
     if( AS_ASSERT(asw) )
         return ;
 
+/*	get_extwm_state_flags (asw->w, &i); */
+	
 LOCAL_DEBUG_CALLER_OUT( "(%p,%s Update display,%s Reconfigured)", asw, update_display?"":"Don't", reconfigured?"":"Not" );
 LOCAL_DEBUG_OUT( "status geometry = %dx%d%+d%+d", asw->status->width, asw->status->height, asw->status->x, asw->status->y );
     if( ASWIN_GET_FLAGS(asw, AS_Iconic ) )
@@ -1259,12 +1332,12 @@ LOCAL_DEBUG_CALLER_OUT( "(%p,%s)", asw, context2text(pressed_context));
 void
 save_aswindow_anchor( ASWindow *asw, Bool hor, Bool vert )
 {
-    if( hor && asw->saved_anchor.width == 0 )
+    if( hor && asw->saved_anchor.width == 0 && asw->status->width < Scr.MyDisplayWidth )
     {
         asw->saved_anchor.x = asw->anchor.x ;
         asw->saved_anchor.width = asw->anchor.width ;
     }
-    if( vert && asw->saved_anchor.height == 0 )
+    if( vert && asw->saved_anchor.height == 0 && asw->status->height < Scr.MyDisplayHeight  )
     {
         asw->saved_anchor.y = asw->anchor.y ;
         asw->saved_anchor.height = asw->anchor.height ;
@@ -1274,7 +1347,7 @@ save_aswindow_anchor( ASWindow *asw, Bool hor, Bool vert )
 void
 moveresize_aswindow_wm( ASWindow *asw, int x, int y, unsigned int width, unsigned int height, Bool save_anchor )
 {
-	LOCAL_DEBUG_CALLER_OUT( "asw(%p)->geom(%dx%d%+d%+d)->save_anchor(%d)", asw, x, y, width, height, save_anchor );
+	LOCAL_DEBUG_CALLER_OUT( "asw(%p)->geom(%dx%d%+d%+d)->save_anchor(%d)", asw, width, height, x, y, save_anchor );
     if( !AS_ASSERT(asw) )
     {
         ASStatusHints scratch_status  = *(asw->status);
@@ -1360,84 +1433,26 @@ init_aswindow_status( ASWindow *t, ASStatusHints *status )
 		t->status->viewport_x = Scr.Vx ;
 		t->status->viewport_y = Scr.Vy ;
 	}
-	if (!get_flags (t->status->flags, AS_Sticky))
+	if (!get_flags (t->status->flags, AS_Sticky) )
 	{
-		t->status->x -= t->status->viewport_x;
-		t->status->y -= t->status->viewport_y;
+		Bool absolute_origin = (!ASWIN_HFLAGS(t, AS_UseCurrentViewport));
+		
+		if( absolute_origin && 
+			get_flags( t->hints->flags, AS_Transient ) &&
+			get_flags( t->status->flags, AS_StartPositionUser ))
+		{/* most likely stupid KDE or GNOME app that is abusing USPosition 
+			for no good reason - place it on current viewport */
+			absolute_origin = ( t->status->x >= Scr.MyDisplayWidth || 
+								t->status->y >= Scr.MyDisplayHeight ) ;
+		}
+		if( absolute_origin )
+		{	
+			t->status->x -= t->status->viewport_x;
+			t->status->y -= t->status->viewport_y;
+		}
 	}
 	LOCAL_DEBUG_OUT( "status->pos = %+d%+d, Scr.Vpos = %+d%+d", t->status->x, t->status->y, Scr.Vx, Scr.Vy );
-
-	if( get_flags (t->hints->flags, AS_MinSize) ) 
-	{	
-		int width = t->status->width ;
-		int height = t->status->height ;
-		if( (!get_flags (t->status->flags, AS_StartSizeUser) && width < t->hints->min_width) || width == 1 )
-			width = min( t->hints->min_width, Scr.MyDisplayWidth );
-		if( (!get_flags (t->status->flags, AS_StartSizeUser) && height < t->hints->min_height) || height == 1 )
-			width = min( t->hints->min_height, Scr.MyDisplayHeight );
-		if( width != t->status->width || height != t->status->height ) 
-		{
-			int dx = 0, dy = 0 ;
-			if( t->hints->gravity == EastGravity || 
-				t->hints->gravity == SouthEastGravity || 
-				t->hints->gravity == NorthEastGravity ||
-				t->hints->gravity == CenterGravity )
-				dx = (int)(t->status->width) - width ; 
-			if( t->hints->gravity == SouthGravity || 
-				t->hints->gravity == SouthEastGravity || 
-				t->hints->gravity == SouthWestGravity ||
-				t->hints->gravity == CenterGravity)
-				dy = (int)(t->status->height) - height ; 
-			if( t->hints->gravity == EastGravity || t->hints->gravity == CenterGravity ) 
-				dx = dx/2 ;
-			if( t->hints->gravity == SouthGravity || t->hints->gravity == CenterGravity ) 
-				dy = dy/2 ;
-			t->status->x += dx ;			
-			t->status->y += dy ;			   
-			t->status->width = width ;			   
-			t->status->height = height ;			   
-			XResizeWindow( dpy, t->w, width, height );
-		}	 
-	}
-    if( !get_flags(AfterStepState, ASS_NormalOperation) )
-    {
-        int min_x, min_y, max_x, max_y ;
-        int margin = Scr.MyDisplayWidth>>5 ;
-        if (!ASWIN_GET_FLAGS( t, AS_Sticky ))
-        {
-            min_x = -Scr.Vx ;
-            max_x = Scr.VxMax+Scr.MyDisplayWidth ;
-            min_y = -Scr.Vy ;
-            max_y = Scr.VyMax+Scr.MyDisplayHeight ;
-        }else
-        {
-            min_x = 0 ;
-            max_x = Scr.MyDisplayWidth ;
-            min_y = 0 ;
-            max_y = Scr.MyDisplayHeight ;
-        }
-        /* we have to make sure that window is visible !!!! */
-		LOCAL_DEBUG_OUT( "x_range(%d,%d), y_range(%d,%d), margin = %d", min_x, max_x, min_y, max_y, margin );
-        if( (int)t->status->x + (int)t->status->width < min_x + margin )
-            t->status->x = min_x + margin - (int)t->status->width ;
-        else if( (int)t->status->x > max_x-margin )
-            t->status->x = max_x - margin ;
-        if( (int)t->status->y + (int)t->status->height < min_y + margin )
-            t->status->y = min_y + margin - (int)t->status->height ;
-        else if( (int)t->status->y > max_y-margin )
-            t->status->y = max_y - margin ;
-
-		LOCAL_DEBUG_OUT( "status->pos = %+d%+d, Scr.Vpos = %+d%+d", t->status->x, t->status->y, Scr.Vx, Scr.Vy );
-
-        set_flags( t->status->flags, AS_Position );
-
-    }else if( get_flags(Scr.Feel.flags, NoPPosition))
-	{ 
-      	if( !get_flags( t->hints->flags, AS_Transient ) && 
-		    !get_flags( status->flags, AS_StartPositionUser ) )
-	        clear_flags( t->status->flags, AS_Position );
-	}
-    
+	
 	/* TODO: AS_Iconic */
 	if( !ASWIN_GET_FLAGS(t, AS_StartLayer ) )
         ASWIN_LAYER(t) = AS_LayerNormal ;
@@ -1446,39 +1461,119 @@ init_aswindow_status( ASWindow *t, ASStatusHints *status )
     else if( ASWIN_LAYER(t) > AS_LayerHighest )
         ASWIN_LAYER(t) = AS_LayerHighest ;
 
-    if( get_flags( status->flags, AS_MaximizedX|AS_MaximizedY ))
-        pending_placement = True ;
-    else if( !get_flags( t->status->flags, AS_Position ))
-    {
-        if( ! get_flags( t->status->flags, AS_StartsIconic ) )
-        {
-            int x = -1, y = -1;
-			pending_placement = True ;
-            ASQueryPointerRootXY( &x, &y );
-	        if( get_flags( t->hints->flags, AS_Transient|AS_ShortLived ) )
+	if( get_flags( t->status->flags, AS_Fullscreen ))
+	{
+        t->status->width = Scr.MyDisplayWidth ;
+        t->status->height = Scr.MyDisplayHeight ;
+        t->status->x = 0 ;
+        t->status->y = 0 ;
+	}else
+	{	
+		if( get_flags (t->hints->flags, AS_MinSize) ) 
+		{	
+			int width = t->status->width ;
+			int height = t->status->height ;
+			if( (!get_flags (t->status->flags, AS_StartSizeUser) && width < t->hints->min_width) || width == 1 )
+				width = min( t->hints->min_width, Scr.MyDisplayWidth );
+			if( (!get_flags (t->status->flags, AS_StartSizeUser) && height < t->hints->min_height) || height == 1 )
+				width = min( t->hints->min_height, Scr.MyDisplayHeight );
+			if( width != t->status->width || height != t->status->height ) 
 			{
-				x -= (int)t->status->width / 2 ;
-				y -= (int)t->status->height / 2 ;
-				set_flags( t->status->flags, AS_Position );
-				pending_placement = False ;
-			}
-
-			if( x + (int)t->status->width > (int)Scr.MyDisplayWidth )
-				x = (int)Scr.MyDisplayWidth - (int)t->status->width ;
-			if( x < 0 )
-				x = 0 ;
-
-			if( y + (int)t->status->height > (int) Scr.MyDisplayHeight )
-				y = Scr.MyDisplayHeight - (int)t->status->height ;
-			if( y < 0 ) 
-				y = 0 ;
-
-            t->status->x = x ;
-            t->status->y = y ;
+				int dx = 0, dy = 0 ;
+				if( t->hints->gravity == EastGravity || 
+					t->hints->gravity == SouthEastGravity || 
+					t->hints->gravity == NorthEastGravity ||
+					t->hints->gravity == CenterGravity )
+					dx = (int)(t->status->width) - width ; 
+				if( t->hints->gravity == SouthGravity || 
+					t->hints->gravity == SouthEastGravity || 
+					t->hints->gravity == SouthWestGravity ||
+					t->hints->gravity == CenterGravity)
+					dy = (int)(t->status->height) - height ; 
+				if( t->hints->gravity == EastGravity || t->hints->gravity == CenterGravity ) 
+					dx = dx/2 ;
+				if( t->hints->gravity == SouthGravity || t->hints->gravity == CenterGravity ) 
+					dy = dy/2 ;
+				t->status->x += dx ;			
+				t->status->y += dy ;			   
+				t->status->width = width ;			   
+				t->status->height = height ;			   
+				XResizeWindow( dpy, t->w, width, height );
+			}	 
+		}
+    	if( !get_flags(AfterStepState, ASS_NormalOperation) )
+    	{
+        	int min_x, min_y, max_x, max_y ;
+        	int margin = Scr.MyDisplayWidth>>5 ;
+        	if (!ASWIN_GET_FLAGS( t, AS_Sticky ))
+        	{
+            	min_x = -Scr.Vx ;
+            	max_x = Scr.VxMax+Scr.MyDisplayWidth ;
+            	min_y = -Scr.Vy ;
+            	max_y = Scr.VyMax+Scr.MyDisplayHeight ;
+        	}else
+        	{
+            	min_x = 0 ;
+            	max_x = Scr.MyDisplayWidth ;
+            	min_y = 0 ;
+            	max_y = Scr.MyDisplayHeight ;
+        	}
+        	/* we have to make sure that window is visible !!!! */
+			LOCAL_DEBUG_OUT( "x_range(%d,%d), y_range(%d,%d), margin = %d", min_x, max_x, min_y, max_y, margin );
+        	if( (int)t->status->x + (int)t->status->width < min_x + margin )
+            	t->status->x = min_x + margin - (int)t->status->width ;
+        	else if( (int)t->status->x > max_x-margin )
+            	t->status->x = max_x - margin ;
+        	if( (int)t->status->y + (int)t->status->height < min_y + margin )
+            	t->status->y = min_y + margin - (int)t->status->height ;
+        	else if( (int)t->status->y > max_y-margin )
+            	t->status->y = max_y - margin ;
 
 			LOCAL_DEBUG_OUT( "status->pos = %+d%+d, Scr.Vpos = %+d%+d", t->status->x, t->status->y, Scr.Vx, Scr.Vy );
-        }
-    }
+
+        	set_flags( t->status->flags, AS_Position );
+
+    	}else if( get_flags(Scr.Feel.flags, NoPPosition))
+		{ 
+		
+      		if( !get_flags( t->hints->flags, AS_Transient ) &&
+				!get_flags( t->status->flags, AS_StartPositionUser ))
+	        	clear_flags( t->status->flags, AS_Position );
+		}
+		if( get_flags( status->flags, AS_MaximizedX|AS_MaximizedY ))
+        	pending_placement = True ;
+		else  if( !get_flags( t->status->flags, AS_Position ))
+    	{
+        	if( ! get_flags( t->status->flags, AS_StartsIconic ) )
+        	{
+            	int x = -1, y = -1;
+				pending_placement = True ;
+            	ASQueryPointerRootXY( &x, &y );
+	        	if( get_flags( t->hints->flags, AS_Transient|AS_ShortLived ) )
+				{
+					x -= (int)t->status->width / 2 ;
+					y -= (int)t->status->height / 2 ;
+					set_flags( t->status->flags, AS_Position );
+					pending_placement = False ;
+				}
+
+				if( x + (int)t->status->width > (int)Scr.MyDisplayWidth )
+					x = (int)Scr.MyDisplayWidth - (int)t->status->width ;
+				if( x < 0 )
+					x = 0 ;
+
+				if( y + (int)t->status->height > (int) Scr.MyDisplayHeight )
+					y = Scr.MyDisplayHeight - (int)t->status->height ;
+				if( y < 0 ) 
+					y = 0 ;
+
+            	t->status->x = x ;
+            	t->status->y = y ;
+
+				LOCAL_DEBUG_OUT( "status->pos = %+d%+d, Scr.Vpos = %+d%+d", t->status->x, t->status->y, Scr.Vx, Scr.Vy );
+        	}
+    	}
+	}		
 
     if( is_output_level_under_threshold(OUTPUT_LEVEL_HINTS) )
         print_status_hints( NULL, NULL, t->status );
@@ -1629,9 +1724,9 @@ LOCAL_DEBUG_OUT( "updating status to iconic for client %p(\"%s\")", asw, ASWIN_N
                 }
             }
         }
+	    on_window_status_changed( asw, True, False );
     }
 
-    on_window_status_changed( asw, True, False );
     if( !get_flags(asw->wm_state_transition, ASWT_FROM_WITHDRAWN ) )
         broadcast_config(M_CONFIGURE_WINDOW, asw );
 
@@ -1826,6 +1921,31 @@ void change_aswindow_desktop( ASWindow *asw, int new_desk, Bool force )
     }
 }
 
+static Bool
+restore_anchor_x( ASWindow *asw )
+{
+	if( asw->saved_anchor.width > 0)
+    {
+    	asw->anchor.x = asw->saved_anchor.x ;
+        asw->anchor.width = asw->saved_anchor.width ;
+        asw->saved_anchor.width = 0 ;  /* invalidating saved anchor */
+		return True;
+	}
+	return False;
+}
+static Bool
+restore_anchor_y( ASWindow *asw )
+{
+    if( asw->saved_anchor.height > 0)
+    {
+        asw->anchor.y = asw->saved_anchor.y ;
+        asw->anchor.height = asw->saved_anchor.height ;
+        asw->saved_anchor.height = 0 ; /* invalidating saved anchor */
+        return True ;
+	}
+	return False;	  
+}	  
+
 void toggle_aswindow_status( ASWindow *asw, ASFlagType flags )
 {
     ASFlagType on_flags, off_flags ;
@@ -1834,9 +1954,13 @@ void toggle_aswindow_status( ASWindow *asw, ASFlagType flags )
 
     if( AS_ASSERT(asw) )
         return ;
+
     if( flags == 0 )
         return ;
 
+	if( get_flags( flags, AS_Fullscreen ) )
+		clear_flags( flags, AS_MaximizedX|AS_MaximizedY);
+
     on_flags = (~(asw->status->flags))&flags ;
     off_flags = (asw->status->flags)&(~flags) ;
     asw->status->flags = on_flags|off_flags ;
@@ -1871,32 +1995,56 @@ LOCAL_DEBUG_OUT( "flags = %lx, on_flags = %lx, off_flags = %lx", flags, on_flags
 		}
 	}
 
+	if( get_flags( flags, AS_Fullscreen) )
+	{
+		if( !ASWIN_GET_FLAGS( asw, AS_Fullscreen ) )			   
+		{	                   /* fullscreen->normal */
+			if( !ASWIN_GET_FLAGS( asw, AS_MaximizedX ) )			
+	            if( !restore_anchor_x( asw ) )
+				{
+					asw->anchor.width = Scr.MyDisplayWidth/3;
+					asw->anchor.x = (Scr.MyDisplayWidth - asw->anchor.width) / 2;
+				}
+
+	        if( !ASWIN_GET_FLAGS( asw, AS_MaximizedY ) )
+	            if(!restore_anchor_y( asw ))
+				{
+					asw->anchor.height = Scr.MyDisplayHeight/3;
+					asw->anchor.y = (Scr.MyDisplayHeight - asw->anchor.height) / 2;
+				}
+
+			reconfigured = True;
+			if( ASWIN_GET_FLAGS( asw, AS_MaximizedY|AS_MaximizedX ) )
+				need_placement = True ;	
+		}else
+		{ 	                   /* normal->fullscreen */
+			ASStatusHints scratch_status = *(asw->status) ;
+			scratch_status.x = 0 ;
+			scratch_status.y = 0 ;
+			scratch_status.width = Scr.MyDisplayWidth ;
+			scratch_status.height = Scr.MyDisplayHeight ;
+	 		
+			save_aswindow_anchor( asw, True, True );		   
+			status2anchor(&(asw->anchor), asw->hints, &scratch_status, Scr.VxMax, Scr.VyMax);
+			reconfigured = True ;
+		}
+	}
     if( get_flags( flags, AS_MaximizedX) )
     {
         if( !ASWIN_GET_FLAGS( asw, AS_MaximizedX ) )
-        {
-            if( asw->saved_anchor.width > 0)
-            {
-                asw->anchor.x = asw->saved_anchor.x ;
-                asw->anchor.width = asw->saved_anchor.width ;
-                asw->saved_anchor.width = 0 ;  /* invalidating saved anchor */
-                reconfigured = True ;
-            }
-        }else
+		{	
+            if( restore_anchor_x( asw ) ) 
+				reconfigured = True ;
+        }else if( !ASWIN_GET_FLAGS( asw, AS_Fullscreen ) )
             need_placement = True ;
     }
     if( get_flags( flags, AS_MaximizedY) )
     {
         if( !ASWIN_GET_FLAGS( asw, AS_MaximizedY ) )
         {
-            if( asw->saved_anchor.height > 0)
-            {
-                asw->anchor.y = asw->saved_anchor.y ;
-                asw->anchor.height = asw->saved_anchor.height ;
-                asw->saved_anchor.height = 0 ; /* invalidating saved anchor */
-                reconfigured = True ;
-            }
-        }else
+            if(restore_anchor_y( asw ))
+				reconfigured = True;
+        }else if( !ASWIN_GET_FLAGS( asw, AS_Fullscreen ) )
             need_placement = True ;
     }
 
